#ifndef _NNETZ_H
#define _NNETZ_H

#include <iostream>
#include <fstream>
using namespace std;

typedef struct networkTrainingResults {
	unsigned long int total_epochs, last_epochs, max_epochs;
	double error, trainError;

	// Konstruktor
	networkTrainingResults () {
		this->total_epochs = 0;
		this->error = 0.0;
		this->trainError = 0.0;
		this->last_epochs = 0;
		this->max_epochs = 0;
	}

} NETWORKTRAININGRESULTS;


class Network {

private: 

	float connection_rate;
        float learning_rate_eta;
        unsigned int num_input;
        unsigned int num_output;
        unsigned int num_layers;
        unsigned int num_neurons_hidden;
	unsigned int *hidden_neurons;
        float desired_error;
        unsigned int max_iterations;
        unsigned int iterations_between_reports;
	unsigned int dataCount;
	double activation_stepness_hidden;
	double activation_stepness_output;
	int activation_function_hidden;
	int activation_function_output;
	char dataFilename[30];
	char networkFilename[30];
	FILE* dataFile;
	struct fann *ann;
	double minInputValue;
	double maxInputValue;
	double minOutputValue;
	double maxOutputValue;

	/** Funktion um den Netz-Input in einen bestimmten Wertebereich zu normalisieren */
	double normIn(double _in);
	
	/** Funktion um den Netz-Output in einen bestimmten Wertebereich zu normalisieren */
	double normOut(double _out);
	
	/** Funktion um den normalisierten Netz-Output in den Wertebereich zurueck zu normalisieren */
	double reNormOut(double _out);

public:
	/** Konstruktor */
	Network(int _num_input, int _num_output, int _num_layers, int _num_neurons_hidden, const char* _netName);
	/** Destruktor */
	~Network();

	/** Netz Grenzen setzen um Input sowie Output bestmoeglich zu skalieren */
	void setMinMaxInputOutputValues (double _minInput, double _maxInput, double _minOutput, double _maxOutput);

	/** Funktion fuer Konfiguration der Netzwerk-Parameter */
	void setParameters(int _max_epochs, int _iterations_between_report, double _desired_error, double _dEta, double _activation_stepness_hidden, double _activation_stepness_output, int _activation_function_hidden, int _activation_function_output);

	/** Funktion um den Fehler zu setzen, bis zu welchem maximal gelernt werden soll */
	void setDesiredError(double _desired_error);

	/** Funktion um eine neue Trainingsdatendatei in /tmp zu erzeugen */
	void createNewTrainingDataFile();

	/** Funktion um ein neues Trainingsbeispiel der Trainingsdatendatei hinzuzufuegen */
	void addTrainingData(double _in[], double _out[]);

	/** Funktion um das Netz-Training zu beginnen */
	void trainNet( NETWORKTRAININGRESULTS* _results);

	/** Funktion um das Netz fuer einen gegebenen Input-Vektor in[] zu befragen. Die Antwort wir nach out[] geschrieben.*/
	void getNetValue(double _in[], double _out[]);

	/** Funktion um die Gewichte im Netz zufaellig initialisieren zu lassen */
	void clear();
};

#endif
