/*********************************************************
 * C Implementation: v24
 * =====================
 *
 * Description: 
 *   Author: Michel Tokic,,, <michel@tokic>, (C) 2006
 *
 *********************************************************/

#include "V24.h"
#include "V24_strings.h"

#define SERIAL_BAUDRATE BAUD19200
#define SERIAL_DEVICE "/dev/ttyUSB0"

/**
*
* @param _interface
*/
bool V24Control::setInterface(QString _interface) {
	this->sDeviceInterface = _interface;
	printf ("Serial Interface set  to: %s\n", _interface.ascii());
	if (port->isOpen()) this->closeDevice();
	
	port = new Posix_QextSerialPort(this->sDeviceInterface.ascii());
	
	return this->openDevice();
}

/**
*
* @param _guiMutex
* @param _parent
*/
V24Control::V24Control(QMutex *_guiMutex, QMainWindow *_parent) {

	sDeviceInterface = "/dev/ttyUSB0";

	port = new Posix_QextSerialPort(sDeviceInterface);
	this->guiMutex = _guiMutex;
	this->conMutex = new QMutex();
	this->receiving = this->transmitting = false;
	this->parent = _parent;

	rxtx = true;
}

/**
*
* @param tData
* @param rData
*/
bool V24Control::transmitAndReceive(unsigned char tData, unsigned char rData) {

	int retransmitLoop=0, rxLoop=0, back;
	unsigned char _data;

	retransmitLoop=0;
	rxLoop=0;
	do {
		transmit(tData);

		rxLoop = 0;
		do {
			back = port->readBlock((char*)&_data, 1);
			printf (".");
	
			if (back >= 1) {
				printf ("###RX=%d###", _data);
				if (rData != _data) {
					rxLoop++;
				} else {
					usleep(10000);
					rxLoop = 501;
					retransmitLoop = 11;
				}
			} else {
				fflush(stdout);
				usleep(10000);
				rxLoop++;
			}

		} while (rxLoop < 500);

		printf ("\n");
		retransmitLoop++;
	} while (retransmitLoop < 10 && rData != _data);
	if (retransmitLoop== 10) return false;

	return true;

}

/**
* Diese Funktion sendet ein Byte über die Serielle Schnittstelle
* (low-level). Liefert true zurück, wenn Daten korrekt gesendet
* wurden, "false" wenn nicht.
* @param data
* @return
*/
bool V24Control::transmit (unsigned char  data) {

	// Datenbyte senden
	printf(" TX=%u", data);
	port->writeBlock((char*)&data, 1);
		
	return true;
}

/**
* Diese Funktion empfängt ein Byte von der Seriellen Schnittstelle
* (low-level).
* @param data
* @return
*/
bool V24Control::receive (unsigned char *data) {

	int errorLoop = 0;
	int back;

	do {
		back = port->readBlock((char*)data, 1);
		printf (".");

		if (back >= 1) {
			printf ("#RX=%d#", *data);
			errorLoop = 501;
		} else {
			usleep(10000);
			errorLoop++;
			fflush(stdout);
		}
	} while (errorLoop <500);
	
	if (errorLoop == 500) {
		this->showErrorCode(E_READ_FAILED);
		return false;
	}

	return true;
}

/**
*Diese Funktion sendet ein Kommando über die Seriellen Schnittstelle
* (high-level) 
* Funktion liefert "true" zurück, wenn alles korrekt übertragen wurde.
* Im Fehlerfall => "false"
* @param c
* @return
*/
bool V24Control::transmitCommand (V24COMMAND *c) {

	// Port muss sicher geöffnet sein
	this->openDevice();


	printf("\nBegin transmitCommand():\n");

	/**************************
		Synchronisierung
	***************************/
	if (!this->transmitAndReceive(V24_COMMAND_SYNC_COMMAND, V24_COMMAND_SYNC_OK_COMMAND)) return false;
	if (!this->transmitAndReceive(c->command, c->command));
	if (!this->transmitAndReceive(c->dataLength, c->dataLength));

 	for (int i=0; i < c->dataLength; i++) {
 
 		if (!this->transmitAndReceive(c->data[i], c->data[i])) return false;
 		printf ("\n");
 	}
	
	printf("End transmitCommand():\n");

	return true;
}

/**
* Diese Funktion empfängt ein Kommando von der Seriellen Schnittstelle
* (high-level).
* @param c
* @return
*/
bool V24Control::receiveCommand (V24COMMAND *c) {
	// Port muss sicher geöffnet sein
	this->openDevice();

	printf("\nBegin receiveCommand():\n");
	
	// Komamndo empfangen
	if (!receive(&c->command)) {
		return false;
	}
	if (rxtx) {
	}
	printf ("\n");

	// Data-Length empfangen	
	if (!receive(&c->dataLength)) {
		return false;
	}
	if (rxtx) {
	}
	printf ("\n");

	for (int i=0; i< c->dataLength; i++) {

		if (!receive(&c->data[i])) {
			return false;
		}	
		if (rxtx) {
		}
		printf ("\n");
	}	
	printf("End receiveCommand():\n");


	return true;
}

/**
* Diese Funktion schließt die Verbindung zum V24 Gerät
*/
void V24Control::closeDevice() {
	port->close();
}

/**
* Diese Funktion öffnet die Verbindung zum V24 Gerät
*/
bool V24Control::openDevice() {

	// Wenn Port noch nicht geöffnet, dann öffnen 
	if (!port->isOpen()) {

		// 8N1, 9600 Baud
		if (!port->open()) {
			DBG();
			QMessageBox::critical(this->parent, "Serial Communcation ERROR:", "Could not open Interface: " + sDeviceInterface);
			
			return false;
		} else {
		
			port->setFlowControl(FLOW_OFF);
			port->setParity(PAR_NONE);
			port->setDataBits(DATA_8);
			port->setStopBits(STOP_1);
			port->setBaudRate(SERIAL_BAUDRATE);
			port->setTimeout(1,0); // Timeout auf 10 Sekunden setzen
		}
	}
	
	return true;
}


/**
* Diese Funktionen, parst und interpretiert das Data-Array als Float-Zahl
* und liefert diese dann zurück
* @param c
* @param _data
*/
void V24Control::parseFloatData(unsigned char *c, float *_data) {
	
	unsigned char* _cpfValue = (unsigned char*)_data;

	for (int i=0; i<(int)sizeof(float); i++) {
		_cpfValue[i] = c[i];
	}
}


/**
* Diese Funktionen, parst und interpretiert das Data-Array als Integer-Zahl
* und liefert diese dann zurück
* @param c
* @param _data
*/
void V24Control::parseIntegerData(unsigned char* c, unsigned int *_data) {

	unsigned char* _cpiValue = (unsigned char*)_data;

	for (int i=0; i<(int)sizeof(unsigned int); i++) {
		*(_cpiValue+i) = c[i];
	}
}

/**
* Diese Funktion, nimmt eine Empfangsbestätigung vom Controller zurück.
* Wenn das Kommando in Ordnung war, dann liefert sie "true" zurück,
* ansonsten "false".
* @return
*/
bool V24Control::receiveTransmitOKCommand() {
	return true;
}


/**
*
* @param _errorCode
*/
void V24Control::showErrorCode(unsigned long _errorCode) {

	// Bei keinem Fehler, abbrechen
	if (_errorCode == E_NO_ERROR) {
		return;	
	}

	// Ansonsten den Fehlercode ausgeben		
	if (_errorCode == E_INVALID_FD) {
		QMessageBox::critical(this->parent, "Serial Communcation ERROR:", "Invalid file descriptor (port was not opened correctly)");
		printf ("Invalid file descriptor (port was not opened correctly)\n");
	} else if (_errorCode == E_NO_MEMORY) {
		QMessageBox::critical(this->parent, "Serial Communcation ERROR:", "Unable to allocate memory tables");
		printf ("Unable to allocate memory tables\n");
	} else if (_errorCode == E_CAUGHT_NON_BLOCKED_SIGNAL) {
		QMessageBox::critical(this->parent, "Serial Communcation ERROR:", "Caught a non-blocked signal");
		printf ("Caught a non-blocked signal\n");
	} else if (_errorCode == E_PORT_TIMEOUT) {
		QMessageBox::critical(this->parent, "Serial Communcation ERROR:", "Operation timed out");
		printf ("Operation timed out\n");
	} else if (_errorCode == E_INVALID_DEVICE) {
		QMessageBox::critical(this->parent, "Serial Communcation ERROR:", "The file opened by the port is not a character device");
		printf ("The file opened by the port is not a character device\n");
	} else if (_errorCode == E_BREAK_CONDITION) {
		QMessageBox::critical(this->parent, "Serial Communcation ERROR:", "The port detected a break condition");
		printf ("The port detected a break condition\n");
	} else if (_errorCode == E_FRAMING_ERROR) {
		QMessageBox::critical(this->parent, "Serial Communcation ERROR:", "The port detected a framing error");
		printf ("The port detected a framing error\n");
	} else if (_errorCode == E_IO_ERROR) {
		QMessageBox::critical(this->parent, "Serial Communcation ERROR:", "There was an I/O error while communicating with the port");
		printf ("There was an I/O error while communicating with the port\n");
	} else if (_errorCode == E_BUFFER_OVERRUN) {
		QMessageBox::critical(this->parent, "Serial Communcation ERROR:", "Character buffer overrun");
		printf ("Character buffer overrun\n");
	} else if (_errorCode == E_RECEIVE_OVERFLOW) {
		QMessageBox::critical(this->parent, "Serial Communcation ERROR:", "Receive buffer overflow");
		printf ("Receive buffer overflow\n");
	} else if (_errorCode == E_RECEIVE_PARITY_ERROR) {
		QMessageBox::critical(this->parent, "Serial Communcation ERROR:", "The port detected a parity error in the received data");
		printf ("The port detected a parity error in the received data\n");
	} else if (_errorCode == E_TRANSMIT_OVERFLOW) {
		QMessageBox::critical(this->parent, "Serial Communcation ERROR:", "Transmit buffer overflow");
		printf ("Transmit buffer overflow\n");
	} else if (_errorCode == E_READ_FAILED) {
		QMessageBox::critical(this->parent, "Serial Communcation ERROR:", "General read operation failure. Please check robot connection.");
		printf ("General read operation failure. Please check robot connection.\n");
	} else if (_errorCode == E_WRITE_FAILED) {
		QMessageBox::critical(this->parent, "Serial Communcation ERROR:", "General write operation failure");
		printf ("General write operation failure\n");
	}
}



