#ifndef FUNCTIONS_H
#define FUNCTIONS_H

#include "SimulationMainWindow.h"
#include "SimulationDialog.h"
#include "valueIteration.h"
#include "qLearning.h"
#include "gui_canvas.h"
#include "V24.h"
#include "NNetwork.h"


#define XMAX	100
#define YMAX	100
#define UP	00
#define DOWN	01
#define LEFT   02
#define RIGHT	03
#define NOTHING	04

#include <qapplication.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qslider.h>
#include <qpushbutton.h>
#include <qtable.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qstring.h>
#include <qsignal.h>
#include <qmutex.h>
#include <qspinbox.h>
#include <qtabwidget.h>
#include <qpoint.h>
#include <qfiledialog.h>
#include <qfile.h>
#include <qdatastream.h>
#include <qmessagebox.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qtimer.h>
#include <qcombobox.h>

#include "gui_debug.h"
#include "V24.h"
#include "qWorld.h"

//class Simulation : public QMainWindow, public QObject
/**
*
*/
class Simulation : public QMainWindow
{
        Q_OBJECT

    public:
        void initDialog();
        Simulation(QApplication *_mainApp);
	~Simulation();


	SimulationMainWindow *lg;
	ValueIterationThread *pkt;
	QTimer *st;
	V24Control *v24;
	SimulationDialog *sd;

	char ki_in;
	float gamma;
	float alpha;
	float epsilon;
	float expParameter;
	uchar expMode;
	uchar scalX, scalY;

    private:
	int i_update;
	int speed;
	QApplication *dad;
	QPixmap pix[6];
	QWidget *_widget;
	QMutex *tableMutex;
	QMutex *debugMutex;
	QMutex *guiMutex;
	bool guiReady;
	int i_cell;
	int iMantisse;
	double feedbackMemory[YMAX][XMAX][5];
	double qValueMemory[YMAX][XMAX][4];
	float fValueMemory[YMAX][XMAX];
	int action[YMAX][XMAX];
	QPoint simuPos, simuPosNew;
	QDialog *simuDialog;
	
	void setReward(int x, int y, int action, double reward);
	double getReward(int x, int y, int action);

    private slots:
	void apply_button_clicked();
	void learn_button_clicked();
	void delete_button_clicked();
	void speed_slider_changed(int _value);
	void vi_train_random_values_changed (int _value);
	void q_train_random_values_changed (int _value);
	void iteration_button_clicked();
	void stop_button_clicked();
	void updatePolicyTable(int _row, int _column, double _value, int _bestdirection);
	void updateFeedbackTable(int _row, int _column, double _up, double _down, double _left, double _right);
	void debugOutput (QString _d);
	void feedbackClicked(int row, int col, int button, const QPoint & mousePos );
	void save_button_clicked();
	void save_matlab_button_clicked();
	void load_button_clicked();
	void save_feedback_clicked();
	void start_simulation_clicked();
	void stop_simulation_clicked();
	void resume_simulation_clicked();
	void simulationWalk();
	void stopLearningThread();
	void adjust_network_error(double _newError);

	// Learning Methods
	void learn_method_vi_clicked();
	void learn_method_nn_clicked();
	void q_learn_method_vi_clicked();
	void q_learn_method_nn_clicked();
	void learn_method_vi_nn_clicked();
	

	// V24-Communication
	void v24_start_walking_button_clicked();
	void v24_stop_walking_button_clicked();
	void v24_start_learning_button_clicked();
	void v24_copy_states_and_gamma_to_robot_button_clicked();
	void v24_get_states_and_gamma_from_robot_button_clicked();
	void v24_getPositionAndMode_button_clicked();
	void v24_ramtest_button_clicked();
	void v24_explore_feedback_button_clicked();
	void v24_accept_button_clicked();
	void v24_distance_button_clicked();
	void v24_reset_distance_button_clicked();
	void v24_exploration_accept_clicked();
	void v24_get_exploration_clicked();
	void v24_exploration_mode_changed();
	void v24_debug_button_clicked();
};

#endif
