#include "gui_canvas.h"

/**
* This function initialize the Canvas and set all Object to the basic position 0,0
*/
void MyCanvasView::initCanvas() {
	this->setCanvas (canvas);

	// set covered distance to 0
	this->distance = 0;

	// draw Bottom first
	this->paintBottom();

	// load Pictures
	this->frame_pix = new QCanvasPixmapArray (FRAMEPIX, 0);
	if(!this->frame_pix->isValid()){
		cout << "Could not load pix: " << FRAMEPIX << endl;
	}
	this->base_pix = new QCanvasPixmapArray (BASEPIX, 0);
	if(!this->base_pix->isValid()){
		cout << "Could not load pix: " << BASEPIX << endl;
	}
	this->holder_pix = new QCanvasPixmapArray (HOLDERPIX, 0);
	if(!this->holder_pix->isValid()){
		cout << "Could not load pix: " << HOLDERPIX << endl;
	}

	// create CanvasSprite
	this->frame = new QCanvasSprite (this->frame_pix, canvas);
	this->base = new QCanvasSprite (this->base_pix, canvas);
	this->holder = new QCanvasSprite (this->holder_pix, canvas);

	// create Text-LAbel
	this->distanceText = new QCanvasText ("distance travelled: " + QString::number(this->distance), canvas);
	this->distanceText->move(10, bottom_start_y + 10);
	this->posText = new QCanvasText ("robot state: {x,y} = {2,2}", canvas);
	this->posText->move(10, bottom_start_y + 30);


	// at least set to position 0,0
	this->setDimension (5, 5, 3, 3);
	this->moveRobot(3, 3);

	// add remaining elements
	this->holder->show();
	this->distanceText->show();
	this->posText->show();
	this->base->show();
	this->frame->show();
	
	
	// draw Canvas
	this->canvas->update();

}

/**
* This function pitch the Objects(Frame, stand, holder) to a position
* @param _row
* @param _col
*/
void MyCanvasView::moveRobot(int _row, int _col) {
	
	QPoint rpos, spos, hpos;
	float pixX, pixY;

	_row--;
	_col--;

	// Calculate pixel for one Step on the x and y axis
	pixX = stand_inner_width / (float)iCols;
	pixY = base_inner_height / (float)iRows;
	
	/**********************************************
	// set frame and base position:
	// =====================
	//   - if _row < 1/2*iRows, 
        //     frame move up 
        //   - if _row >= 1/2*iRows, then
	//     base move up
	**********************************************/
	// frame X-Position
	rpos.setX((int)(_col * pixX));
	// base X-Position
	spos.setX((int)(stand_beam_width + rpos.x() + (pixX * (float)(iCols - _col))));
	
	// frame up and base on the bottom
	if (_row <= (0.5 * (float)(iRows))) {

		// base Y-Position
		spos.setY((int)(bottom_start_y - base_total_height));

		// frame Y-Position		
		rpos.setY((int)(spos.y() + (float)(_row * pixY)));
	
	// base up and frame on the bottom
	} else {

		// frame Y-Position
		rpos.setY((int)(bottom_start_y - stand_total_height));
		
		// base Y-Position
		spos.setY((int)(rpos.y() - (float)(_row * pixY)) +1);	

		// the base can get a new x-position if its up. Therefore the textlabel and the distance must be updated
		if (_col != this->oldPos.x()) {
			distance += this->oldPos.x() - _col;
			this->distanceText->setText("distance travelled: " + QString::number(this->distance));
		}
	}

	// Position of the red holder
	hpos.setX((int)(  spos.x() + (base_total_width/2) - (holder_total_width/2) )); 
	hpos.setY((int)(  rpos.y() - (holder_total_height/2) + (stand_beam_height/2) )); 

	// calculate new stand-Position
	this->frame->move(rpos.x(), rpos.y());
	this->base->move(spos.x(), spos.y());
	this->holder->move(hpos.x(), hpos.y());
	// update Position-Label 
	this->posText->setText("robot state: {x,y} = {" +
				QString::number(_col+1) + "," + QString::number(_row+1) + "}");

	this->canvas->update();

	// save position
	this->oldPos.setX(_col);
	this->oldPos.setY(_row);
}


/**
* This function draw the bottom
*/
void MyCanvasView::paintBottom() {

	this->bottom = new QCanvasRectangle (0, bottom_start_y, canvas_width, canvas_height, this->canvas);
	this->bottom->setBrush( QColor(200,200,200) );
	this->bottom->show();
}


/**
* Diese Funktion, setzt die Anzahl der maximalen Zeilen und Spalten,
* damit die Zwischenzustände bestimmt werden können.
* @param _rows
* @param _cols
* @param _initRow
* @param _initCol
*/
void MyCanvasView::setDimension(int _rows, int _cols, int _initRow, int _initCol) {

	_rows--;
	_cols--;
	_initRow--;
	_initCol--;

	this->iRows = _rows;
	this->iCols = _cols;

	// Weg mit 0 initialisieren
	this->distance = 0;
	this->distanceText->setText("distance travelled: " + QString::number(this->distance));

	// Initialposition einnehmen
	this->moveRobot(_initRow, _initCol);

	// Diese initial-Position als aktuelle Position speichern
	this->oldPos.setX(_initCol);
	this->oldPos.setY(_initRow);
}

