#include "gui.h"
//#include "valueIteration.h"
//#include "gui_simu_simuthread.h"


/**
* Funktion setzt das Feedback für einen Zustand entweder im RAM oder im neuronalen Netz
* @param _row
* @param _colum
* @param _action
* @param _reward
**/
void Simulation::setReward (int _row, int _column, int _action, double _reward) {

	this->feedbackMemory[_row][_column][_action] = _reward;
}
/**
*
* @param _row
* @param _column
* @param _action
* @return double
*/
double Simulation::getReward (int _row, int _column, int _action) {
	return this->feedbackMemory[_row][_column][_action];
}

/**
* Funktion zum updaten der Feedback-Tabelle
* @param _row
* @param _column
* @param _up
* @param _down
* @param _left
* @param _right
**/
void Simulation::updateFeedbackTable(int _row, int _column, double _up, double _down, double _left, double _right) {

	QTableItem *feedback_item;
	static int updateCall = 0;

	this->dad->lock();


	// Feedback-Memory aktualisieren
 	setReward(_row, _column, UP, _up);
 	setReward(_row, _column, DOWN, _down);
 	setReward(_row, _column, LEFT, _left);
 	setReward(_row, _column, RIGHT, _right);

	// Feedback-Tabellen Item
	feedback_item = lg->envRewardTable->item (_row, _column);
	feedback_item->setText ("   " + QString::number ((float)getReward(_row, _column, UP)) + "\n" + 
					QString::number ((float)getReward(_row, _column, LEFT)) + 
				"    " + QString::number ((float)getReward(_row, _column, RIGHT)) + "\n" +
				"   " + QString::number ((float)getReward(_row, _column, DOWN)));

	updateCall++;
	if (updateCall == i_cell ) {
		for (int i=0; i < lg->numRows->text().toInt(); i++) {
			lg->envRewardTable->adjustRow (i);
		}
		for (int i=0; i < lg->numCols->text().toInt(); i++) {
			lg->envRewardTable->adjustColumn (i);
		}
		updateCall = 0;
	}

	this->pkt->unlockMutex();
	
	// GUI-Mutex wieder freigeben
	this->dad->unlock();
}


/**
* Funktion wird aufgerufen, wenn auf eine Zelle in der Feedback-Tabelle geklickt wurde
* @param row
* @param col
**/
void Simulation::feedbackClicked(int row, int col, int _button, const QPoint &_mousePos ) {

	this->lg->envUpRewardInput->setValue( (int)getReward(row, col, UP) );
	this->lg->envDownRewardInput->setValue( (int)getReward(row, col, DOWN) );
	this->lg->envLeftRewardInput->setValue( (int)getReward(row, col, LEFT) );
	this->lg->envRightRewardInput->setValue( (int)getReward(row, col, RIGHT) );

	this->lg->envRowLabel->setText(QString::number(row+1));
	this->lg->envColLabel->setText(QString::number(col+1));
}



/**
* Funktion wird aufgerufen, wenn auf "save feedback" geklickt wurde
*/
void Simulation::save_feedback_clicked() {
	int _row, _col;
	
	int _up = this->lg->envUpRewardInput->value();
	int _down = this->lg->envDownRewardInput->value();
	int _left = this->lg->envLeftRewardInput->value();
	int _right = this->lg->envRightRewardInput->value();


	// Alle Zellen checken, ob sie jeweils selektiert sind
	for (int i=0; i < this->lg->envRewardTable->numRows(); i++) {
		for (int j=0; j < this->lg->envRewardTable->numCols(); j++) {
			if (this->lg->envRewardTable->isSelected(i,j)) {

				_row = i;
				_col = j;

				// Feedback-Memory aktualisieren
				this->feedbackMemory[_row][_col][UP] = _up;
				this->feedbackMemory[_row][_col][DOWN] = _down;
				this->feedbackMemory[_row][_col][LEFT] = _left;
				this->feedbackMemory[_row][_col][RIGHT] = _right;
	
				updateFeedbackTable ( _row, _col, _up, _down, _left, _right);

				this->pkt->setFeedback(	_row, _col, _up, _down,	_left, _right);
			}
		}
	}

	// Breite und H�he der Zellen anpassen
	for (int i=0; i < this->lg->envRewardTable->numRows(); i++) {
		lg->envRewardTable->adjustRow (i);
	}
	
	for (int j=0; j < this->lg->envRewardTable->numCols(); j++) {
		lg->envRewardTable->adjustColumn (j);
	}
}

/** 
* Funktion wird aufgerufen, wenn auf "save to file" geklickt wurde 
*/
void Simulation::save_button_clicked() {

    	QString s = QFileDialog::getSaveFileName(
                    ".",
                    "Feedback (*.feedback);;All files(*.*)",
                    this,
                    "open file dialog",
                    "Choose a file" );

	this->lg->debugLabel->setText("save button: " + s);

	QFile qf(s);
	if ( qf.open(IO_WriteOnly) ) {

		QDataStream stream( &qf ); // we will serialize the data into the file

		//Header schreiben mit Anzahl von Zeilen und Spalten
		stream 	<< (Q_INT32)this->lg->numRows->text().toInt()
			<< (Q_INT32)this->lg->numCols->text().toInt();
		
		for (int i=0; i < this->lg->numRows->text().toInt(); i++) {
			for (int j=0; j < this->lg->numCols->text().toInt(); j++) {
				stream 	<< (Q_INT32)getReward(i, j, UP)
					<< (Q_INT32)getReward(i, j, DOWN)
					<< (Q_INT32)getReward(i, j, LEFT)
					<< (Q_INT32)getReward(i, j, RIGHT);
			}
		}
		qf.close();
	} else {
		QMessageBox::critical( 	this, "Laufroboter Dialog", "ERROR: Could not write to file: " + s);
	}
	
}

/** 
* Funktion wird aufgerufen, wenn auf "save to file" geklickt wurde 
*/
void Simulation::save_matlab_button_clicked() {

	char zeile[256];

    	QString s = QFileDialog::getSaveFileName(
                    ".",
                    "Matlab/Octave (*.m);;All files(*.*)",
                    this,
                    "open file dialog",
                    "Choose a file" );

	this->lg->debugLabel->setText("save matlab button: " + s);

	QFile qf(s);
	if ( qf.open(IO_WriteOnly) ) {

		//Header schreiben mit Anzahl von Zeilen und Spalten
		sprintf (zeile, "envRows=%d;\nenvCols=%d;\n", this->lg->numRows->text().toInt(), this->lg->numCols->text().toInt());
		qf.writeBlock(zeile, strlen(zeile));

		for (int i=0; i < this->lg->numRows->text().toInt(); i++) {
			for (int j=0; j < this->lg->numCols->text().toInt(); j++) {
				sprintf (zeile, "env{%d} = [%f, %f, %f, %f];\n", 
					((i*this->lg->numCols->text().toInt()) + j + 1),
					getReward(i, j, UP),
					getReward(i, j, DOWN),
					getReward(i, j, LEFT),
					getReward(i, j, RIGHT));
				qf.writeBlock(zeile, strlen(zeile));
			}
		}
		qf.close();
	} else {
		QMessageBox::critical( 	this, "Laufroboter Dialog", "ERROR: Could not write to file: " + s);
	}
	
}


/** 
* Funktion wird aufgerufen, wenn auf "load from file" geklickt wurde 
*/
void Simulation::load_button_clicked() {

    	QString s = QFileDialog::getOpenFileName(
                    ".",
                    "Feedback (*.feedback);;All files(*.*)",
                    this,
                    "open file dialog",
                    "Choose a file" );

	this->lg->debugLabel->setText("load button: " + s);

	QFile qf(s);
	Q_INT32 _up, _down, _left, _right, _rows, _cols;

	if ( qf.open(IO_ReadOnly) ) {
		QDataStream stream( &qf ); // we will serialize the data into the file

		//Header lesen mit Anzahl von Zeilen und Spalten
		stream >> _rows >> _cols;

		if (_rows != this->lg->numRows->text().toInt() ||
	 	    _cols != this->lg->numCols->text().toInt()) {
			QMessageBox::critical( 	this, "Laufroboter Dialog", 
						"Your " + this->lg->numRows->text() + "x" +
						this->lg->numCols->text() + " matrix, doesn't fit with a " +
						QString::number(_rows) + "x" + QString::number(_cols) + 
						" matrix, \nwhich is hold inside your selected file. \n\n" + s);
		}  else {

			for (int row=0; row < this->lg->numRows->text().toInt(); row++) {
				for (int col=0; col < this->lg->numCols->text().toInt(); col++) {
					stream >> _up >> _down >> _left >> _right;
					this->feedbackMemory[row][col][UP] = _up;
					this->feedbackMemory[row][col][DOWN] = _down;
					this->feedbackMemory[row][col][LEFT] = _left;
					this->feedbackMemory[row][col][RIGHT] = _right;
					updateFeedbackTable(row, col, _up, _down, _left, _right);
					this->pkt->setFeedback(row, col, _up, _down, _left, _right);
				}
			}
		}
		qf.close();
	} else {
		this->lg->debugLabel->setText("ERROR: Could not read from file: " + s);
	}
}


