#include "gui.h"

/**
* Destructor
*/
Simulation::~Simulation() {
	printf("Destruktor!1\n");
	QWorld::destroyInstance();
	delete (this->pkt);
}
/**
* Konstructor
* @param *_mainApp
*/
Simulation::Simulation(QApplication *_mainApp) {

	dad = _mainApp;
	
	this->lg = new SimulationMainWindow (this);
	this->sd = new SimulationDialog();

	this->lg->viSpeedSlider->setEnabled(false);
	this->lg->ViTab->setEnabled(false);
	this->lg->QTab->setEnabled(false);
	this->lg->LinkTabs->setEnabled(false);

	// Default-Werte initialisieren
	this->lg->viValueTable->setNumRows(0);
	this->lg->viValueTable->setNumCols(0);
	this->lg->strategyTable->setNumRows(0);
	this->lg->strategyTable->setNumCols(0);

	this->lg->viSpeedSlider->setEnabled(false);
	this->lg->viGammaInput->setText("0.9");
	this->lg->debugLabel->setEnabled(true);
	this->lg->numRows->setMaxValue(YMAX);
	this->lg->numCols->setMaxValue(XMAX);
	this->lg->viLearnWithNN->setEnabled(false);
	this->lg->viLearnWithVI->setEnabled(false);
	this->lg->viLearnWithNN->setChecked(false);
	this->lg->viLearnWithVI->setChecked(false);

	this->pkt = NULL;

	this->tableMutex = new QMutex;
	this->debugMutex = new QMutex;
	this->guiMutex = new QMutex;
	this->guiMutex->unlock();

	DBG();

	lg->show();
	this->i_update=0;
	this->iMantisse = 0;

	// Timer für die Simulation initialisieren
	this->st = new QTimer();
	connect(this->st, SIGNAL(timeout()), this, SLOT(simulationWalk()));
	this->simuDialog = NULL;
	this->simuPos.setX(0);
	this->simuPos.setY(0);


        connect(lg->applyButton, SIGNAL(clicked()), this, SLOT(apply_button_clicked()));
        connect(lg->applyButtonVI, SIGNAL(clicked()), this, SLOT(apply_button_clicked()));
	connect(lg->applyButtonQ, SIGNAL(clicked()), this, SLOT(apply_button_clicked()));

        connect(lg->viLernenButton, SIGNAL(clicked()), this, SLOT(learn_button_clicked()));
        connect(lg->viDeleteButton, SIGNAL(clicked()), this, SLOT(delete_button_clicked()));
	connect(lg->viStopButton, SIGNAL(clicked()), this, SLOT(stop_button_clicked()));
	connect(lg->viIterationButton, SIGNAL(clicked()), this, SLOT(iteration_button_clicked()));
        connect(lg->viLernenButton2, SIGNAL(clicked()), this, SLOT(learn_button_clicked()));
        connect(lg->viDeleteButton2, SIGNAL(clicked()), this, SLOT(delete_button_clicked()));
	connect(lg->viStopButton2, SIGNAL(clicked()), this, SLOT(stop_button_clicked()));
	connect(lg->viIterationButton2, SIGNAL(clicked()), this, SLOT(iteration_button_clicked()));


	connect(lg->envLoadFileButton, SIGNAL(clicked()), this, SLOT(load_button_clicked()));
	connect(lg->envSaveFileButton, SIGNAL(clicked()), this, SLOT(save_button_clicked()));
	connect(lg->envSaveMatlabFileButton, SIGNAL(clicked()), this, SLOT(save_matlab_button_clicked()));
	connect(lg->envSaveRewardButton, SIGNAL(clicked()), this, SLOT(save_feedback_clicked()));
	connect(lg->viLearnWithNN, SIGNAL(clicked()), this, SLOT(learn_method_nn_clicked()));
	connect(lg->viLearnWithVI, SIGNAL(clicked()), this, SLOT(learn_method_vi_clicked()));
	connect(lg->qLearnWithNN, SIGNAL(clicked()), this, SLOT(q_learn_method_nn_clicked()));
	connect(lg->qLearnWithVI, SIGNAL(clicked()), this, SLOT(q_learn_method_vi_clicked()));

	// V24 Test
	connect(lg->v24StartWalkingButton, SIGNAL(clicked()), this, SLOT(v24_start_walking_button_clicked()));
	connect(lg->v24StopWalkingButton, SIGNAL(clicked()), this, SLOT(v24_stop_walking_button_clicked()));
	connect(lg->v24StartLearningButton, SIGNAL(clicked()), this, SLOT(v24_start_learning_button_clicked()));
	connect(lg->v24CopyStatesAndGammaToRobotButton, SIGNAL(clicked()), this, SLOT(v24_copy_states_and_gamma_to_robot_button_clicked()));
	connect(lg->v24GetStatesAndGammaFromRobotButton, SIGNAL(clicked()), this, SLOT(v24_get_states_and_gamma_from_robot_button_clicked()));
	connect(lg->v24GetPositionAndModeButton, SIGNAL(clicked()), this, SLOT(v24_getPositionAndMode_button_clicked()));
	connect(lg->v24RamtestButton, SIGNAL(clicked()), this, SLOT(v24_ramtest_button_clicked()));
	connect(lg->v24ExploreFeedbackButton, SIGNAL(clicked()), this, SLOT(v24_explore_feedback_button_clicked()));
	v24 = new V24Control(this->guiMutex, this);
	connect(lg->v24AcceptButton, SIGNAL(clicked()), this, SLOT(v24_accept_button_clicked()));
	connect(lg->v24GetDistanceButton, SIGNAL(clicked()), this, SLOT(v24_distance_button_clicked()));
	connect(lg->v24ResetDistanceButton, SIGNAL(clicked()), this, SLOT(v24_reset_distance_button_clicked()));
	connect(lg->v24ExplorationAcceptButton, SIGNAL(clicked()), this, SLOT(v24_exploration_accept_clicked()));
	connect(lg->v24GetExplorationButton, SIGNAL(clicked()), this, SLOT(v24_get_exploration_clicked()));
	connect(lg->v24ExplorationModeEGreedy, SIGNAL(clicked()), this, SLOT(v24_exploration_mode_changed()));
	connect(lg->v24ExplorationModeValueBased, SIGNAL(clicked()), this, SLOT(v24_exploration_mode_changed()));
	connect(lg->v24DebugButton, SIGNAL(clicked()), this, SLOT(v24_debug_button_clicked()));
	
	// Bilder initialisieren
	pix[0] = QPixmap ((QString)"pix/0.png");
	pix[1] = QPixmap ((QString)"pix/1.png");
	pix[2] = QPixmap ((QString)"pix/2.png");
	pix[3] = QPixmap ((QString)"pix/3.png");
	pix[4] = QPixmap ((QString)"pix/4.png");
	pix[5] = QPixmap ((QString)"pix/5.png");


	// Simulationsdialog
	this->lg->stopSimulationButton->setEnabled(false);
	this->lg->startSimulationButton->setEnabled(false);
	this->lg->resumeSimulationButton->setEnabled(false);	
	this->sd->stopSimulationButton->setEnabled(false);
	this->sd->startSimulationButton->setEnabled(false);
	this->sd->resumeSimulationButton->setEnabled(false);		
	connect(lg->startSimulationButton, SIGNAL(clicked()), this, SLOT(start_simulation_clicked()));
	connect(lg->stopSimulationButton, SIGNAL(clicked()), this, SLOT(stop_simulation_clicked()));
	connect(lg->resumeSimulationButton, SIGNAL(clicked()), this, SLOT(resume_simulation_clicked()));
	connect(sd->startSimulationButton, SIGNAL(clicked()), this, SLOT(start_simulation_clicked()));
	connect(sd->stopSimulationButton, SIGNAL(clicked()), this, SLOT(stop_simulation_clicked()));
	connect(sd->resumeSimulationButton, SIGNAL(clicked()), this, SLOT(resume_simulation_clicked()));


}

/**
 * Diese Funktion stellt einen Übergebenen 
 * String im Debug-Label dar.
 * @param _d String zur Darstellung im Debug-Label
 */
void Simulation::debugOutput (QString _d) {

	lg->debugLabel->setText (_d);
}

/**
*
*/
void Simulation::stopLearningThread() {
	DBG();
	if (this->pkt != NULL) {
		pkt->setModusFromExtern('p');
	}
}
/**
*
*/
void Simulation::adjust_network_error(double _newError) {
	// Debug-Ausgabe
	DBG();
	printf ("network error automatically adjusted from %f to: %f\n", 					this->lg->netDesiredError->text().toDouble(), 
		_newError);

	// Textfeld aktualisieren
	this->lg->netDesiredError->setText (QString::number(_newError));
}
