#include "qLearning.h"
#include "gui_debug.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

qLearningThread::qLearningThread(int _g, int _iHiddenLayers, int _iHiddenUnits) :ValueIterationThread(QWorld::getInstance()->getX(), QWorld::getInstance()->getY(), QWorld::getInstance()->getGamma(), _g, _iHiddenLayers, _iHiddenUnits)
{
	printf("Konstruktor qLearning!\n");
}

void qLearningThread::learn(int _speed, long double _gamma){
	this->setSpeed (_speed);
	printf("QLearning - learn() start\n");
	gamma = _gamma;
	char _input;
	double in[2];

	while( (_input = this->getModus()) != 'q' ) {

		switch (_input) {
			/*************************************
			* GUI 'next iteration'-Button oder   *
			* 'start learning'-Button	     *
			*************************************/
			// fortlaufend lernen (iterieren)
			case 'l':
			// Eine einzige VI Iteration	
			case 'i':
				this->netOn = false;

				printf ("performing Q-Learning\n");
				setValues();

				iteration++;
				printMatrix();

				// beim weiterlernen - kurz schlafen legen
				if (_input == 'l') {
					usleep(speed * 10000);

				// ansonsten nach einer iteration abbrechen
				} else {
					setModus('q');
				}

				break;

			/*************************************
			 * GUI 'next iteration'-Button oder  *
			 * 'start learning'-Button wenn nach *
			 * m "Value Iterationen" n "Back-    *
			 * propagation"-Schritte gemacht     *
			 * werden sollen.		     *
			 *************************************/
			// fortlaufend lernen (iterieren)
			case 'B':
			// Eine einzige VI Iteration	
			case 'b':

				this->netOn = true;
				printf ("performing Q-Learning\n");
				setValues();	
				iteration++;
				printf ("start training network ...\n");
				this->coachNet();

				// beim weiterlernen - kurz schlafen legen
				if (_input == 'B') {
					usleep(speed * 10000);
					this->printMatrix();
				
				// ansonsten nach einer iteration abbrechen
				} else {
					cout <<"abbrechen" <<endl;
					setModus('p');
				}

				break;

			// Tabelle zurücksetzen => gedächtnis löschen
			case 'c':

				printf ("clearing table ...\n");
				for(int y=0; y<scalY; y++) {
					for(int x=0; x<scalX; x++) {
						value[x][y] = 0.00000;
						emit emitValue (y, x, value[x][y], 5);
					}	
				}
				printf ("setting random net-values\n");


				delete (network);
				// Dummy zuerst, damit auf jeden Fall ein komplett neues Netz erzeugt wird
				network = new Network(2, 1, this->iHiddenLayers+3, this->iHiddenUnits, "qLearning");
				this->initNet();
				delete (network);

				// Jetzt das richtige Netz erzeugen
				network = new Network(2, 1, this->iHiddenLayers+2, this->iHiddenUnits, "qLearning");

				this->initNet();

				iteration = 0;
				printf ("table cleared\n");
				printMatrix();

				// thread verlassen
				setModus('q');

				calculatetrainValues = false;
				break;
			
			// Net-Values zu VI kopieren
			case 'k':
				
				for(int y=0; y<scalY; y++) {
					for(int x=0; x<scalX; x++) {
						in[0] = x;
						in[1] = y;
						network->getNetValue(in, &value[x][y]);
						emit emitValue (y, x, value[x][y], getBestDirection(x,y));
					}	
				}

				// thread verlassen
				setModus('q');
				break;

			// Values ausgeben
			case 'p':
				printMatrix();

				// thread verlassen
				setModus('q');

				sleep(1);
				break;
	
			default:
				printMatrix();
				DBG();
				printf ("INPUT=%c\n", _input);
				sleep(1);
				break;
		}
	}

	printf ("Q-Learning - learn() - finished()\n");
}

/**
*
* @param action
* @param col
* @param row
* @return 
*/
long double qLearningThread::evaluate(int action, int col, int row) {
	int xNew, yNew;
	long double dbl_value;
	double in[2], Feedback, out;

	xNew = newX(action, col);
	yNew = newY(action, row);

	Feedback = feedback[col][row][action];

	// Bewertungs-Formel (setzt Kenntnis des zukuenftigen Feedbacks voraus)
	if(netOn) {	
		in[0] = xNew;
		in[1] = yNew;
		network->getNetValue(in, &out);
		dbl_value = Feedback + (long double)(this->gamma * out);
	} else {
		dbl_value = Feedback + (long double)(this->gamma * value[xNew][yNew]);
	}

	// Check ob Wert, das maximum �berschritten hat (bei Neuronalen Netz)
	if (netOn) {
		if (dbl_value > (this->maxReward * (1.101/(1.0-this->gamma)))) {
			printf ("ADJUSTING OVERSIZED NET VALUE (y=%d, x=%d) FROM %f TO %f\n", yNew, xNew, (double)dbl_value, (this->maxReward * (1/(1-this->gamma))));
			dbl_value=(this->maxReward * (1/(1-this->gamma)))*0.9;
		}
	}
	return dbl_value;
}

/**
*
*/
void qLearningThread::printMatrix() {

	int x=0, y=0;
	char cur_mode = this->getModus();
	this->setModus(cur_mode);
	double in[2], out;
	double minValue=100000, maxValue=-100000;

	if ((cur_mode == 'l' && (
		speed >= 10 ||     			
		(speed==0 && (iteration%1000)==0)||	
		(speed>0 && speed<10 && (iteration%100)==0)))
		|| cur_mode=='i' || cur_mode=='p' || this->netOn==true ) {

		for(y=0; y < scalY; y++) {
			for(x=0; x < scalX; x++) {
				this->printMutex->lock();

				if(netOn) {
					in[0] = x;
					in[1] = y;
					network->getNetValue(in, &out);
					if (out < minValue) minValue=out;
					if (out > maxValue) maxValue=out;
					emit emitValue(y, x, out, getBestDirection(x,y));
				}
				else {
					if (value[x][y] < minValue) minValue=value[x][y];
					if (value[x][y] > maxValue) maxValue=value[x][y];
					emit emitValue(y, x, value[x][y], getBestDirection(x,y));
				}

			}
		}

		// minimalen und maximalen Value ausgeben
		cout << "minValue=" <<minValue <<", maxValue=" <<maxValue 
		   <<", difference=" <<maxValue-minValue <<" (" <<(100*(maxValue-minValue))/maxValue <<"%)"<<endl;

		// Am Ende der Ausgabe noch die Iterationsnummer ausgeben
		if (!netOn) {
			emit debugOutput (QString("displaying Q-table - iteration #") + QString::number(iteration));

		// Anzahl der Neural-Network trainingsschritte ausgeben
		} else {
			emit debugOutput (
			"displaying network \tvi-count=" + QString::number(iteration) + 
			", bp-count=" + QString::number (this->network_training_result->total_epochs) + 
			", last error=" + QString::number(this->network_training_result->error) );
		}
	}
}

