#include "valueIteration.h"
#include "gui_debug.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

 
/*********** VALUE-ITERATION Funktionen ******************************************************************************************/
/**
*
*/
void ValueIterationThread::printMatrix() {

	int x=0, y=0;
	char cur_mode = this->getModus();
	this->setModus(cur_mode);
	double in[2], out;
	double minValue=100000, maxValue=-100000;

	if ((cur_mode == 'l' && (
		speed >= 10 ||     			
		(speed==0 && (iteration%1000)==0)||	
		(speed>0 && speed<10 && (iteration%100)==0)))
		|| cur_mode=='i' || cur_mode=='p' || this->netOn==true ) {

		for(y=0; y < scalY; y++) {
			for(x=0; x < scalX; x++) {
				this->printMutex->lock();

				if(netOn) {
					in[0] = x;
					in[1] = y;
					network->getNetValue(in, &out);
					if (out < minValue) minValue=out;
					if (out > maxValue) maxValue=out;
					emit emitValue(y, x, out, getBestDirection(x,y));
				}
				else {
					if (value[x][y] < minValue) minValue=value[x][y];
					if (value[x][y] > maxValue) maxValue=value[x][y];
					emit emitValue(y, x, value[x][y], getBestDirection(x,y));
				}

			}
		}

		// minimalen und maximalen Value ausgeben
		cout << "minValue=" <<minValue <<", maxValue=" <<maxValue 
		   <<", difference=" <<maxValue-minValue <<" (" <<(100*(maxValue-minValue))/maxValue <<"%)"<<endl;

		// Am Ende der Ausgabe noch die Iterationsnummer ausgeben
		if (!netOn) {
			emit debugOutput (QString("displaying VI-table - iteration #") + QString::number(iteration));

		// Anzahl der Neural-Network trainingsschritte ausgeben
		} else {
			emit debugOutput (
			"displaying network \tvi-count=" + QString::number(iteration) + 
			", bp-count=" + QString::number (this->network_training_result->total_epochs) + 
			", last error=" + QString::number(this->network_training_result->error) );
		}
	}
}

/**
*
* @param action
* @param _x
* @param _y
*/
bool ValueIterationThread::testBorders(int action, int _x, int _y) {
	switch(action) {
		case DOWN:	{
			if(_y >= (scalY-1)) return false;
			break;
		}
		case UP:	{
			if(_y <= 0) return false;
			break;
		}
		case RIGHT:	{
			if(_x >= (scalX-1)) return false;
			break;
		}
		case LEFT:	{
			if(_x <= 0) return false;
			break;
		}
	}
	return true;
}

/**
*
* @param action
* @param xOld
* @return 
*/
int ValueIterationThread::newX(int action, int xOld) {
	switch(action) {
		case RIGHT:	{
			return (xOld + 1);
			break;
		}
		case LEFT:	{
			return (xOld - 1);
			break;
		}
	}
	return xOld;
}

/**
*
* @param actionextern
* @param yOld
* @return 
*/
int ValueIterationThread::newY(int action, int yOld) {
	switch(action) {
		case DOWN:	{
			return (yOld + 1);
			break;
		}
		case UP:	{
			return (yOld - 1);
			break;
		}
	}
	return yOld;
}

/**
*
* @param action
* @param col
* @param row
* @return 
*/
long double ValueIterationThread::evaluate(int action, int col, int row) {
	int xNew, yNew;
	long double dbl_value;
	double in[2], Feedback, out;

	xNew = newX(action, col);
	yNew = newY(action, row);

	Feedback = feedback[col][row][action];

	// Bewertungs-Formel (setzt Kenntnis des zukuenftigen Feedbacks voraus)
	if(netOn) {	
		in[0] = xNew;
		in[1] = yNew;
		network->getNetValue(in, &out);
		dbl_value = Feedback + (long double)(this->gamma * out);
	} else {
		dbl_value = Feedback + (long double)(this->gamma * value[xNew][yNew]);
	}

	// Check ob Wert, das maximum �berschritten hat (bei Neuronalen Netz)
	if (netOn) {
		if (dbl_value > (this->maxReward * (1.101/(1.0-this->gamma)))) {
			printf ("ADJUSTING OVERSIZED NET VALUE (y=%d, x=%d) FROM %f TO %f\n", yNew, xNew, (double)dbl_value, (this->maxReward * (1/(1-this->gamma))));
			dbl_value=(this->maxReward * (1/(1-this->gamma)))*0.9;
		}
	}
	return dbl_value;
}

/**
*
* @param _x
* @param _y
* @return
*/
int ValueIterationThread::getBestDirection(int _x, int _y) {

	
	long double value, maximum = -10000;
	bool improved = false;
	int best = 4, bestActions=0, bestActionArray[NUMBEROFACTIONS];

	for(int action=0; action < NUMBEROFACTIONS; action++) {

		if( !testBorders(action, _x, _y) ) continue;

		value = evaluate(action, _x, _y);

		if(value == maximum) {
			bestActions++;
			bestActionArray[bestActions] = action;

		} else if(value > maximum) {
			maximum = value;
			best = action;
			improved = true;

			bestActions=0;
			bestActionArray[bestActions] = action;
		}
	}

	if (bestActions > 0) {
		best = bestActionArray[ (rand()%(bestActions+1)) ];
	}

	return best;
}

/**
*
*/
void ValueIterationThread::setValues() {
	long double dbl_value, maximum = -10000;

	// Bei neuronalem Netz, sollen nicht alle Werte gelernt werden um 
	// die Generalisierungsf�higkeit zu testen. Deswegen werden nun X Zufallszahlen gezogen und nur diese gelernt
	if (this->randomStates!=NULL) delete this->randomStates;
	this->randomStates = new int[this->trainRandomValues];
	int uniqueStatesCount=0, iTest;
	bool found=false;
	while (uniqueStatesCount < this->trainRandomValues) {
		found = false;
		this->randomStates[uniqueStatesCount] = (int)((float)((float)rand() / RAND_MAX) * (float)(this->scalX * this->scalY));

		// Testen ob Zufallszahl schon einmal vorkam
		for (iTest=0; iTest<uniqueStatesCount; iTest++) {
			if (this->randomStates[iTest] == this->randomStates[uniqueStatesCount]) {
				found=true;
				iTest = uniqueStatesCount;
			}
		}
		// Wenn Zahl noch nicht gezogen, dann n�chste suchen
		if (!found) {
			uniqueStatesCount++;
		}
	}
	cout <<endl <<endl;
	

	// Beim neuronalen Netz nur die Werte trainieren, welche als Zufallszahl gezogen wurden
		int x, y;

		// Werte berechnen und zun�chst lokal speichern
		for (int s=0; s<this->trainRandomValues; s++) {
			y=(this->randomStates[s]/scalX);
			x=this->randomStates[s] -(y*scalY);
			for(int action=0; action<NUMBEROFACTIONS; action++) {
				if( !testBorders(action, x, y) ) continue;
				dbl_value = evaluate(action, x, y);
				if(dbl_value > maximum) maximum = dbl_value;
			}
			
			value[x][y] = (long double) maximum;

			maximum = -10000;
			steps++;
		}

}

/**
* 
* @param _speed
* @param _gamma
*/
void ValueIterationThread::learn(int _speed, long double _gamma) {

	this->setSpeed (_speed);

	this->gamma = _gamma;
	char _input;
	double in[2];

	while( (_input = this->getModus()) != 'q' ) {

		switch (_input) {
			/*************************************
			* GUI 'next iteration'-Button oder   *
			* 'start learning'-Button	     *
			*************************************/
			// fortlaufend lernen (iterieren)
			case 'l':
			// Eine einzige VI Iteration	
			case 'i':
				this->netOn = false;

				printf ("performing value iteration\n");
				setValues();

				iteration++;
				printMatrix();

				// beim weiterlernen - kurz schlafen legen
				if (_input == 'l') {
					usleep(speed * 10000);

				// ansonsten nach einer iteration abbrechen
				} else {
					setModus('q');
				}

				break;

			/*************************************
			 * GUI 'next iteration'-Button oder  *
			 * 'start learning'-Button wenn nach *
			 * m "Value Iterationen" n "Back-    *
			 * propagation"-Schritte gemacht     *
			 * werden sollen.		     *
			 *************************************/
			// fortlaufend lernen (iterieren)
			case 'B':
			// Eine einzige VI Iteration	
			case 'b':

				this->netOn = true;
				printf ("performing value iteration\n");
				setValues();	
				iteration++;
				printf ("start training network ...\n");
				this->coachNet();

				// beim weiterlernen - kurz schlafen legen
				if (_input == 'B') {
					usleep(speed * 10000);
					this->printMatrix();
				
				// ansonsten nach einer iteration abbrechen
				} else {
					cout <<"abbrechen" <<endl;
					setModus('p');
				}

				break;

			// Tabelle zurücksetzen => gedächtnis löschen
			case 'c':

				printf ("clearing table ...\n");
				for(int y=0; y<scalY; y++) {
					for(int x=0; x<scalX; x++) {
						value[x][y] = 0.00000;
						emit emitValue (y, x, value[x][y], 5);
					}	
				}
				printf ("setting random net-values\n");


				delete (network);
				// Dummy zuerst, damit auf jeden Fall ein komplett neues Netz erzeugt wird
				network = new Network(2, 1, this->iHiddenLayers+3, this->iHiddenUnits, "valueIteration");
				this->initNet();
				delete (network);

				// Jetzt das richtige Netz erzeugen
				network = new Network(2, 1, this->iHiddenLayers+2, this->iHiddenUnits, "valueIteration");

				this->initNet();

				iteration = 0;
				printf ("table cleared\n");
				printMatrix();

				// thread verlassen
				setModus('q');

				calculatetrainValues = false;
				break;
			
			// Net-Values zu VI kopieren
			case 'k':
				
				for(int y=0; y<scalY; y++) {
					for(int x=0; x<scalX; x++) {
						in[0] = x;
						in[1] = y;
						network->getNetValue(in, &value[x][y]);
						emit emitValue (y, x, value[x][y], getBestDirection(x,y));
					}	
				}

				// thread verlassen
				setModus('q');
				break;

			// Values ausgeben
			case 'p':
				printMatrix();

				// thread verlassen
				setModus('q');

				sleep(1);
				break;
	
			default:
				printMatrix();
				DBG();
				printf ("INPUT=%c\n", _input);
				sleep(1);
				break;
		}
	}

	printf ("Value-Iteration - learn() - finished()\n");
}




/*********** Klassen-Methoden ******************************************************************************************/
/**
* Destructor
*/
ValueIterationThread::~ValueIterationThread() {
	delete (network);
}

/**
*
* @param _trainRandomValues
*/
void ValueIterationThread::setTrainRandomValues(int _trainRandomValues) {
	this->trainRandomValues = _trainRandomValues;
}

/**
*
* @param _x
* @param _y
* @param _r
* @param _g
* @param _iHiddenLayers
* @param _iHiddenUnits
*/
ValueIterationThread::ValueIterationThread(int _x, int _y, double _r, int _g, int _iHiddenLayers, int _iHiddenUnits)  {

	// Zufallszahlengenerator initalisieren
	srand( (unsigned int) time(NULL));

	this->auto_adjust_error=false;
	this->skipNextPrintMatrix = false;
	this->scalX = _x;
	this->scalY = _y;
	this->trainRandomValues = _x*_y;  // 100% => Alle Values trainieren

	this->maxReward = 1;
	this->minReward = -1;
	this->randomStates = NULL;

	// Alle Values auf 0 setzen
	for (int x=0; x<this->scalX; x++) {
		for (int y=0; y<this->scalY; y++) {
			value[x][y] = 0;
		}
	}

	this->printMutex = new QMutex;
	this->inputMutex = new QMutex;
	this->explored = false;
	this->iteration = 0;
	this->netOn = calculatetrainValues = false;
	
	this->setSpeed (_g);
	this->setModusFromExtern(0);
	this->setGamma (_r);
	printf ("scalX=%d, scalY=%d\n",  scalX, scalY);

	this->iHiddenUnits = _iHiddenUnits;
	this->iHiddenLayers = _iHiddenLayers;

	network = new Network(2, 1, _iHiddenLayers+2, _iHiddenUnits, "valueIteration");
	this->initNet();
}

/**
*
* @param _max_epochs
* @param _iterations_between_report
* @param _desired_error
* @param _dEta
* @param  _auto_adjust_error
* @param _activation_stepness_hidden
* @param _activation_stepness_output
* @param _activation_function_hidden
* @param _activation_function_output
*/
void ValueIterationThread::setNetworkParameters (int _max_epochs, int _iterations_between_report, double _desired_error, double _dEta, bool _auto_adjust_error, double _activation_stepness_hidden, double _activation_stepness_output, int _activation_function_hidden, int _activation_function_output ) {
	this->network->setParameters (_max_epochs, _iterations_between_report, _desired_error, _dEta, _activation_stepness_hidden, _activation_stepness_output, _activation_function_hidden, _activation_function_output);
	this->auto_adjust_error = _auto_adjust_error;
}


/**
* Diese Funktion, wird bei thread->start() automatisch aufgerufen
*/
void ValueIterationThread::run() {
	learn (this->speed, this->gamma);
}

/**
*
* @param _gamma
*/
void ValueIterationThread::setGamma (double _gamma) {
	printf ("Value-Iteration - new gamma (%f)\n", _gamma);
	this->gamma = _gamma;
	if (netOn) {
		network->setMinMaxInputOutputValues(0, scalX > scalY ? (scalX-1) : (scalY-1), this->minReward/(1-this->gamma), this->maxReward/(1-this->gamma));
	}
}

/**
*
* @param _row
* @param _col
* @param _value
*/
void ValueIterationThread::setViValue (int _row, int _col, double _value) {
	printf ("Value-Iteration - new Value {%d, %d} = %f\n", _row, _col, _value);
	value[_col][_row] = _value;	
}

/**
*
* @param _g
*/
void ValueIterationThread::setSpeed (int _g) {
	printf ("Value-Iteration - new speed (%d)\n", _g);
	speed = _g;
}

/**
*
* @param _x
*/
void ValueIterationThread::setModus (char _x) {
	printf ("Value-Iteration - intenal modus changed to: '%c'\n", _x);
	this->in = _x;
}

/**
*
* @param _x
*/
void ValueIterationThread::setModusFromExtern (char _x) {
	printf ("Value-Iteration - extern modus: '%c'\n", _x);
	this->in_ext = _x;
}

/**
*
* @return
*/
char ValueIterationThread::getModus () {

	char in_tmp;
	// externer Modus hat priorität !!!
	if (this->in_ext != 0) {
		in_tmp = this->in_ext;
		this->in = this->in_ext;

		this->in_ext = 0;
		printf ("returning extern modus: '%c'\n", in_tmp);
		return in_tmp;
	} else {
		printf ("returning internal modus: '%c'\n", this->in);
		return this->in;
	}
}

/**
*
*/
void ValueIterationThread::unlockMutex() {
	this->printMutex->unlock();
}

/**
*
* @param row
* @param _col
* @param _up
* @param _down
* @param _left
* @param _right
*/
void ValueIterationThread::setFeedback(int row, int _col, double _up, double _down, double _left, double _right) {
	feedback[_col][row][UP] = _up;
	feedback[_col][row][DOWN] = _down;
	feedback[_col][row][LEFT] = _left;
	feedback[_col][row][RIGHT] = _right;
}


/***** ERWEITERUNG MIT NNetz (von Peter) *********************************************************/
/**
*
*/
void ValueIterationThread::deactivateNet() {
	this->netOn = false;
	printf ("Value Iteration - neural network is temporarily deactivated\n");
}

/**
*
*/
void ValueIterationThread::activateNet() {
	this->netOn = true;
	printf ("Value Iteration - neural network is activated\n");
}

/**
*
*/
void ValueIterationThread::initNet() {
	// Grenzen setzen
	this->network->setMinMaxInputOutputValues(
		0, 					// Achsen (Input)
		scalX > scalY ? (scalX-1) : (scalY-1), 	//
		this->minReward/(1-this->gamma), 			// Value: maxfeedback* 1/(1-gamma)
		this->maxReward/(1-this->gamma));			// 

	// Gewichte 
	this->network->clear();

	// Struktur f�r Trainingsoutput
	this->network_training_result = new NETWORKTRAININGRESULTS();

}

/**
*
*/
void ValueIterationThread::coachNet() {
	double in[2];
	int newMaxReward=0, newMinReward=0;
	int x, y;

	// neue Trainingsdatendatei erzeugen
	network->createNewTrainingDataFile();

	// Maximales sowie Minimales Feedback suchen um Netz optimal anzupassen
	cout <<"checking for (minReward, maxReward) = (" <<minReward <<", " <<maxReward <<")" <<endl;
	for(int y=0; y<scalY; y++) {
		for(int x=0; x<scalX; x++) {
			for(int aktion=0; aktion<NUMBEROFACTIONS; aktion++) {
				// Check f�r maximales Feedback
				if (feedback[x][y][aktion] > newMaxReward) {
					newMaxReward = (int)feedback[x][y][aktion];

				// Check f�r minimales Feedback
				} else if (feedback[x][y][aktion] < newMinReward) {
					newMinReward = (int)feedback[x][y][aktion];	
				}
			}
		}
	}

	// Bei Bedarf skaling anpassen
	if ((newMaxReward != this->maxReward) || (newMinReward != this->minReward)) {
		cout 	<<"adjusting (minReward, maxReward) to: (" 
			<<newMinReward <<", " <<newMaxReward <<")" <<endl;		
		this->maxReward = newMaxReward;
		this->minReward = newMinReward;
		this->initNet();
	}
	for (int s=0; s<this->trainRandomValues; s++) {

		y=(this->randomStates[s]/scalX);
		x=this->randomStates[s] -(y*scalY);

		// Netz trainieren und gelerntes Value wieder holen
		in[0] = x;
		in[1] = y;
		network->addTrainingData(in, &value[x][y]);
	}

	// Netz mit allen Values trainieren
	network->trainNet(network_training_result);

	emit debugOutput (
	   "train net \tvi-count=" + QString::number(iteration) + 
	   ", bp-count=" + QString::number(this->network_training_result->total_epochs) + 
	   ", error=" + QString::number(this->network_training_result->error));

	if (this->auto_adjust_error) {

		// * Wenn Fehler zu hoch (perfekt gelernt weniger als 5% der 
		//   maximalen Lernschritte) => Fehler auf letzten Fehler setzten
		// * Wenn Fehler zu klein (netz hat nicht gelernt) auf letzten
		//   fehler setzen
		// Wenn Fehler zu klein (netz hat nicht gelernt) auf letzten fehler setzen
		if (this->network_training_result->last_epochs == this->network_training_result->max_epochs) {
			emit emitNewNetworkError ( this->network_training_result->error );
			network->setDesiredError ( this->network_training_result->error );
		// Wenn Fehler zu hoch (perfekt gelernt weniger als 5% der maximalen Lernschritte) => Fehler auf letzten Fehler setzten
		} else if (this->network_training_result->last_epochs <= (this->network_training_result->max_epochs*(5.0/100.0))) {
			emit emitNewNetworkError ( this->network_training_result->trainError/2.0 );
			network->setDesiredError ( this->network_training_result->trainError/2.0 );
		}
	}
}

