#include "NNetwork.h"
#include "gui_debug.h"

#include <doublefann.h>


#include <ios>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <stdio.h>

using std::cout;
using std::cerr;
using std::endl;
using namespace std;
using std::setw;
using std::left;
using std::right;
using std::showpos;
using std::noshowpos;

unsigned long int LAST_EPOCHS=0;
double LAST_ERROR = 0;

/**
* Callback Funktion um den Lernfehler auf der Konsole auszugeben
* @param *ann
* @param *train
* @param max_epochs
* @param epochs_between_reports
* @param desired_error
* @param epochs
* @return
*/

int FANN_API print_callback(struct fann *ann, struct fann_train_data *train,
                           unsigned int max_epochs, unsigned int epochs_between_reports,
                           float desired_error, unsigned int epochs)
{
    cout << "Epochs     " << setw(8) << epochs << ". "
        << "current error: " << left << fann_get_MSE(ann) << right << endl;
    LAST_EPOCHS = epochs;
    LAST_ERROR = fann_get_MSE(ann);
    return 0;
}

/** 
* Network Grenzen setzen um Input sowie Output bestmoeglich zu skalieren 
* @param _minInput
* @param _maxInput
* @param _minOutput
* @param _maxOutput
*/
void Network::setMinMaxInputOutputValues  (double _minInput, double _maxInput, double _minOutput, double _maxOutput) {

	DBG();
	this->minInputValue = _minInput;
	this->maxInputValue = _maxInput;
	this->minOutputValue = _minOutput;
	this->maxOutputValue = _maxOutput;

	cout 	<<"minInputValue=" <<this->minInputValue
		<<", maxInputValue=" <<this->maxInputValue <<endl
		<<"minOutputValue=" <<this->minOutputValue
		<<", maxOutputValue=" <<this->maxOutputValue
		<<endl;
}

/** 
* Funktion um das Network-Training zu beginnen 
* @param *_results
*/
void Network::trainNet (NETWORKTRAININGRESULTS *_results) {
	DBG();
	// Anzahl von inputs, outputs sowie trainingsdatacount an dateianfang setzen
	if (dataFile != NULL) {
		// An Dateianfang gehen
		rewind(dataFile) ;

		// Header schreiben
		fprintf (dataFile, "%d %d %d", this->dataCount, this->num_input, this->num_output);
		fclose(dataFile);

		dataFile = NULL;
	}

	// Falls noch kein Network geladen, wird eins erzeugt und die Counter zur�ckgesetzt
	if (this->ann == NULL) {
		this->clear();
		LAST_EPOCHS=0;
	}

	cout <<"maxInputValue=" <<maxInputValue << ", minInputValue=" <<minInputValue <<endl;
	cout <<"maxOutputValue=" <<maxOutputValue << ", minOutputValue=" <<minOutputValue <<endl;

	// Error-Funktion linear
	fann_set_train_error_function(this->ann, FANN_ERRORFUNC_LINEAR);

	// Activation-Function setzen
	fann_set_activation_function_hidden(this->ann, FANN_SIGMOID_SYMMETRIC);
	fann_set_activation_function_output(this->ann, FANN_SIGMOID_SYMMETRIC);

	// Activation-Stepness setzen
	fann_set_activation_steepness_hidden(
		this->ann, this->activation_stepness_hidden);
	fann_set_activation_steepness_output(
		this->ann, this->activation_stepness_output);

	cout <<"Training network on datafile: " <<this->dataFilename
	     <<", max_iterations=" <<this->max_iterations 
	     <<", iterations_between_reports=" <<this->iterations_between_reports 
	     <<", desired_error=" <<this->desired_error 
             <<", trainingDataCount=" <<this->dataCount <<endl;

	//fann_print_parameters(this->ann);

	fann_set_callback(this->ann, print_callback);
	fann_train_on_file(this->ann, this->dataFilename, max_iterations, iterations_between_reports, desired_error);

	// Anzahl der Trainingsepochen erg�nzen
	_results->total_epochs += LAST_EPOCHS;
	_results->error = LAST_ERROR;
	_results->last_epochs = LAST_EPOCHS;
	_results->trainError = this->desired_error;
	_results->max_epochs = this->max_iterations;
	
	cout <<"Saving network to file: " <<this->networkFilename <<endl; 
	fann_save(this->ann, networkFilename);
}

/**
* Funktion fuer Konfiguration der Networkwerk-Parameter.
* @param _max_epochs
* @param _iterations_between_report
* @param _desired_error
* @param _dEta
* @param _activation_stepness_hidden
* @param _activation_stepness_output
* @param _activation_function_hidden
* @param _activation_function_output
*/
void Network::setParameters(int _max_epochs, int _iterations_between_report, double _desired_error, double _dEta, double _activation_stepness_hidden, double _activation_stepness_output, int _activation_function_hidden, int _activation_function_output) {
	DBG();
	cout <<"setting network training parameters: max_epochs=" << _max_epochs 
	     <<", iterations_between_report=" <<_iterations_between_report
             <<", desired_error=" <<_desired_error 
	     <<", activation_function_hidden=" <<_activation_function_hidden 
	     <<", activation_stepness_hidden=" <<_activation_stepness_hidden
	     <<", activation_function_output=" <<_activation_function_output 
	     <<", activation_stepness_output=" <<_activation_stepness_output <<endl;
	this->max_iterations = _max_epochs;
	this->iterations_between_reports = _iterations_between_report;
	this->desired_error = _desired_error;
	this->activation_stepness_hidden = _activation_stepness_hidden;

	this->activation_stepness_output = _activation_stepness_output;

	if (_activation_function_hidden==0) {
		this->activation_function_hidden = FANN_SIGMOID;
	} else if (_activation_function_hidden==1) {
		this->activation_function_hidden = FANN_SIGMOID_SYMMETRIC;
	} else  {
		this->activation_function_hidden = FANN_LINEAR;
	}

	if (_activation_function_output==0) {
		this->activation_function_output = FANN_SIGMOID;
	} else if (_activation_function_output==1) {
		this->activation_function_output = FANN_SIGMOID_SYMMETRIC;
	} else {
		this->activation_function_output = FANN_LINEAR;
	}
}

/** 
* Funktion um den Fehler zu setzen, bis zu welchem maximal gelernt werden soll 
* @param _desired_error
*/
void Network::setDesiredError(double _desired_error) {
	DBG();
	cout <<"setting desired_error=" <<_desired_error <<endl;
	this->desired_error = _desired_error;
}

/** Destruktor */
Network::~Network() {
	
	// Network wieder freigeben
	DBG();
	if (this->ann != NULL) fann_destroy(this->ann);
}

/** Funktion um eine neue Trainingsdatendatei in /tmp zu erzeugen */
void Network::createNewTrainingDataFile() {

	DBG();
	// Wenn Datei noch ge�ffnet => schlie�en
	if (dataFile != NULL) fclose(dataFile);

	// und wieder neu zum �berschreiben �ffnen
	dataFile = fopen(this->dataFilename, "w+");
	fprintf (dataFile, "                ");

	// Datensatz-Z�hler zur�cksetzen
	this->dataCount=0;
}

/** 
* Konstruktor 
* @param _num_input
* @param _num_output
* @param _num_layers
* @param _num_neurons_hidden
* @param _netName
*/
Network::Network(int _num_input, int _num_output, int _num_layers, int _num_neurons_hidden, const char* _netName) {

	DBG();
	// Initialparameter
	this->connection_rate = 1;
	this->learning_rate_eta = 0.9;
	this->num_input = _num_input;
	this->num_output = _num_output;
	this->num_layers = _num_layers;
	this->num_neurons_hidden = _num_neurons_hidden;
	this->activation_stepness_hidden=this->activation_stepness_output=0.01;
	// Standardm�ssig tanh aktivierung
	this->activation_function_hidden=this->activation_function_output=FANN_SIGMOID_SYMMETRIC;
	
	this->desired_error = 0.00003;
	this->max_iterations = 10000;
	this->iterations_between_reports = 1000;
	this->dataCount=0;

	//this->num_input = _num_input;
	//this->num_output = _num_output;
	//this->num_layers = _num_layers;
	//this->num_neurons_hidden = _num_neurons_hidden;

	/************************************************************
	 * Dateien zum speichern der Daten sowie des Networkes erzeugen
	 ***********************************************************/
	sprintf (this->dataFilename, "/tmp/nn-%s.data", _netName);
	sprintf (this->networkFilename, "/tmp/nn-%s.net", _netName);
	printf ("NN-Datei: %s, NN-Datendatei: %s\n", this->networkFilename, this->dataFilename);

	dataFile = fopen(this->dataFilename, "w+");
	// Datensatz-Z�hler zur�cksetzen
	this->dataCount=0;

	// Zeichen, dass noch kein Network existiert setzen
	this->ann = NULL;
}

/** 
* Funktion um ein neues Trainingsbeispiel der Trainingsdatendatei hinzuzufuegen 
* @param in[]
* @param out[]
*/
void Network::addTrainingData(double in[], double out[]) {

	char printString[200];

	// Erstes Input-Neuron	
	sprintf (printString, "adding traindata: in[%f(%f)", in[0], normIn(in[0]));
	//fprintf (dataFile, "\n%.30f", normIn(in[0]));
	fprintf (dataFile, "\n%f", normIn(in[0]));

	// ggf. weitere Input-Neuronen
	for (unsigned int i=1; i<this->num_input; i++) {
		sprintf(printString, "%s,%f(%f)", printString, in[i], normIn(in[i]));
		//fprintf (dataFile, " %.30f", normIn(in[i]));
		fprintf (dataFile, " %f", normIn(in[i]));
	}

	// Erstes Output-Neuron
	sprintf (printString, "%s], out[%f(%f)", printString, out[0], normOut(out[0]));
	//fprintf (dataFile, "\n%.30f", normOut(out[0]));
	fprintf (dataFile, "\n%f", normOut(out[0]));

	// ggf. weitere Output-Neuronen
	for (unsigned int i=1; i<this->num_output; i++) {
		sprintf(printString, "%s,%f(%f)", printString, out[i], normOut(out[i]));
		//fprintf (dataFile, " %.30f", normOut(out[i]));
		fprintf (dataFile, " %f", normOut(out[i]));
	}

	// Zeilenende
	sprintf (printString, "%s]\n", printString);

	// Datenz�hler inkrementieren
	dataCount++;	

	// Ausgabe des Trainingsdatensatzes auf Bildschirm
	//printf (printString);
}

/** 
* Funktion um den Network-Output von in[] nach out[] zu schreiben
* @param in[]
* @param out[]
*/
void Network::getNetValue(double in[], double out[]) {

	char printString[300];

	fann_type *normalizedIn = new fann_type[this->num_input];
	fann_type *calc_out = new fann_type[this->num_output];

	// Input im Skalierungsbereich -1..1 normalisieren
	for (unsigned int i=0; i<this->num_input; i++) {
		//cout <<"in[i] = " <<in[i] <<", normIn(in[i]) = " <<normIn(in[i]) <<endl;
		normalizedIn[i] = normIn(in[i]);
	}
		
	// Network befragen
        sprintf (printString, "asking network: (%f, %f) = ", normIn(in[0]), normIn(in[1]));
	this->ann = fann_create_from_file(this->networkFilename);
	if ((calc_out = fann_run(this->ann, normalizedIn)) == false) {
		cout <<"ERROR asking network" <<endl;
	}

	sprintf (printString, "%s[", printString);
	
	// Normalisierung f�r Output R�ckg�ngig machen
	for (unsigned int i=0; i<this->num_output; i++) {
		out[i] = reNormOut((double)calc_out[i]);
		sprintf (printString, "%s(%f => %f), ", printString, calc_out[i], out[i]);
	}
	sprintf (printString, "%s]\n", printString);

	// printf(printString)
}

/** Funktion um die Gewichte im Network zufaellig initialisieren zu lassen */
void Network::clear() {

	DBG();

	// altes Network freigeben
	if (this->ann != NULL) fann_destroy(this->ann);

	cout 	<<"CREATING NETWORK: " 
		<<"connection_rate=" <<connection_rate 
		<<", num_layers=" <<num_layers
		<<", num_input=" <<num_input
		<<", num_neurons_hidden=" <<num_neurons_hidden
		<<", num_output=" <<num_output <<endl;

	// Anzahl der Neuronen auf der Zwischenebene dynamisch bestimmen
	hidden_neurons = new unsigned int[this->num_layers];
	hidden_neurons[0] = this->num_input;	// Input-Schicht
	hidden_neurons[this->num_layers-1] = this->num_output; // Output-Schicht
	for (unsigned int i=1; i<=this->num_layers-2; i++) {
		hidden_neurons[i] = this->num_neurons_hidden; // Hidden-Schichten
	}

	// Networkwerk erzeugen
	this->ann = fann_create_standard_array(this->num_layers, hidden_neurons);

	// Zuf�llige Gewichte
	cout <<"randomizing weights..." <<endl;
	fann_randomize_weights(this->ann, -0.1, 0.1);
	
	fann_set_activation_function_hidden(this->ann, (fann_activationfunc_enum)this->activation_function_hidden);
    	fann_set_activation_function_output(this->ann, (fann_activationfunc_enum)this->activation_function_output);

	// Activation-Stepness setzen
	fann_set_activation_steepness_hidden(
		this->ann, this->activation_stepness_hidden);
	fann_set_activation_steepness_output(
		this->ann, this->activation_stepness_output);

	
	// Network in Datei speichern
	fann_save(this->ann, this->networkFilename);
}

/** 
* Funktion um den Network-Input in einen bestimmten Wertebereich zu normalisieren 
* @param in
* @return
*/
double Network::normIn(double in) {


	double scalingIn = (maxInputValue-minInputValue);
	in -= minInputValue;
	in /= scalingIn;

	in = (in*2)-1;  // Bereich -0.9..0.9

	return in;
}

/** 
* Funktion um den Network-Output in einen bestimmten Wertebereich zu normalisieren
* @param in
* @return
*/
double Network::normOut(double out) {
	double scalingOut = (maxOutputValue-minOutputValue);

	out -= minOutputValue;
	out /= scalingOut;

	out = (out*2)-1;  // Bereich -1..1
	return out;
}

/** 
* Funktion um den normalisierten Network-Output in den Wertebereich zurueck zu normalisieren 
* @param netOut
* @return
*/
double Network::reNormOut(double netOut) {
	double scalingOut = (maxOutputValue-minOutputValue);
	netOut = (netOut+1)/2;
	netOut *= scalingOut;
	netOut += minOutputValue;

	return netOut;
}
