#ifndef V24_H
#define V24_H 1

#include "SimulationMainWindow.h"
#include "V24_commands.h"
#include "gui_debug.h"
#include "qmessagebox.h"

#include "qextserialport.h"


/********************************************************
 * Einheitliche Kommandostruktur, welche die Kommando's
 * auf µC und GUI-Seite vereinheitlicht.
 *****************************************************/
typedef struct V24Command {
	unsigned char command;
	unsigned char dataLength;
	unsigned char data[18];
} V24COMMAND;

/******************************************************************************************
 * Einheitliche Struktur, die einen Zustand darstellt.                                    *
 * Diese Stuktur ist 12-Byte lang, da immer in 4-Byte Blöcken vom Compiler allokiert wird,*
 * aus diesem Grund wird die reele länge der bytes in STATESIZE angegeben
 ******************************************************************************************/
typedef struct State {
	float value;
	//unsigned char best_action;
	char feedback_up;
	char feedback_down;
	char feedback_left;
	char feedback_right;
} STATE;
#define STATESIZE 9

/**
 * Steuerklasse für V24-Kommunikation
 */
class V24Control {

  public:
		V24Control(QMutex *_guiMutex, QMainWindow *_vater);

		bool openDevice();
		void closeDevice();

		bool transmitCommand (V24COMMAND *c);
		bool receiveCommand (V24COMMAND *c);
		bool receiveTransmitOKCommand();

		void parseFloatData (unsigned char *c, float *_data);
		void parseIntegerData (unsigned char *d, unsigned int *_data);
		bool setInterface (QString _interface);

  private:
		bool transmit(unsigned char data);
		bool receive(unsigned char *data);
		bool transmitAndReceive(unsigned char tData, unsigned char rData);
		bool transmitting;
		bool receiving;
		QMainWindow *parent;
		void showErrorCode(unsigned long errorCode);

		Posix_QextSerialPort *port;
		char filename[50];
		QMutex *guiMutex;
		QMutex *conMutex;
		QString sDeviceInterface;
		bool rxtx;
};


#endif

