#include "gui.h"

void Simulation::apply_button_clicked()
{
	this->guiReady = false;
	static int firstRun=true;

	this->lg->ViTab->setEnabled (this->lg->ViLearningButton->isOn());
	this->lg->QTab->setEnabled (this->lg->QLearningButton->isOn());


	static int iHiddenLayers_old = 0, iHiddenUnits_old = 0, iRows_old = 0, iCols_old = 0;
	static double dGamma_old = 0;
	
	int iHiddenLayers = 0, iHiddenUnits = 0, iRows = 0, iCols = 0;
	double dGamma = 0, dEta = 0;
	QTableItem *strategy_item, *value_item, *feedback_item;
	
	// Gamma(Iterationsrate)
	this->gamma = dGamma = this->lg->viGammaInput->text().toDouble();

	// Netz Variablen
	dEta = this->lg->netEta->text().toDouble();
	iHiddenUnits = this->lg->netHiddenUnits->text().toInt();
	iHiddenLayers = this->lg->netHiddenLayers->text().toInt();

	// Grid Variablen
	this->scalY = iRows = lg->numRows->text().toInt();
	this->scalX = iCols = lg->numCols->text().toInt();
	iMantisse = this->lg->mantisseInput->text().toInt();
	i_cell = lg->numRows->text().toInt() * lg->numCols->text().toInt();

	// Anzahl Spalten und Zeilen
	lg->viValueTable->setNumRows(iRows);
	lg->viValueTable->setNumCols(iCols);
	lg->strategyTable->setNumRows(iRows);
	lg->strategyTable->setNumCols(iCols);

	// Wenn Neuronales Netz angeschaltet, dann 3x3 Feedback aktivieren
	lg->envRewardTable->setNumRows(iRows);
	lg->envRewardTable->setNumCols(iCols);

	// Nur wenn sich bei der Spalten und Zeilenanzahl was geändert hat, wird upgedated
	if ((iCols != iCols_old) || (iRows != iRows_old))
	{
		// Default-Werte setzen
		for (int i=0; i<lg->numRows->text().toInt(); i++) {
			for (int j=0; j<lg->numCols->text().toInt(); j++) {
	
				// Neue Items anlegen
				value_item = new QTableItem(lg->viValueTable, QTableItem::Never);
				strategy_item = new QTableItem(lg->strategyTable, QTableItem::Never);
				feedback_item = new QTableItem(lg->envRewardTable, QTableItem::Never);
	
				// Item der Value-Table zuweisen
				value_item->setText (QString::number ((double)0.000));
				value_item->setPixmap (pix[5]);
				lg->viValueTable->setItem(i, j, value_item);
	
				// Item der Strategy-Table zuweisen
				strategy_item->setPixmap (pix[5]);
				lg->strategyTable->setItem(i, j, strategy_item);
	
				// Item in der Feedback-Table
				feedback_item->setText ("");
				lg->envRewardTable->setItem(i, j, feedback_item);			
			
			}
		}
		
		// Tabelle passend resizen
		for (int j=0; j < lg->numCols->text().toInt(); j++) {
			lg->viValueTable->adjustColumn(j);
			lg->strategyTable->adjustColumn(j);
		}	
	
		// Thread beenden, wenn schon läuft
		if (this->pkt != NULL) {

			DBG();
			// Thread sicher beenden
			pkt->setModusFromExtern ('q');
			while (pkt->running());
			DBG();

			delete (pkt);
		}	
	
		// Geschwindigkeit auslesen
		speed = this->lg->viSpeedSlider->maxValue()-this->lg->viSpeedSlider->value();
	
		// Neuen Thread erzeugen
		this->pkt = new ValueIterationThread (
					(int)(this->lg->numCols->text().toInt()), 
					(int)(this->lg->numRows->text().toInt()), 
					this->gamma, 
					speed,
			 		this->lg->netHiddenLayers->value(),
			 		this->lg->netHiddenUnits->value());

		this->pkt->setTrainRandomValues((int)((float)this->scalX * (float)this->scalY * 
			(float)this->lg->viRandomValuesSlider->value() / (float)this->lg->viRandomValuesSlider->maxValue())
		);
	
		// Verbindung zum KI-Thread um richtig die Tabelle aktualisieren zu lassen
		connect(	(QObject*)this->pkt, 
				SIGNAL(emitValue(int, int, double, int)), 
				(QObject*)this, 
				SLOT(updatePolicyTable(int, int, double, int))
		);
		connect(	(QObject*)this->pkt, 
				SIGNAL(debugOutput(QString)), 
				(QObject*)this, 
				SLOT (debugOutput(QString))
		);
		connect(	(QObject*)this->pkt,
				SIGNAL(emitNewNetworkError(double)),
				(QObject*)this,
				SLOT (adjust_network_error(double))
		);
		
		this->lg->viSpeedSlider->setEnabled (true);
		connect(lg->viSpeedSlider, SIGNAL(sliderMoved(int)), this, SLOT(speed_slider_changed(int)));
		connect(lg->viRandomValuesSlider, SIGNAL(sliderMoved(int)), this, SLOT(vi_train_random_values_changed(int)));
		connect(lg->envRewardTable, SIGNAL(clicked(int, int, int, const QPoint&)),
			this, SLOT (feedbackClicked(int, int, int, const QPoint&)));
	
		// Buttons wieder enabeln
		this->lg->viLernenButton->setEnabled (true);
		this->lg->viIterationButton->setEnabled (true);
		this->lg->viSpeedSlider->setEnabled(true);
		this->lg->startSimulationButton->setEnabled(true);
		this->lg->viDeleteButton->setEnabled(true);
		this->lg->envLoadFileButton->setEnabled(true);
		this->lg->envSaveFileButton->setEnabled(true);
		this->lg->envSaveMatlabFileButton->setEnabled(true);
		this->lg->envSaveRewardButton->setEnabled(true);


		// Neuronales Netz zunächst deaktivieren
		if (firstRun==true) {
			this->learn_method_vi_clicked();
			firstRun=false;
		}

		printf ("Configuration successfully applied\n");

		// In GUI integrierte Simulation
		this->lg->canvasView->setDimension (	this->lg->numRows->value(),
							this->lg->numCols->value(),
							(this->lg->numRows->value()-1)/2,
							(this->lg->numCols->value()-1)/2
							//this->simuPos.y()+1,
							//this->simuPos.x()+1
		);
		// In externen Dialog integrierte Simulation
		this->sd->canvasView->setDimension (	this->lg->numRows->value(),
							this->lg->numCols->value(),
							(this->lg->numRows->value()-1)/2,
							(this->lg->numCols->value()-1)/2
							//this->simuPos.y()+1,
							//this->simuPos.x()+1
		);
	
	// Netz neu initialisieren, wenn Parameter ver�ndert wurden
	} else if ((iHiddenUnits != iHiddenUnits_old) || 
		   (iHiddenLayers != iHiddenLayers_old)) {

		// Thread beenden, wenn schon läuft
		if (this->pkt != NULL) {

			DBG();
			// Thread sicher beenden
			pkt->setModusFromExtern ('q');
			while (pkt->running());
			DBG();

			delete (pkt);
		}

		// Neuen Thread erzeugen
		this->pkt = new ValueIterationThread (
					(int)(this->lg->numCols->text().toInt()), 
					(int)(this->lg->numRows->text().toInt()), 
					this->gamma, 
					speed,
			 		this->lg->netHiddenLayers->value(),
			 		this->lg->netHiddenUnits->value());

		this->pkt->setTrainRandomValues((int)((float)this->scalX * (float)this->scalY * 
			(float)this->lg->viRandomValuesSlider->value() / (float)this->lg->viRandomValuesSlider->maxValue())
		);


		// Verbindung zum KI-Thread um richtig die Tabelle aktualisieren zu lassen
		connect(	(QObject*)this->pkt, 
				SIGNAL(emitValue(int, int, double, int)), 
				(QObject*)this, 
				SLOT(updatePolicyTable(int, int, double, int))
		);
		connect(	(QObject*)this->pkt, 
				SIGNAL(debugOutput(QString)), 
				(QObject*)this, 
				SLOT (debugOutput(QString))
		);
		connect(	(QObject*)this->pkt,
				SIGNAL(emitNewNetworkError(double)),
				(QObject*)this,
				SLOT (adjust_network_error(double))
		);

	} else if (dGamma != dGamma_old) {
		this->pkt->setGamma (dGamma);
	}

	// Anzahl von Zeilen und Spalten speichern
	iCols_old = iCols;
	iRows_old = iRows;

	// Netzvariablen speichern
	iHiddenUnits_old = iHiddenUnits;
	iHiddenLayers_old = iHiddenLayers;

	// Network-Training Parameter anpassen
	this->pkt->setNetworkParameters(
		this->lg->netMaxEpochs->value(),
		this->lg->netIterationsBetweenReport->value(), 
		this->lg->netDesiredError->text().toDouble(),
		dEta,
		this->lg->netAutoAdjustError->isChecked(),
		this->lg->netActivationStepnessHidden->text().toDouble(),
		this->lg->netActivationStepnessOutput->text().toDouble(),
		this->lg->netHiddenActivationFunction->currentItem(),
		this->lg->netOutputActivationFunction->currentItem()
	);

	int max;
	if(iCols> iRows) max = iCols;
	else max = iRows;

	// GUI ist nun bereit
	this->guiReady = true;
}

