#include "gui.h"

/**
*
*/
void Simulation::iteration_button_clicked() {

	DBG();

	// GUI sperren
	this->guiMutex->lock();

	this->stopLearningThread();

	// Kombination zwischen Value-Iteration+ Backpropagation
	if (this->lg->viLearnWithNN->isChecked()) {

		// Network-Training Parameter anpassen
		this->pkt->setNetworkParameters(
			this->lg->netMaxEpochs->value(),
			this->lg->netIterationsBetweenReport->value(), 
			this->lg->netDesiredError->text().toDouble(),
			this->lg->netEta->text().toDouble(),
			this->lg->netAutoAdjustError->isChecked(),
			this->lg->netActivationStepnessHidden->text().toDouble(),
			this->lg->netActivationStepnessOutput->text().toDouble(),
			this->lg->netHiddenActivationFunction->currentItem(),
			this->lg->netOutputActivationFunction->currentItem()
		);

		pkt->setModusFromExtern('b');

	// Value-Iteration mit Value-Memory im Hauptspeicher
	} else {
		pkt->setModusFromExtern('i');
	}


	if (!this->pkt->running()) this->pkt->start();

	// GUI wieder freigeben
	this->guiMutex->unlock();
}

/**
*
*/
void Simulation::learn_button_clicked()
{
	DBG();

	// GUI sperren
	this->guiMutex->lock();

	// Geschwindigkeit setzen
	speed = this->lg->viSpeedSlider->maxValue() - this->lg->viSpeedSlider->value();
	pkt->setSpeed (speed);

	this->stopLearningThread();

	// Kombination zwischen Value-Iteration+ Backpropagation
	if (this->lg->viLearnWithNN->isChecked()) {

		// Network-Training Parameter anpassen
		this->pkt->setNetworkParameters(
			this->lg->netMaxEpochs->value(),
			this->lg->netIterationsBetweenReport->value(), 
			this->lg->netDesiredError->text().toDouble(),
			this->lg->netEta->text().toDouble(),
			this->lg->netAutoAdjustError->isChecked(),
			this->lg->netActivationStepnessHidden->text().toDouble(),
			this->lg->netActivationStepnessOutput->text().toDouble(),
			this->lg->netHiddenActivationFunction->currentItem(),
			this->lg->netOutputActivationFunction->currentItem()
		);

		pkt->setModusFromExtern('B');

	// Pures Value-Iteration mit speicherung der Values in 
	} else {
		pkt->setModusFromExtern('l');
	}

	
	if (!this->pkt->running()) this->pkt->start();

 	lg->viLernenButton->setEnabled (false);
 	lg->viIterationButton->setEnabled (false);
 	lg->viStopButton->setEnabled (true);
 	lg->viDeleteButton->setEnabled (false);
 	lg->viLernenButton2->setEnabled (false);
 	lg->viIterationButton2->setEnabled (false);
 	lg->viStopButton2->setEnabled (true);
 	lg->viDeleteButton2->setEnabled (false);

 	lg->viSpeedSlider->setEnabled (true);

	// GUI wieder freigeben
	this->guiMutex->unlock();
}

/**
*
*/
void Simulation::delete_button_clicked()
{
	DBG();

	// GUI sperren
	this->guiMutex->lock();

	this->stopLearningThread();
	pkt->setModusFromExtern('c');

	if (!this->pkt->running()) this->pkt->start();

	// Button wieder freigeben
	lg->viStopButton->setEnabled (false);
	lg->viLernenButton->setEnabled (true);
	lg->viIterationButton->setEnabled(true);
	lg->viStopButton2->setEnabled (false);
	lg->viLernenButton2->setEnabled (true);
	lg->viIterationButton2->setEnabled(true);

	// GUI wieder freigeben
	this->guiMutex->unlock();
}

/**
*
*/
void Simulation::stop_button_clicked() {

	DBG();

	// GUI sperren
	this->guiMutex->lock();

	this->stopLearningThread();
	// Button wieder freigeben
	lg->viStopButton->setEnabled (false);
	lg->viDeleteButton->setEnabled (true);
	lg->viLernenButton->setEnabled (true);
	lg->viIterationButton->setEnabled (true);
	lg->viStopButton2->setEnabled (false);
	lg->viDeleteButton2->setEnabled (true);
	lg->viLernenButton2->setEnabled (true);
	lg->viIterationButton2->setEnabled (true);

	// GUI wieder freigeben
	this->guiMutex->unlock();
}

/**
*
* @param _value
*/
void Simulation::vi_train_random_values_changed (int _value) {

	int _percVal = (int)((100.0 * (float)_value) / (float)this->lg->viRandomValuesSlider->maxValue());
	int _valCount = (int)((float)scalX * (float)scalY * ((float)_percVal/100.0));

	// Neue Anzahl setzen, wieviel Prozent der Values trainiert werden
	if (this->pkt != NULL) {
		this->pkt->setTrainRandomValues(_valCount);
	}

	debugOutput ("GUI: training " + QString::number(_percVal) + "% (" + QString::number(_valCount) + ") random values to net");
	this->lg->viRandomNetSliderText->setText("training " + QString::number(_percVal) + "% (" + QString::number(_valCount) + " random) values to net:");
}

/**
*
* @param _value
*/
void Simulation::speed_slider_changed(int _value) {	
	speed = this->lg->viSpeedSlider->maxValue() - _value;
	pkt->setSpeed (speed);
	debugOutput ("Neue Geschwindigkeit: " + QString::number (speed));
	this->lg->viSpeedLabel->setText ("speed (" + QString::number (speed) + "): slow");
}

/**
*
* @param _row
* @param _column
* @param _value
* @param _bestdirection
*/
void Simulation::updatePolicyTable (int 	_row, int _column, double _value, int _bestdirection) {

	QTableItem *strategy_item, *value_item;
	static int updateCall = 0;

	this->dad->lock();

	// Aktion speichern
	this->action[_row][_column] = _bestdirection;
	
	//Value Table
	value_item = lg->viValueTable->item (_row, _column);
	value_item->setPixmap (pix[_bestdirection]);
	value_item->setText (QString::number (_value, 'f', iMantisse));

	// Value-Memory aktualisieren
	fValueMemory[_row][_column] = (float) _value;

	//Strategy-Table
	strategy_item = lg->strategyTable->item (_row, _column);

	strategy_item->setPixmap (pix[_bestdirection]);
		
	
	// Aller (x * y) Aufrufe  die Tabelle an die Größe anpassen
	updateCall++;

	if (updateCall == i_cell ) {
		for (int i=0; i < lg->numCols->text().toInt(); i++) {
			lg->viValueTable->adjustColumn (i);
			lg->strategyTable->adjustColumn (i);
		}
		lg->viValueTable->updateCell(_row, _column);
		lg->strategyTable->updateCell(_row, _column);

		updateCall = 0;
	}

	this->pkt->unlockMutex();
	
	// GUI-Mutex wieder freigeben
	this->dad->unlock();
}

/**
*
*/
void Simulation::learn_method_nn_clicked() {

	DBG();

	// GUI sperren
	this->guiMutex->lock();

	// Alle laufenden Aktionen anhalten
	this->stopLearningThread();
	this->pkt->activateNet();
	this->pkt->setModusFromExtern('p');	
	if (!this->pkt->running()) this->pkt->start();

	debugOutput ("neural network activated");

	this->lg->viLearnWithNN->setEnabled(false);
	this->lg->viLearnWithVI->setEnabled(true);
	this->lg->viLearnWithNN->setChecked(true);
	this->lg->viLearnWithVI->setChecked(false);

	// GUI wieder freigeben
	this->guiMutex->unlock();
}

/**
*
*/
void Simulation::learn_method_vi_clicked() {

	DBG();

	// GUI sperren
	this->guiMutex->lock();

	// Alle laufenden Aktionen anhalten
	this->stopLearningThread();
	this->pkt->deactivateNet();
	this->pkt->setModusFromExtern('p');
	if (!this->pkt->running()) this->pkt->start();

	debugOutput ("neural network deactivated");

	this->lg->viLearnWithNN->setEnabled(true);
	this->lg->viLearnWithVI->setEnabled(false);
	this->lg->viLearnWithNN->setChecked(false);
	this->lg->viLearnWithVI->setChecked(true);

	// GUI wieder freigeben
	this->guiMutex->unlock();
}

/**
*
*/
void Simulation::learn_method_vi_nn_clicked() {

	DBG();

	// GUI sperren
	this->guiMutex->lock();

	// Alle laufenden Aktionen anhalten
	this->stopLearningThread();
	this->pkt->activateNet();
	this->pkt->setModusFromExtern('p');
	if (!this->pkt->running()) this->pkt->start();

	this->lg->viLearnWithNN->setEnabled(true);
	this->lg->viLearnWithVI->setEnabled(true);
	this->lg->viLearnWithNN->setChecked(false);
	this->lg->viLearnWithVI->setChecked(false);


	// GUI wieder freigeben
	this->guiMutex->unlock();
}
