#ifndef VALUEITERATION_H
#define VALUEITERATION_H

#include <qthread.h>
#include "SimulationMainWindow.h"
#include "NNetwork.h"


/**
 * Klasse die den Lern-Algorithmus realisiert
 */
class ValueIterationThread : public QObject, public QThread
{
        Q_OBJECT

    public:
	ValueIterationThread(int _x, int _y, double _r, int _g, int _iHidden, int _iHiddenNeurons);
	~ValueIterationThread();
        void run();
        void setModusFromExtern (char _x);
	void setSpeed (int _g);
	char getModus ();
	void unlockMutex();
	void setFeedback(int _row, int _col, double _up, double _down, double _left, double _right);
	void setGamma (double  _ra);
	void deactivateNet();
	void activateNet();
	void setViValue (int _row, int _col, double _value);
	void setNetworkParameters (int _max_epochs, int _iterations_between_report, double _desired_error, double _dEta, bool _auto_error_adjust, double _activation_stepness_hidden, double _activation_stepness_output, int _activation_function_hidden, int _activation_function_output);

	// Erweiterung zum Neuronalen Netz
	void setNetFeedback(int _row, int _col, int _up, int _down, int _left, int _right);
	void setTrainRandomValues(int _trainRandomValues);

    private: 
	double gamma;
	int iteration;

	// Funktionen von Peter
	void printMatrix();
	int getRandomAction();
	int newX(int action, int xOld);
	int newY(int action, int yOld);
	long double evaluate(int action, int _x, int _y);
	void setValues();
	void learn(int _speed, long double _rate);
        void setModus (char _x);
	int trainRandomValues;
	int *randomStates;
	bool auto_adjust_error;

	// Funktionen von Michel
	int getBestDirection(int x, int y);
	bool testBorders(int action, int _x, int _y);

	// Variablen von Peter
	char in, in_ext;	
	int scalX, scalY;
	int steps, speed;
	bool explored;	
	QMutex *printMutex, *inputMutex;

	// ERWEITERUNGEN FUER NEURONALES NETZ
	Network *network;
	bool netOn, calculatetrainValues;
	int iHiddenUnits, iHiddenLayers;
	NETWORKTRAININGRESULTS *network_training_result;

	void initNet();
	void calculateTrainingValues();
	void coachNet();

	bool skipNextPrintMatrix;
	int maxReward, minReward;


    signals:
        void emitValue(int _row, int _column, double _v, int _bestdirection);
	void debugOutput (QString _d);
	void emitNewNetworkError (double _newError);
};


#endif
