/***************************************************************************
 *   Copyright (C) 2009 by Arne Usadel   *
 *   Arne.Usadel@hs-weingarten.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <iostream>
#include <string.h>
#include <qcombobox.h>
#include <qdial.h>
#include <qtable.h>
#include <qstring.h>
#include <qpushbutton.h>
#include <qheader.h>

#include "BusController.h"
#include "serialport_d2xx.h"
#include "serialport_ftdi.h"
#include "connection.h"
#include "packet.h"
#include "ax12.h"

BusController* BusController::singletonInst = NULL;

BusController::BusController()
{
	//connect( BioloidMainWindow::Scan_Bus, SIGNAL( activated() ), this, SLOT( scanBus() ) );
}

BusController::BusController(BioloidMainWindow *mw)
{
	this->mainWin = mw;
	this->activeAX = -1;
	//mainWin->AX_Position->hide();
	connect( mainWin->Scan_Bus, SIGNAL( activated() ), this, SLOT( scanBus() ) );
	connect( mainWin->axactivator, SIGNAL( pressed() ), this, SLOT( activateAX() ) );
	//connect( mainWin->AX_Position_Value_Changed, SIGNAL( ValueChanged(int) ), this, SLOT( ) );
	connect( mainWin->AX_Position, SIGNAL( valueChanged(int) ), this, SLOT( positionChanged(int) ) );
	try {
		pSerialPort = new SerialPort_FTDI();
	} catch ( int e ) {
		qWarning("There was a problem with the usb");
		if (-3 == e)
			qWarning("Is the USB device plugged in?");
		throw e;
	}
	pConn = new Connection(pSerialPort);

	//Timer Stuff for updating the infotable AXTable

	this->pUpdateTimer = new QTimer(this);
	connect(pUpdateTimer, SIGNAL(timeout()), this, SLOT(updateTable()));
	this->pUpdateTimer->start(40);

	//Table View Stuff
	//QTableItem testitem(mainWin->AXTable, QTableItem::Never, "Hi there");
	//mainWin->AXTable->setText(0,0,"Hu Hu");

}

BusController::~BusController()
{
}

BusController* BusController::getSingletonInst()
{
	if ( NULL == singletonInst )
		singletonInst = new BusController();
	return singletonInst;
}

BusController* BusController::getSingletonInst(BioloidMainWindow *mw)
{
	if ( NULL == singletonInst )
		singletonInst = new BusController(mw);
	return singletonInst;
}

void BusController::shutdown()
{
	delete pConn;
	delete pSerialPort;
}

void BusController::scanBus()
{
	//TODO: Scan real usb bus here and populate the QComboBox
	//SerialPort *port = new SerialPort_FTDI();
	//Connection conn(port);

	int cServo = 0;

	QHeader* hRow;
	hRow = mainWin->AXTable->verticalHeader();

	  for (int i = 1; i < 20; i++) {
		AX12 *test = new AX12(i, this->pConn);
		if (test->ping() != NO_ERROR) {
			char* pStringTemp;
			pStringTemp = new char[20];
			snprintf(pStringTemp, 20, "ax%im\0",i);
			QString tHeader(pStringTemp);
			if (0 == cServo) {
				hRow->setLabel(cServo, tHeader, -1);
			} else {
				qWarning("Adding new label");
				mainWin->AXTable->setNumRows(mainWin->AXTable->numRows()+1);
				hRow->setLabel(cServo, tHeader, -1);
			}
			cServo++;
			qWarning("Found new servo motor");
			actuatorList.push_front(i);
			mainWin->axselector->insertItem(pStringTemp, -1);
			delete pStringTemp;
		}
 		delete test;
	}

	if ( 0 == cServo )
		qWarning("No servos found - is the power supply connected?");

	
	//mainWin->axselector->insertItem("AX1",-1);
	//mainWin->axselector->insertItem("AX2",-1);
	//qWarning("scanBus() called - not yet implemented");
	//delete port;
}
 void BusController::activateAX()
{
	
	word position;
	QString * pActualString;

	qWarning(mainWin->axselector->currentText());
	pActualString = new QString(mainWin->axselector->currentText());
	//int test = mainWin->axselector->currentItem();
	if (pActualString->isNull())
		return;
	this->activeAX = mainWin->axselector->currentItem();

	AX12* pAX12 = new AX12(actuatorList[this->activeAX], this->pConn);
	pAX12->getPresentPosition(&position);
	mainWin->AX_Position->setValue( (int)((int)(position)*360/1024) );
	mainWin->AX_Position->setEnabled( TRUE );

	delete pAX12;

	//if (0 == mainWin->axselector->currentItem())
	//	qWarning("Was 0");
	//if (1 == mainWin->axselector->currentItem())setText
	//	qWarning("Was 1");
	

	//AX_Controller* ac1 = new AX_Controller();
	//if ( ac1->isVisible() )
	//	qWarning("Visible");
	//qWarning("activateAX() called - not yet implemented");
}

void BusController::positionChanged(int v)
{
	int normalizedValue = (int)(v * 1023 / 360);
	//SerialPort *port = new SerialPort_FTDI();
	//Connection conn(port);

	AX12* test = new AX12(actuatorList[this->activeAX], this->pConn);
	test->setGoalPosition(normalizedValue);

	delete test;

	//qWarning("Position changed");
}

void BusController::updateTable()
{
	word position, speed;
	byte temp, voltage;

	if (this->activeAX < 0)
		return;

	AX12* pReader = new AX12(actuatorList[this->activeAX], this->pConn);
	pReader->getPresentPosition(&position);
	pReader->getPresentSpeed(&speed);
	pReader->getPresentTemperature(&temp);
	pReader->getPresentVoltage(&voltage);

	//format QString
	char* pStringTemp;
	pStringTemp = new char[20];
	memset(pStringTemp, '\0', 20);
	snprintf(pStringTemp, 20, "%i",(int)position);
	QString po(pStringTemp);
	
	memset(pStringTemp, '\0', 20);
	snprintf(pStringTemp, 20, "%i",(int)speed);
	QString sp(pStringTemp);

	memset(pStringTemp, '\0', 20);
	snprintf(pStringTemp, 20, "%i",(int)temp);
	QString te(pStringTemp);

	memset(pStringTemp, '\0', 20);
	snprintf(pStringTemp, 20, "%i", (int)voltage);
	QString vo(pStringTemp);
	
	delete pStringTemp;


	mainWin->AXTable->setText(this->activeAX, 0, po);
	mainWin->AXTable->setText(this->activeAX, 1, sp);
	mainWin->AXTable->setText(this->activeAX, 2, te);
	mainWin->AXTable->setText(this->activeAX, 3, vo);
	//qWarning("Updated Table");
}