//
// C++ Interface: BoardController
//
// Description: description of the board controller class
//
//
// Author: Michel Tokic <michel@tokic.com>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef _BOARD_CONTROLLER_H_
#define _BOARD_CONTROLLER_H_


#include "global.h"


#define LED_POWER	0x00
#define LED_WALK	0x01
#define LED_LEARN	0x02
#define LED_EXPLORATION	0x03

#define SWITCH_WALK	    		1
#define SWITCH_RANDOM_STATE 		2
#define SWITCH_INV_FB_0	 		3
#define SWITCH_ENABLE_EXPLORATION 	4
#define SWITCH_FULL_EXP_AT_START	5
#define SWITCH_V24			6
#define SWITCH_VDBE_EGREEDY		7
#define SWITCH_GAMMA			8

// possible future pin assignments
#define SWITCH_RAM			9
#define SWITCH_RAM_VALUE		1 // 1:serial RAM (64kB) - 0:internal memoryRAM (2kB)

// serial ports
#define RTS PC3
#define CTS PC2
#define EN_TXD_BIOLOID 	PC7
#define EN_RXD_BIOLOID 	PC6
#define EN_AIR_V24	PC5

// baud rates 
#define BIOLOID_UBRR 1	/* 1Mbps at 16 MHz */
#define V24_UBRR 103 	/* 19.2kbps at 16 MHz */


class BoardController {

	public:
		static void msDelay (int n);
		static void powerOnLED(unsigned char _ledID, bool _status);
		static unsigned char getSwitchState (unsigned char _switch);
		static void blinkLED (unsigned char led_id, unsigned int count, unsigned int delay);

		static void enableSerialBioloidInterface();
		static void enableSerialV24Interface();
		static void disableSerialInterfaces();

		static void initPorts();

		static void enableV24Interrupt();
		static void disableV24Interrupt();
};

#endif
