
#include "FeedbackSensor.h"
#include <avr/interrupt.h>
#include <avr/signal.h>


/** globals for ISR */
//volatile int feedback_value0, feedback_value1;
extern int wheel1, wheel2;


/** initialized the external interrupt */
FeedbackSensor::FeedbackSensor() {


	// set data direction to input
	DDRD &= ~((1<<PD2) | (1<<PD6)); // for second sensor | | (1<<PD3) | (1<<PD7));

	// enable pullups
	PORTD |= (1<<PD2) | (1<<PD6); // for second sensor | (1<<PD3) | (1<<PD7);

	// consider falling and rising edges 
	MCUCR  |= (1<<ISC10) | (1<<ISC00);
	
	// consider only falling edges
	//MCUCR  |= (1<<ISC01) | (1<<ISC11);// | (1<<ISC01) | (1<<ISC11);

	// disable sensor
	this->disableSensor();

	// initialize feedback values
	feedback_inv1 = feedback_inv2 = false;
	wheel1 = wheel2 = 0;
}

/** active external incremental encoder (external interrupt) */
void FeedbackSensor::enableSensor() {

	// reset counters
	wheel1 = wheel2 = 0;

	// set interrupt register flags
	GICR |= (1<<INT0); // for second sensor |  (1<<INT1);
	GIFR |= (1<<INTF0); // for second sensor | (1<<INTF1);
	sei();
}

/**  disable serial interrupt */
void FeedbackSensor::disableSensor() {


	cli();

	// set interrupt register flags
	GICR  &= ~((1<<INT0)); // for second sensor | (1<<INT1));
	GIFR  &= ~((1<<INTF0)); // for second sensor | (1<<INTF1));

}

/** returns the collected reward from both sensors */
int  FeedbackSensor::getFeedback() {

	int _fb = 0;
	
	
	// add sensor0 value
	if (!feedback_inv1) {
		_fb += wheel1;
	} else {
		_fb -= wheel1;
	}

	// add sensor1 value
	if (!feedback_inv2) {

		if (((int)_fb + (int)wheel2) <= 127) {
			_fb += wheel2;
		} else {
			_fb = 127;
		}

	} else {
		if (((int)_fb + (int)wheel2) > -127) {
			_fb -= wheel2;
		} else {
			_fb = -127;
		}
	}

	// reset counters
	wheel1 = wheel2 = 0;


	// check for reward-signal invertion
	if (BoardController::getSwitchState(SWITCH_INV_FB_0)==0) {
		_fb *= -1;
	} 

	// suppress noise
	if (abs(_fb) >= FEEDBACK_THRESHOLD) return _fb;
	else return(0);
}



// Disabled on PCBv2
/** Interrupt-Service-Routine (ISR) for incremental encoder on INT1 */
// SIGNAL(SIG_INTERRUPT1) {
// 
// 	cli();
// 
// 	register unsigned char pd = PIND;
// 
// 	if (( (pd & (1<<PIND7))>=1  &&  (pd & (1<<PIND3))==0 ) ||
// 	    ( (pd & (1<<PIND7))==0  &&  (pd & (1<<PIND3))>=1 ) 
// 	) {
// 		if (feedback_value1 < 127) {
// 			feedback_value1++;
// 		}
// 
// 	} else {	
// 		if (feedback_value1 > -127) {
// 			feedback_value1--;
// 		}	
// 	}
// 
// 	sei();
// }







