#include "Ram.h"

/** saves a state into the memory */
void Ram::writeBytes(uint16_t _addr, unsigned char *_data, unsigned char _length) {

	unsigned char *bytes;
	register unsigned char i;
	unsigned char slave = 0xA0;
	
	// send start-condition 
	i2c->sendStart();
	
	// FM24C512 specific addressing
	i2c->writeByte( slave ); 
	
	bytes = (unsigned char*)&_addr;
	i2c->writeByte(bytes[0]);
	i2c->writeByte(bytes[1]);
	
	// send data
	for (i=0; i<_length; i++) {
		i2c->writeByte(_data[i]);
	}
	
	// send stop condition
	i2c->sendStop();
}

/** read a state from the memory */
void Ram::readBytes(uint16_t _addr, unsigned char *_data, unsigned char _length) {


	unsigned char *bytes;
	register unsigned char i=0;
	unsigned char slave = 0xA0;

	// send start-condition 
	i2c->sendStart();
	
	// FM24C512 specific addressing
	i2c->writeByte( slave ); 
		
	// send address
	bytes = (unsigned char*)&_addr;
	i2c->writeByte(bytes[0]);
	i2c->writeByte(bytes[1]);
	i2c->sendStart();
	i2c->writeByte(0xA1);
	
	// send data
	for (i=0; i<(_length-1); i++) {
		_data[i] = i2c->readByte();
	}
	_data[i] = i2c->readByteWithoutAck();
	
	
	// send stop condition
	i2c->sendStop();
}


/** initialize the memory module */
Ram::Ram() {

	//initialize SPI bus
	i2c = new I2c();
	
}


/** destructor */
Ram::~Ram() {
	delete i2c;
}

/**
 * Diese Funktion, setzt die Chip-Select Leitung
 * auf low (aktiv) und aktiviert somit das RAM 
 * Zusätzlich wird Write-protect ausgeschaltet.
 */
void Ram::enableRAMChip() {

	// Chip-Select auf Low
// 	PORTA &= ~((1<<PA6));
// 	PORTA |= (1<<PA1) | (1<<PA0);
}

/**
 * Diese Funktion, setzt die Chip-Select Leitung
 * auf high (inaktiv) und deaktiviert somit das
 * RAM. Zusätzlich wird Write-protect ausgeschaltet.
 */
void Ram::disableRAMChip() {

	// Chip-Select auf High
// 	PORTA |= ((1<<PA6));
// 	PORTA |= (1<<PA1) | (1<<PA0);
}


/** Funktion liefert das RAM Statusregister zurück */
unsigned char Ram::getRamStatusRegister() {
/*
	unsigned char _sr = 0x00;

	// SPI Schnittstelle aktivieren
	spi.enableSPI();	

	// RAM Chip aktivieren 
	this->enableRAMChip();

	// Kommando schicken
	spi.writeData(RAM_RDSR);
	
	// Antwort auslesen
	_sr = spi.readData();
	
	// RAM Chip wieder deaktivieren
	this->disableRAMChip();

	// SPI wieder disabeln 
	spi.disableSPI();

	return _sr;*/
}

/** Funktion setzt das RAM Statusregister auf _sr  */
void Ram::setRamStatusRegister(unsigned char _sr) {

// 	// SPI Schnittstelle aktivieren
// 	spi.enableSPI();
// 
// 	// Write-Enable Kommando schicken
// 	this->enableRAMChip();
// 	spi.writeData(RAM_WREN);
// 	this->disableRAMChip();
// 
// 	// Write-Status-Register Kommando schicken
// 	this->enableRAMChip();
// 	spi.writeData(RAM_WRSR);
// 	spi.writeData(_sr);
// 	this->disableRAMChip();
// 
// 	// SPI wieder disabeln 
// 	spi.disableSPI();
}
