#ifndef __SENSORONLYMODE_H__
#define __SENSORONLYMODE_H__ 1

#include "FeedbackSensor.h"

#define MAX_PARAM       8
#define MAX_PACKELEN    MAX_PARAM+4
#define BROADCAST_ID    0xFE
#define SELF_ID         200

//--- ATMEGA ---
#define ENABLE_BIT_DEFINITIONS 
#define cbi(REG,BITNUM) REG &= ~(_BV(BITNUM)) // clear bit
#define sbi(REG,BITNUM) REG |= _BV(BITNUM)    // set bit

#define WORD_MAX_SIZE  32767
#define WORD_MIN_SIZE -32767

// convert a word into 2 bytes with sign
#define highbyte_s(w) (w&0x80000000?(byte)(w >> 8)|(0x8000):(byte)(w >> 8))

//--- ERRORCODES ---
#define NO_ERROR        0
#define E_INPUT_VOLTAGE 1
#define E_ANGLE_LIMIT   2
#define E_OVERHEATING   4
#define E_RANGE         8
#define E_CHECKSUM      16
#define E_OVERLOAD      32
#define E_INSTRUCTION   64

//---INSTRUCTIONS ---
#define INST_PING           0x01
#define INST_READ           0x02
#define INST_WRITE          0x03
#define INST_REG_WRITE      0x04
#define INST_ACTION         0x05
#define INST_RESET          0x06
#define INST_DIGITAL_RESET  0x07
#define INST_SYSTEM_READ    0x0C
#define INST_SYSTEM_WRITE   0x0D
#define INST_SYNC_WRITE     0x83
#define INST_SYNC_REG_WRITE 0x84

//--FLASH--
#define P_DIST 1

typedef unsigned char byte;

/***********************************************************************
* PACKET
***********************************************************************/

typedef struct
{
  byte id;
  byte len;
  byte cmd;
  byte paramPtr;
  byte param[ MAX_PARAM ];
  byte crc; 
} Packet;

/***********************************************************************
* STATEMACHINE
***********************************************************************/

#define STATE_IDLE          0   
#define STATE_1ST_FF_RCVD   1  
#define STATE_2ND_FF_RCVD   2   
#define STATE_ID_RCVD       3  
#define STATE_LENGTH_RCVD   4   
#define STATE_COMMAND_RCVD  5   
#define STATE_PKT_COMPLETE  6   

typedef struct 
{
  byte id;
  byte state;
  Packet pkt;
  void (*callback)(Packet *pkt);
} Statemachine;




/***********************************************************************
* SMARTBUFFER
***********************************************************************/

#define init(sb)        (sb.readPtr = sb.writePtr = 0)
#define isEmpty(sb)     (sb.writePtr == sb.readPtr)
#define push(sb, value) (sb.buffer[ sb.writePtr++ ] = value)
#define pop(sb)         (sb.buffer[ sb.readPtr++ ])
#define len(sb) ( sb.readPtr > sb.writePtr ? (sb.writePtr - sb.readPtr + 256) : (sb.writePtr - sb.readPtr) )

typedef struct {
  volatile byte  readPtr;
  volatile byte  writePtr;
  byte     buffer[ 256 ];
} Smartbuffer;




/***********************************************************************
* MAIN
***********************************************************************/

#define RS485_TXD sbi(PORTD, PD5), cbi(PORTD, PD6), cbi(UCSRB,RXCIE)
#define RS485_RXD sbi(PORTD, PD6), cbi(PORTD, PD5), sbi(UCSRB,RXCIE)

#define SET_TXD_FINISH   sbi(UCSRA,TXC)
#define RESET_TXD_FINISH cbi(UCSRA,TXC)
#define CHECK_TXD_FINISH bit_is_set(UCSRA,TXC)

#define TXD_READY			bit_is_set(UCSRA,UDRE)
#define TXD_DATA			UDR
#define RXD_DATA			UDR

#define READDIST_X  TCNT0
#define READDIST_Y  TCNT1
#define RESET_MOUSE TCNT0 = TCNT1 = 0
#define RESET_WHEEL wheel1 = wheel2 = 0
#define RESET_SENSOR RESET_WHEEL

//#define XTAL 11059000
//#define BAUD 9600

#define XTAL 16000000
#define BAUD 1000000


int sensorMain (void);

#endif 
