#include "StateSpace.h"


/** constructor function. */
StateSpace::StateSpace(Environment* env) {

	// use internal memory instead of FM24C256
	this->useInternalRAM = true;

	// init ram
	ram1 = new Ram();

	// 
	ENV_POS d, pos;
	env->getDiscretization(&d);
	internStateSpace = new unsigned char[d.x * d.y];

	// set's the environment
	this->env = env;

	// initialize values with zeros
	STATE tmpState;

 	for (int _row=0; _row < d.y; _row++) {
 		for (int _column=0; _column<d.x; _column++) {
 			
 				pos.x = _column;
 				pos.y = _row;
 
 				tmpState.value = 0.0; 
 
 				tmpState.feedback_up = 0;
 				tmpState.feedback_down = 0;
 				tmpState.feedback_left = 0;
 				tmpState.feedback_right = 0;
 			
 				setState (&pos, &tmpState);
 		}
 	}
}

/** destructor */
StateSpace::~StateSpace() {
	delete ram1;
	delete[] internStateSpace;
}


/** returns a state into _state */
void StateSpace::getState (ENV_POS* pos, STATE *_state) {

	unsigned int _addr;
	unsigned char i;
	unsigned char* _bytes = (unsigned char*)_state;

	ENV_POS d;
	env->getDiscretization(&d);

	// compute memory address
	_addr = ((pos->y * d.x) + pos->x)*STATESIZE;

	// internal RAM
	if (this->useInternalRAM) {
		for (i=0; i<STATESIZE; i++) {
			_bytes[i] = internStateSpace[(_addr + i)];
		}

	// SPI memory
	} else {
		ram1->readBytes(_addr, (unsigned char*)&_state[0], STATESIZE);
	}
}

/** update state (_row, _column) with _state*/
void StateSpace::setState (ENV_POS* pos, STATE *_state) {

	unsigned int _addr;
	unsigned char i;
	unsigned char* _bytes = (unsigned char*)_state;

	ENV_POS d;
	env->getDiscretization(&d);

	// compute memory address
	_addr = ((pos->y * d.x) + pos->x)*STATESIZE;

	// internal RAM
	if (this->useInternalRAM) {
		for (i=0; i<STATESIZE; i++) {
			internStateSpace[_addr + i] = _bytes[i];
		}

	// SPI Memory
	} else {
		ram1->writeBytes(_addr, (unsigned char*)&_state[0], STATESIZE);
	}
}

/** set Value in state (_row, _column) */
void StateSpace::setValue(ENV_POS* pos, float _value) {

	STATE _s;
	this->getState ( pos, &_s );
	_s.value = _value;
	this->setState ( pos, &_s );
}

/** set reward r(s,_action)=_fb in state (_row, _column) */
void StateSpace::setFeedback(ENV_POS* pos, unsigned char _action, char _fb) {

	STATE _s;
	this->getState (pos, &_s);

	if (_action == UP) {
		_s.feedback_up = _fb;
	} else if (_action == DOWN) {
		_s.feedback_down = _fb;
	} else if (_action == LEFT) {
		_s.feedback_left = _fb;
	} else if (_action == RIGHT) {
		_s.feedback_right = _fb;
	}

	this->setState (pos, &_s);
}

/** returns the Value from state (_row, _column) */
double StateSpace::getValue(ENV_POS* pos) {
	STATE _s;
	this->getState (pos, &_s);
	return _s.value;
}

/** returns r(s,a) from state (_row, _column) */
char StateSpace::getFeedback (ENV_POS* pos, unsigned char _action) {

	STATE _s;
	this->getState (pos, &_s);

	if (_action == UP) {
		return _s.feedback_up;
	} else if (_action == DOWN) {
		return _s.feedback_down;
	} else if (_action == LEFT) {
		return _s.feedback_left;
	} else if (_action == RIGHT) {
		return _s.feedback_right;
	}
}


/** RAM-Test function (e.g. writes random numbers from *data to memory)  */
void StateSpace::ramtestSaveBytes(unsigned int _addr, unsigned char *data, unsigned char _length) {

	unsigned char* statesPtr = (unsigned char*)&internStateSpace[0];
	unsigned char i;

	if (this->useInternalRAM) {
		for (i=0; i<_length; i++) {
			statesPtr[_addr + i] = data[i];
		}

	} else {
		ram1->writeBytes(_addr, data, _length);
	}
}

/** RAM-Test function (e.g. reads random numbers into *data from memory)  */
void StateSpace::ramtestReadBytes(unsigned int _addr, unsigned char *data, unsigned char _length) {

	unsigned char* statesPtr = (unsigned char*)&internStateSpace[0];
	unsigned char i;

	if (this->useInternalRAM) {
		for (i=0; i<_length; i++) {
			data[i] = statesPtr[_addr + i];
		}

	} else {
		ram1->readBytes(_addr, data, _length);
	}
}

/** returns the memory-size of the state-space */
unsigned int StateSpace::getMemorySize() {

	// Interner RAM
	if (this->useInternalRAM) {
		return sizeof(internStateSpace);

	// SPI RAM
	} else {
		return 65535;
	}
}

