#ifndef __GLOBAL_H__
#define __GLOBAL_H__

#include <avr/io.h>
#include <avr/interrupt.h>
#include <math.h>
#include <stdlib.h>


//#define TESTMODE 1



/********************
 * global constants *
 ********************/
/** 
 * CPU clock
 */
//#define F_CPU 	11059200UL
#define F_CPU 		16000000UL

#include <avr/delay.h>

/** 
 * Maximum size of statespace (amount of states) 
 */
#define MAX_STATES 200

/** 
 * time that a servo needs for the movement from LEFT to RIGHT
 */
//#define DELAY_TIME 1500L
#define DELAY_TIME 300L

/**
 * Action Set
 */
enum ActionSet {UP, DOWN, LEFT, RIGHT, NONE};
#define NUM_ACTIONS 0x04

/**
 * Software mode
 */
enum SoftwareMode{SENSOR_MODE, ROBOT_STANDALONE};

/**
 * state structure
 */
typedef struct State {
	float value;
	//unsigned char best_action;
	char feedback_up;
	char feedback_down;
	char feedback_left;
	char feedback_right;
} STATE;
/**
 * determine the amount of memory-bytes which a state requires
 */
#define STATESIZE sizeof(State)

/**
	Exploration Modes
*/
#define EXPLORATION_EGREEDY 0
#define EXPLORATION_VALUE_BASED 1


/**
  Main modes
*/
enum mainModes {
	MAIN_MODE_WALK_POLICY,
	MAIN_MODE_EXPLORE_FEEDBACK,
	MAIN_MODE_LEARN,
	MAIN_MODE_STOP,
	MAIN_MODE_UNCHANGED
};

/**
 * defines a state in the environment
 */
typedef struct envPosition {
	unsigned int x;
	unsigned int y;

	envPosition(unsigned int _x, unsigned int _y) {
		this->x = _x;
		this->y = _y;
	}
	envPosition(){};
} ENV_POS;


/** 
	support for new- and delete-operators
*/
void * operator new(size_t size);
void operator delete(void * ptr); 
void * operator new[](size_t size);
void operator delete[](void * ptr); 


#endif

