#include "V24Control.h"
#include "SensorOnlyMode.h"



//#include "test.h"

volatile int softwareMode = ROBOT_STANDALONE; // for interrupt sharing

/** the main function */
int main( void )
{	



	// initialize ports+LEDs
	BoardController::initPorts();

	// software mode
	if (0) {
		softwareMode = SENSOR_MODE;
		sensorMain();	
	} else {
		softwareMode = ROBOT_STANDALONE;
	}


	// initialize serial communication class with standard VI agent
	V24Control *v24 = new V24Control();
	ValueIteration *vi = v24->vi;

#ifdef TESTMODE
#warning compiled TESTMODE!



	//BoardController::blinkLED((1<<LED_POWER)|(1<<LED_WALK)|(1<<LED_LEARN)|(1<<LED_EXPLORATION), 20, 200);

	// initialize serial communication class with standard VI agent
	//V24Control *v24 = new V24Control();
	//ValueIteration *vi = v24->vi;



	/******************************************
	* Test Mode
	******************************************/


	BoardController::enableSerialV24Interface();

	ACTION_RETURN aRet;

	while (1) {
		for (unsigned char i=48; i<55; i++) {
			v24->transmit(i);
			BoardController::msDelay(500);
		}
	}

	unsigned char sByte=0;
	while (1) {
		if (v24->newDataReceived()) {

			BoardController::powerOnLED(LED_LEARN, true);
			//BoardController::msDelay(1000);	

			v24->receive(&sByte);
			//PORTC |= (1<<RTS);

			//sByte++;
			//PORTC |= (1<<RTS);
			//v24->transmit(sByte);

			// LEFT
			if (sByte == 'a') {
				vi->env->doAction(LEFT, 1000, &aRet);
				v24->transmit('b');
			// RIGHT
			} else if (sByte == 's') {
				vi->env->doAction(RIGHT, 1000, &aRet);
				v24->transmit('t');
			// UP
			} else if (sByte == 'd') {
				vi->env->doAction(UP, 1000, &aRet);
				v24->transmit('e');
			// DOWN
			} else if (sByte == 'f') {
				vi->env->doAction(DOWN, 1000, &aRet);
				v24->transmit('g');
			}

			// indicate that controller is again ready for reception

			BoardController::powerOnLED(LED_LEARN, false);
		}

		
		/*
		for (unsigned char i=33; i<125; i++) {
			v24->transmit(i);
		}
		
		v24->transmit(0x30);
		BoardController::msDelay(1000);
		*/

		/*
		BoardController::blinkLED (
			(1<<LED_WALK)|(1<<LED_LEARN)|(1<<LED_EXPLORATION), 1, 500
		);
		*/

		//PORTC |= (1<<RTS);
		//v24->transmit(0x30);
		//BoardController::msDelay(1000);
		PORTC &= ~(1<<RTS);

	}

#else 

	/******************************************
	* NORMAL Mode
	******************************************/


	// initialize auto start counter 
	unsigned char autoStartCounter = 0;
	bool autoStart = 1;

	int mainMode = MAIN_MODE_WALK_POLICY;
	int newMainMode = MAIN_MODE_WALK_POLICY;

	while ( 1 ) {	

		// set new mode in case of interrupt
		mainMode = newMainMode;

		// only walk when PAUSE 
		if (BoardController::getSwitchState(SWITCH_WALK)) {

			// Check for new serial data
			if (v24->newDataReceived()) {

				// stop robot
				BoardController::disableV24Interrupt();
				cli();
				newMainMode = MAIN_MODE_STOP; 
	
				v24->processCommand();
				autoStart = 0;

				newMainMode = v24->getNewMainMode(newMainMode);
				
		
			// Autostart-Routine
			} else if (autoStart) {

				// delay for at least 5 seconds
				if (autoStartCounter < 5) {

					autoStartCounter++;
	
					BoardController::blinkLED (
					  (1<<LED_WALK)|(1<<LED_LEARN)|(1<<LED_EXPLORATION), 1, 500);
			
				// start after 5 seconds
				} else {

					// disable auto-start
					autoStart = false;
					
					// explore environment if SWITCH_FULL_EXP_AT_START is enabled
					if (BoardController::getSwitchState(SWITCH_FULL_EXP_AT_START)) {
						vi->exploreAllFeedbacks(false);
					}
					BoardController::enableV24Interrupt();
							
					newMainMode = MAIN_MODE_WALK_POLICY;
				}

			// do next action
			} else if (mainMode == MAIN_MODE_WALK_POLICY) {

				vi->doNextAction();

			// explore environment
			} else if (mainMode == MAIN_MODE_EXPLORE_FEEDBACK) {
	
				vi->exploreAllFeedbacks(true);
	
				// perform value-iteration to learn the modell
				newMainMode = MAIN_MODE_LEARN;
	
			// learning-only mode
			} else if (mainMode == MAIN_MODE_LEARN) {

				// value iterate all states
				vi->valueIterateAllStates();	
				
			// pause
			} else if (mainMode == MAIN_MODE_STOP) {
	
				// dummy
			}

		// if robot is in PAUSE-mode, only check for new serial data
		} else {
			// check for new serial data
			if (v24->newDataReceived()) {

				// stop robot
				newMainMode = MAIN_MODE_STOP; 
	
				// process serial command
				v24->processCommand();
	
				// update main mode
				newMainMode = v24->getNewMainMode(mainMode);

			// otherwise perform value-iteration
			} else if (mainMode == MAIN_MODE_LEARN) {
				vi->valueIterateAllStates();
			}
		}
	}
#endif


}


