#ifndef ROBOTER_H_
#define ROBOTER_H_

//#include "global.h"
#include "Servo.h"
#include "StateSpace.h"

/***************************
 * modes for main function *
 ***************************/
#define MAIN_MODE_STOP			0	// do nothing
#define MAIN_MODE_WALK_POLICY		1	// walk current policy
#define MAIN_MODE_EXPLORE_FEEDBACK	2	// explore environment
#define MAIN_MODE_READ_V24		3	// process serial command 
#define MAIN_MODE_LEARN			4	// learning only mode (without walking)


#define LED_POWER	0x00
#define LED_WALK	0x01
#define LED_LEARN	0x02
#define LED_EXPLORATION	0x03

#define SWITCH_WALK	    		1
#define SWITCH_RANDOM_STATE 		2
#define SWITCH_INV_FB_0	 		3
#define SWITCH_ENABLE_EXPLORATION 	4
#define SWITCH_FULL_EXP_AT_START	5
#define SWITCH_V24			6
#define SWITCH_VDBE_EGREEDY		7
#define SWITCH_GAMMA			8

// possible future pin assignments
#define SWITCH_RAM			9
#define SWITCH_RAM_VALUE		1 // 1:serial RAM (64kB) - 0:internal memoryRAM (2kB)
//#define SWITCH_V24			8


/************************
 * function prototypes  *
 ************************/
struct Controller {
	public:
		void init();
		void msDelay (int n);
		void powerOnLED(unsigned char _ledID, bool _status);
		void setDefaultPolicy();
		void gotoStateWithoutSavingFeedback(unsigned char _row, unsigned char _column);
		unsigned char getSwitchState (unsigned char _switch);
		void blinkLED (unsigned char led_id, unsigned int count, unsigned int delay);

		// Servos
		Servo servo_x;
		Servo servo_y;
		Servo servo_z;

		// The state space
		StateSpace states;

		// the position memory
		unsigned char currentX, currentY;

		// modes for main function
		unsigned char newMainMode;
		unsigned char mainMode;
		
		// Autostart
		bool autoStart;

		void enableV24Interrupt();
		void disableV24Interrupt();

	private:
		void initLEDs();

};


#endif /*ROBOTER_H_*/
