
#include "FeedbackSensor.h"
#include <avr/interrupt.h>
#include <avr/signal.h>


/** globals for ISR */
volatile int feedback_value0, feedback_value1;


/** initialized the external interrupt */
void FeedbackSensor::init() {


	// set data direction to input
	DDRD &= ~((1<<PD2) | (1<<PD6)); // for second sensor | | (1<<PD3) | (1<<PD7));

	// enable pullups
	PORTD |= (1<<PD2) | (1<<PD6); // for second sensor | (1<<PD3) | (1<<PD7);

	// consider falling and rising edges 
	MCUCR  |= (1<<ISC10) | (1<<ISC00);
	
	// consider only falling edges
	//MCUCR  |= (1<<ISC01) | (1<<ISC11);// | (1<<ISC01) | (1<<ISC11);

	// disable sensor
	this->disableSensor();

	// initialize feedback values
	feedback_inv0 = feedback_inv1 = false;
	feedback_value0 = feedback_value1 = 0;
}

/** active external incremental encoder (external interrupt) */
void FeedbackSensor::enableSensor() {

	// reset counters
	feedback_value0 = feedback_value1 = 0;

	// set interrupt register flags
	GICR |= (1<<INT0); // for second sensor |  (1<<INT1);
	GIFR |= (1<<INTF0); // for second sensor | (1<<INTF1);
	sei();
	
}

/**  disable serial interrupt */
void FeedbackSensor::disableSensor() {


	cli();

	// set interrupt register flags
	GICR  &= ~((1<<INT0)); // for second sensor | (1<<INT1));
	GIFR  &= ~((1<<INTF0)); // for second sensor | (1<<INTF1));

}

/** returns the collected reward from both sensors */
char  FeedbackSensor::getFeedback() {

	int _fb = 0;
	
	
	// add sensor0 value
	if (!feedback_inv0) {
		_fb += feedback_value0;
	} else {
		_fb -= feedback_value0;
	}

	// add sensor1 value
	if (!feedback_inv1) {

		if (((int)_fb + (int)feedback_value1) <= 127) {
			_fb += feedback_value1;
		} else {
			_fb = 127;
		}

	} else {
		if (((int)_fb + (int)feedback_value1) > -127) {
			_fb -= feedback_value1;
		} else {
			_fb = -127;
		}
	}

	// reset counters
	feedback_value0 = feedback_value1 = 0;

	// suppress noise
	if (abs(_fb) >= FEEDBACK_THRESHOLD) return _fb;
	else return(0);
}

/** invert a feedback sensor */
void FeedbackSensor::invertSensor(unsigned char _sensorId, bool _state) {

	if (_sensorId == FEEDBACK_SENSOR_0) {
		feedback_inv0 = _state;
	} else if (_sensorId == FEEDBACK_SENSOR_1) {
		feedback_inv1 = _state;
	} 
}

/** Interrupt-Service-Routine (ISR) for incremental encoder on INT0 */
SIGNAL(SIG_INTERRUPT0) {

	cli();

	register unsigned char pd = PIND;

	if (( (pd & (1<<PIND6))>=1  &&  (pd & (1<<PIND2))==0 ) ||
	    ( (pd & (1<<PIND6))==0  &&  (pd & (1<<PIND2))>=1 ) 
	) {
		if (feedback_value0 < 127) {
			feedback_value0++;
		}

	} else {	
		if (feedback_value0 > -127) {
			feedback_value0--;
		}	
	}

	sei();
}



// Disabled on PCBv2
/** Interrupt-Service-Routine (ISR) for incremental encoder on INT1 */
// SIGNAL(SIG_INTERRUPT1) {
// 
// 	cli();
// 
// 	register unsigned char pd = PIND;
// 
// 	if (( (pd & (1<<PIND7))>=1  &&  (pd & (1<<PIND3))==0 ) ||
// 	    ( (pd & (1<<PIND7))==0  &&  (pd & (1<<PIND3))>=1 ) 
// 	) {
// 		if (feedback_value1 < 127) {
// 			feedback_value1++;
// 		}
// 
// 	} else {	
// 		if (feedback_value1 > -127) {
// 			feedback_value1--;
// 		}	
// 	}
// 
// 	sei();
// }







