#include "Servo.h"

/** assign a timer to the servo (Timer0, Timer1A or Timer1B)  */
void Servo::setTimer(char _timer) { 
	this->timer = _timer;
	this->initTimer();
}


/**
 *  PWM Timer init
 *
 *  Pins:	PB3 - PWM from Timer0
 *		PD5 - PWM from Timer1 - CompareA
 * 		PD4 - PWM from Timer1 - CompareB
 */
void Servo::initTimer(void)
{

	unsigned int intValue;

	// *******************************************************************
	// Timer init: 8-bit, normal pwm, prescale 1024
	// *******************************************************************
	if (this->timer == TIMER0) {

		TCCR0  = (1<<WGM00)|(1<<COM01) |(1<<CS02) | (1<<CS00);

		maxUp = T0_MAX_UP;
		maxDown = T0_MAX_DOWN;
		servoMiddle = T0_SERVO_MIDDLE;

	// *******************************************************************
	// Timer1 init: 16-bit, normal pwm, prescale 8
	// *******************************************************************
	} else if (this->timer == TIMER1A || this->timer == TIMER1B) {

		TCCR1A = (1<<WGM11) | (1<<COM1A1) |(1<<COM1B1);
		TCCR1B = (1<<CS11) | (1<<WGM12) |(1<<WGM13);

		maxUp = T1_MAX_UP;
		maxDown = T1_MAX_DOWN;
		servoMiddle = T1_SERVO_MIDDLE;

		// at period end start from 0 again
		ICR1 = T1_T_PERIODE;
	}


	// ********************
	// configure PWM Pins *
	// ********************
	if (this->timer == TIMER0) {

		// PWM Port
		DDRB |= (1<<PB3);
		OCR0 = (unsigned char)(maxDown);

	} else if (this->timer == TIMER1A) {

		DDRD |= (1<<PD5);

		// initial position
		OCR1A = maxUp;

	} else if (this->timer == TIMER1B) {

		DDRD |= (1<<PD4);

		// initial position
		OCR1B = maxDown;
	}
}

/** position servo */
void Servo::setPosition(char _pos, char _scal) {

	unsigned int intValue;

	// invert position
	_pos = (_scal-1)-_pos;

	// check for valid action
	if (_pos <0 || _pos >= _scal) {
		return;

	// position servo
	} else {
	
		// PWM: compute new timer value
		intValue = (unsigned int)(maxDown + (_pos * (float)((float)(maxUp-maxDown)/(_scal-1))));	
		
		// PWM: set new value to timer register
		if (this->timer == TIMER0) {
			OCR0 = (unsigned char)(intValue%256);
		} else if (this->timer == TIMER1A) {
			OCR1A = intValue;
		} else if (this->timer == TIMER1B) {
			OCR1B = intValue;
		}
	}

}

