#include "BioloidServo.h"

/**
 * set the Id, minimum position and maximum position of servo (Id)
 */
/*BioloidServo::BioloidServo(unsigned char _id, unsigned int _minPos, unsigned int _maxPos)
{
	this->minPos = _minPos;
	this->maxPos = _maxPos;
	this->servoId = _id;

	BoardController::enableSerialBioloidInterface();

	// configure AX-12: send no return messages 
	this->txPacket.id = this->servoId; 	// Broadcast
	this->txPacket.paramLen = 2;		
	this->txPacket.instruction = 0x03; 	// WRITE
	this->txPacket.param[0] = 0x10; 	// set status return level
	this->txPacket.param[1] = 0x00;		// send no response
	this->TxPacket(&this->txPacket, 100);

	// enable torque
	this->txPacket.id = this->servoId; //this->servoId;
	this->txPacket.paramLen = 2;		// 1x address (parameter0), 2x position(parameter1-2)
	this->txPacket.instruction = 0x03; 	// WRITE_DATA
	this->txPacket.param[0] = 0x18; 	// set torque 
	this->txPacket.param[1] = 0x01;		// enable
	this->TxPacket(&this->txPacket, 0);

	BoardController::disableSerialInterfaces();
}*/

BioloidServo::BioloidServo(unsigned char _id, unsigned int _minPos, unsigned int _maxPos)
{

	BioloidPacket txPacket;

	this->minPos = _minPos;
	this->maxPos = _maxPos;
	this->servoId = _id;

	//this->servoId = this->getServoId(_id);

	BoardController::enableSerialBioloidInterface();

	// configure AX-12: send no return messages 
	txPacket.id = this->servoId; 	// Broadcast
	txPacket.paramLen = 2;		
	txPacket.instruction = 0x03; 	// WRITE
	txPacket.param[0] = 0x10; 	// set status return level
	txPacket.param[1] = 0x00;		// send no response
	TxPacket(&txPacket, 100);

	// enable torque
	txPacket.id = this->servoId; //this->servoId;
	txPacket.paramLen = 2;		// 1x address (parameter0), 2x position(parameter1-2)
	txPacket.instruction = 0x03; 	// WRITE_DATA
	txPacket.param[0] = 0x18; 	// set torque 
	txPacket.param[1] = 0x01;		// enable
	TxPacket(&txPacket, 0);

	BoardController::disableSerialInterfaces();
}

/*
unsigned char BioloidServo::getServoId(unsigned char _id) {

	BoardController::enableSerialBioloidInterface();
	BioloidPacket txPacket;

	// configure AX-12: send no return messages 
	txPacket.id = 0xff; 	// Broadcast
	txPacket.paramLen = 2;		
	txPacket.instruction = 0x03; 	// WRITE
	txPacket.param[0] = 0x10; 	// set status return level
	txPacket.param[1] = 0x00;		// send no response
	TxPacket(&txPacket, 100);



	// By definition, the servo with the highest id is SERVO_X

	// By definition, the servo with the lowest id is SERVO_Y

	BoardController::disableSerialInterfaces();
}
*/



/** position servo */
void BioloidServo::setPosition(unsigned char _pos, unsigned char _scal) {

	unsigned int posValue;
	unsigned int speed=1023;
	unsigned int desiredPosition=0;

	BioloidPacket txPacket;

	BoardController::enableSerialBioloidInterface();

	// invert axis position
	//if (this->servoId == 0x11)
		desiredPosition = (_scal-1)-_pos;
	//else
		//desiredPosition = _pos;

	posValue = minPos + (((maxPos-minPos) / (unsigned int)(_scal-1)) * (unsigned int)desiredPosition);
	//posValue = 0;

	txPacket.id = this->servoId;
	txPacket.paramLen = 5; // 1x address (parameter0), 2x position(parameter1-2)
	txPacket.instruction = 0x03; // WRITE_DATA
	txPacket.param[0] = 0x1e; 	// set Goal Position
	txPacket.param[1] = lowbyte(posValue);  // 1) low byte
	txPacket.param[2] = highbyte(posValue); // 2) high byte
	txPacket.param[3] = lowbyte(speed);//lowbyte(posValue);  // 1) low byte
	txPacket.param[4] = highbyte(speed); //highbyte(posValue); // 2) high byte

	TxPacket(&txPacket, 10);

	// enable torque
	txPacket.id = this->servoId; //this->servoId;
	txPacket.paramLen = 2;		// 1x address (parameter0), 2x position(parameter1-2)
	txPacket.instruction = 0x03; 	// WRITE_DATA
	txPacket.param[0] = 0x18; 	// set torque 
	txPacket.param[1] = 0x01;		// enable
	TxPacket(&txPacket, 10);

	BoardController::disableSerialInterfaces();
}

/* This functions transmit a bioloid-packet */
void BioloidServo::TxPacket(BioloidPacket* bPacket, unsigned int milliDelay) 
{
	unsigned char bCount, bPacketLength, bCheckSum;
	unsigned char i;
	unsigned char dummy;
	static unsigned char txbuff[ BIOLOID_MAX_PACKETLEN ];


	// send packet - expects serial interface to be proper configured
	PORTC  &= ~((1<<EN_RXD_BIOLOID) | (1<<EN_TXD_BIOLOID) | (1<<EN_AIR_V24));
	PORTC  |= ((1<<EN_TXD_BIOLOID) | (1<<EN_RXD_BIOLOID));
	BoardController::msDelay(1);
	

	txbuff[0] = 0xff;
	txbuff[1] = 0xff;
	txbuff[2] = bPacket->id; 
	txbuff[3] = bPacket->paramLen+2; // Length := instruction + paramter's + checksum
	txbuff[4] = bPacket->instruction;

	// compute packet length
	bPacketLength = bPacket->paramLen+4+2;
 
	// append parameters
	for(bCount = 0; bCount < bPacket->paramLen; bCount++)
		txbuff[bCount+5] = bPacket->param[bCount];
 
	// compute checksum 
	bCheckSum = 0;
	for(bCount = 2; bCount < bPacketLength-1; bCount++) // except 0xff, checksum
		bCheckSum += txbuff[bCount];
 	// Append checksum with bit inversion
	txbuff[bCount] = ~bCheckSum;


	for(bCount = 0; bCount < bPacketLength; bCount++)
	{
		while (!(UCSRA & (1<<UDRE)));
		UDR = txbuff[bCount];
		while(!(UCSRA & (1<<TXC)));
	}
	//BoardController::msDelay(1);

	// enable bioloid reception
	//PORTC  &= ~((1<<EN_RXD_BIOLOID) | (1<<EN_TXD_BIOLOID) | (1<<EN_AIR_V24));
	PORTC  |= (1<<EN_RXD_BIOLOID);
	
	BoardController::msDelay(milliDelay);
}

/* This functions receives a bioloid-packet */
unsigned char BioloidServo::RxPacket(BioloidPacket* bPacket) 
{
	unsigned char i;
	unsigned char dummy;


	// enable reception
	PORTC  &= ~(1<<EN_TXD_BIOLOID);
	PORTC  |= (1<<EN_RXD_BIOLOID);


	// receive garbage
	if (!RxByte(&dummy)) {	
		BoardController::blinkLED((1<<LED_WALK), 10, 20);
		return 0;
	}
	if (!RxByte(&dummy)) {	
		BoardController::blinkLED((1<<LED_LEARN), 10, 20);
		return 1;
	} 

	// receive id
	if (!RxByte(&bPacket->id)) {	
		BoardController::blinkLED((1<<LED_EXPLORATION), 10, 20);
		return 2;
	} 

	if (!RxByte(&bPacket->paramLen)) return 3;

	// receive data
	for (i=0; i<bPacket->paramLen; i++) {
		// receive data
		if (!RxByte(&bPacket->param[i])) return 4+i;
	}
}

/* function receives a byte from the bioloid bus */
bool BioloidServo::RxByte(unsigned char* data) {

	//BoardController::powerOnLED(LED_POWER, false);
	//unsigned char rx;
	int errorLoop = 0;

	// wait a maximum of 1 second until
	//while (!(UCSRA & (1<<RXC)));
	
	while (!(UCSRA & (1<<RXC)) && errorLoop <1000) {
		errorLoop++;
		//BoardController::msDelay(1);
	}
	

	if (errorLoop == 1000) {
		//BoardController::blinkLED((1<<LED_WALK), 10, 30);
		*data = 0xff;
		return false;
	} else {
		*data = UDR;
		return true;
	}
}

// returns 1 if servo with ID=_id responds to the ping commando
bool BioloidServo::pingServo(unsigned char _id) {

	BioloidPacket txPacket;
	BioloidPacket rxPacket;

	unsigned char bytesReceived = 0x00;

	txPacket.id = _id;
	txPacket.paramLen = 0; 
	txPacket.instruction = 0x01;

	TxPacket(&txPacket, 0);
	TxPacket(&txPacket, 0);

	if (bytesReceived = RxPacket(&rxPacket)) {
		return bytesReceived;
	} else {
		BoardController::blinkLED((1<<LED_WALK)|(1<<LED_EXPLORATION), 10, 30);
		return 0x00;
	}
}

/*
void BioloidServo::setBroadcastReturnDelayTime() {

	BioloidPacket txPacket;
	BioloidPacket rxPacket;

	unsigned char bytesReceived = 0x00;

	txPacket.id = _id;
	txPacket.paramLen = 0; 
	txPacket.instruction = 0x03; // Write

	TxPacket(&txPacket, 0);
}
*/

/* sends a ping packet to all id's on the bioloid bus */
void BioloidServo::scanBioloidBus() {

	unsigned char id;
	unsigned char bytesReceived  = 0x00;


	// set delay time 
	BioloidPacket txPacket;


	for (id=2; id<4; id++) {

		bytesReceived = 0x00;

		// set return-delay-time to 508 microseconds
		BoardController::enableSerialBioloidInterface();
		txPacket.id = id;
		txPacket.paramLen = 1; 
		txPacket.instruction = 0x05;
		txPacket.param[0] = 254;
		TxPacket(&txPacket, 0);

		bytesReceived = BioloidServo::pingServo(id);
		BoardController::disableSerialInterfaces();

		/* wait till transmit buffer is ready */
		while (!(UCSRA & (1<<UDRE)));
		UDR = bytesReceived+0x30;
		while (!(UCSRA & (1<<TXC)));
	}
}


