//
// C++ Interface: Environment
//
// Description: Environment description
//
//
// Author: Michel Tokic <michel@tokic.com>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef _ENVIRONMENT_H_
#define _ENVIRONMENT_H_

#include "FeedbackSensor.h"
#include "BioloidServo.h"

/**
 * defines the return for performing an action in the environment (s', reward)
 */
typedef struct actionReturn {
	ENV_POS state;
	int reward;
} ACTION_RETURN;


/**
 * defines the environment 
 */
class Environment {

private:

	FeedbackSensor* r;

	BioloidServo* servo_x;
	BioloidServo* servo_y;

	ENV_POS currentPos;
	ENV_POS discretization;

public:
	Environment(ENV_POS* discretization);
	~Environment();
	void doAction(unsigned int action, unsigned int delayTime, ACTION_RETURN* actionReturn);
	void beamRobotToState(ENV_POS* pos);
	void setDiscretization(ENV_POS* discretization);
	void getDiscretization(ENV_POS* discretization);
	void getCurrentState(ENV_POS* pos);
	bool isValidAction(ENV_POS* pos, unsigned char action);
};


#endif
