#include "V24Control.h"

/** 
 * global objects which are used by the interrupt function SIGNAL(SIG_UART_RECV) 
 */
//extern V24Control v24;
//V24COMMAND tx_command, rx_command;
//volatile bool newDataAvailable = false;

/**
 * the constructor. set's the ValueIteration object
 */
V24Control::V24Control() {


//#ifdef TESTMODE
//#warning VI disabled
//#else	
	// create new valueIteration object with a 5x5 StateSpace
	this->vi = new ValueIteration(new ENV_POS(5,5));
//#endif
}


/** destructor */
V24Control::~V24Control() {
	delete this->vi;
}

/**
 * returns true if new data arrived in USART buffer, otherwise zero
 */
bool V24Control::newDataReceived() {

	//if (BoardController::getSwitchState(SWITCH_V24)) {

		// enable interrupt
		//BoardController::enableV24Interrupt();
		
// 		if (newDataAvailable == true || (UCSRA & (1<<RXC))) {
// 			newDataAvailable = false;
// 			return true;
// 		} else {
// 			return false;
// 		}
		
		return (UCSRA & (1<<RXC));
	//}

	//return false;
}

/**
 * initializes the serial interface according to baud-rate MYUBRR, 1 stop bit and no parity bit
 */
void V24Control::init () {

	// should each received byte be send back to the sender? (for debug)
	rxtx = false;

	// enable serial interface
	BoardController::enableSerialV24Interface();
}


/**
 * transmits a byte on the serial interface (low level)
 */
void V24Control::transmit (unsigned char data) {

	//if (BoardController::getSwitchState(SWITCH_V24)) {

		//BoardController::enableSerialV24Interface();

		/* wait till transmit buffer is ready */
		while (!(UCSRA & (1<<UDRE)));

		/* wait until busy-pin (from RF04) is ready */
		while (PINC & (1<<CTS)); // PC2
	
		/* send data */
		UDR = data;
	
		// wait until transmission is complete
		while (!(UCSRA & (1<<TXC)));
	//}
}


/**
 * function receives one byte from the serial interface and writes it to *data (low-level)
 */
bool V24Control::receive (unsigned char *data) {

	if (BoardController::getSwitchState(SWITCH_V24)) {

		// CTS setzen, damit GUI schicken kann
		//PINC |= (1<<CTS); // PC2
		BoardController::powerOnLED(LED_POWER, false);
		//unsigned char rx;
		int errorLoop = 0;
	
		
		// wait a maximum of 1 second until
		while (!(UCSRA & (1<<RXC)) && errorLoop <100) {
			errorLoop++;
			BoardController::msDelay(10);
		}

		if (errorLoop == 100)
			return false;
		
		
	
		// error check
		/*if (errorLoop>=100) {
			BoardController::powerOnLED(LED_POWER, true);
			//BoardController::newMainMode = MAIN_MODE_STOP;
			return false;
		} else {
		*/
			// return received data
			*data = UDR;
			BoardController::powerOnLED(LED_POWER, true);
		/*
			return true;
		}
		*/
	}

	return true;
}

/**
 * transmits a command on the serial interface
 */
bool V24Control::transmitCommand (V24COMMAND *c) {

	unsigned char loopCount;

	if (BoardController::getSwitchState(SWITCH_V24)) {

	
		unsigned char tmp;

// 		// transmit command-byte
// 		transmit(V24_COMMAND_SYNC_COMMAND);
// 		if (rxtx) {
// 			receive(&tmp);
// 			if (tmp != V24_COMMAND_SYNC_OK_COMMAND) return false;
// 		}
// 
// 	
		// transmit command-byte
		transmit(c->command);
		/*if (rxtx) {
			receive(&tmp);
			if (tmp != c->command) return false;
		}*/

		// transmit command-length	
		transmit(c->dataLength);
		/*if (rxtx) {
			receive(&tmp);
			if (tmp != c->dataLength) return false;
		}*/
	
		// transmit data
		for (loopCount=0; loopCount < c->dataLength; loopCount++) {
	
			transmit(c->data[loopCount]);
			/*if (rxtx) {
				receive(&tmp);
				if (tmp != c->data[loopCount]) return false;
			}*/
		}
	
		return true;
	}
}

/**
 * receive a commando from the serial interface into *c
 */
bool V24Control::receiveCommand (V24COMMAND *c) {

	unsigned char loopCount, rData, data;
	bool sync = false;
	char i;

	if (BoardController::getSwitchState(SWITCH_V24)) {

// 		// synchronisation
// 		rData = !V24_COMMAND_SYNC_COMMAND;
// 		do {
// 			if (receive(&rData) && rData==V24_COMMAND_SYNC_COMMAND) {
// 				transmit(V24_COMMAND_SYNC_OK_COMMAND);
// 				sync = true;
// 			}
// 		} while (!sync);

		// receive command byte
		sync = false;
		do {
			if (receive(&c->command)) {
				//transmit(c->command);
				sync = true;
			}
		} while (!sync);
		
		// receive command-length
		sync = false;
		do {
			if (receive(&c->dataLength)) {
				//transmit(c->dataLength);
				sync = true;
			}
		} while (!sync);

		
		// receive command data
		for (loopCount=0; loopCount < c->dataLength; loopCount++) {
			sync = false;
			do {
				if (receive(&c->data[loopCount])) {
					//transmit(c->data[loopCount]);
					sync = true;
				}
			} while (!sync);		
		}
	
		return true;
	}
}


/**
 *  process command 
 */
void V24Control::processCommand() {

	if (BoardController::getSwitchState(SWITCH_V24)) {
	
		// disable all interrupts
		cli();
		BoardController::disableV24Interrupt();
	
		char i=0;
		char cmd;
		unsigned char* bytes;
		int _row, _column;
		V24COMMAND tmp_command;
			
		// turn off LEDs
		BoardController::powerOnLED (LED_LEARN, false);
		BoardController::powerOnLED (LED_WALK, false);
		BoardController::powerOnLED (LED_EXPLORATION, false);
	
		// disable auto start
		//BoardController::autoStart = false;
	
		// receive command
		//if (!commandParsedByIRQ) {
			if (!this->receiveCommand ( &rx_command )) {
				this->newMainMode = MAIN_MODE_STOP;
				BoardController::msDelay(1000);
				this->flushReceiveBuffer();
				BoardController::enableV24Interrupt();
				return;
			}
		//} else {
		//	commandParsedByIRQ = false;
		//}
	
		cmd = rx_command.command;
	
		/***************
		* walking mode *
		****************/
		if ( cmd == V24_COMMAND_START_WALKING ) {
	
			// go to start position (0,0)
			ENV_POS newPos(0, 0);
			vi->env->beamRobotToState(&newPos);
			this->newMainMode = MAIN_MODE_WALK_POLICY;
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;
		
		/*************
		* stop robot *
		**************/
		} else if ( cmd == V24_COMMAND_STOP_WALKING ) {
	
			this->newMainMode = MAIN_MODE_STOP;
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;
		
		/********************
		* send gamma to GUI *
		*********************/
		} else if ( cmd == V24_COMMAND_GET_GAMMA ) {
	
			processGetGamma();
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;
	
		/************
		* set gamma *
		*************/
		} else if ( cmd == V24_COMMAND_SET_GAMMA ) {
	
			processSetGamma(&rx_command);
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;
	
		/**********************
		* set grid dimensions *
		***********************/
		} else if ( cmd == V24_COMMAND_SET_GRIDSIZE ) {
	
			processSetGridsize(&rx_command);
			
		/*******************************
		* send current grid dimensions *
		********************************/
		} else if ( cmd == V24_COMMAND_GET_GRIDSIZE ) {
	
			processGetGridsize();
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;
	
		/***************************************
		* set a state (Value+rewards)          *
		****************************************/
		} else if ( cmd == V24_COMMAND_SET_STATE ) {
	
			processSetState(&rx_command);
	
		/**************************
		* send a state to the GUI *
		***************************/
		} else if ( cmd == V24_COMMAND_GET_STATE ) {
		
			processGetState(&rx_command);
	
		/*****************************************************
		* returns the exploration/exploitation configuration *
		******************************************************/
		} else if ( cmd == V24_COMMAND_GET_EXPLORATION_CONFIG ) {
		
			processGetExplorationConfig();
	
		/**********************************************
		* returns user defined debug information *
		***********************************************/
		} else if ( cmd == V24_COMMAND_GET_DEBUG ) {
		
			processGetDebug();

		/*************************
		* RAM-Test - write bytes *
		**************************/
		} else if ( cmd == V24_COMMAND_WRITE_RAM_BYTES ) {
	
			this->newMainMode = MAIN_MODE_STOP;
	
			// write random numbers into memory
			vi->states->ramtestSaveBytes(  
				((rx_command.data[0]*256) + rx_command.data[1]), // Address
				&(rx_command.data[2]),  // Pointer to data
				rx_command.dataLength-2 // size
	
			);
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;
	
		/************************
		* RAM-Test - read bytes *
		*************************/
		} else if ( cmd == V24_COMMAND_READ_RAM_BYTES ) {
	
			// stop main program
			this->newMainMode = MAIN_MODE_STOP;
	
			// create commando structure
			tmp_command.command = V24_COMMAND_WRITE_RAM_BYTES;
			tmp_command.dataLength = rx_command.data[2]+2; //
			tmp_command.data[0] = rx_command.data[0];
			tmp_command.data[1] = rx_command.data[1];
	
			// read data from RAM
			vi->states->ramtestReadBytes(  
				((rx_command.data[0]*256) + rx_command.data[1]), // address
				&(tmp_command.data[2]),	 // data pointer
				rx_command.data[2]	 // size
			);
	
			// send back to GUI
			if (!this->transmitCommand (&tmp_command)) return;
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;
	
		/***********************************
		* explore all rewards		   *
		************************************/
		} else if ( cmd == V24_COMMAND_EXPLORE_FEEDBACK ) {
	
			this->newMainMode = MAIN_MODE_EXPLORE_FEEDBACK;
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;
	
		/***********************************
		* learning only mode 		   *
		************************************/
		} else if ( cmd == V24_COMMAND_START_LEARNING ) {
	
			this->newMainMode = MAIN_MODE_LEARN;
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;

		/**********************************
		* returns the distance traveled   *
		***********************************/
		} else if (cmd == V24_COMMAND_GET_DISTANCE ) {

			signed long int _distance = vi->getDistanceCovered();
			unsigned char* bytes = (unsigned char*)&_distance;

			tx_command.command = V24_COMMAND_SET_DISTANCE;
			tx_command.dataLength = 0x04;
			tx_command.data[0] = bytes[0];
			tx_command.data[1] = bytes[1];
			tx_command.data[2] = bytes[2];
			tx_command.data[3] = bytes[3];

			if (!this->transmitCommand (&tx_command)) return; 
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;

		/**************************
 		 * reset distance counter *
 		 **************************/
		} else if (cmd == V24_COMMAND_RESET_DISTANCE_COUNTER ) {

			vi->resetDistanceCounter();
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;

		/***************************
		 * set exploration config  *
		 ***************************/
		} else if (cmd == V24_COMMAND_SET_EXPLORATION_CONFIG ) {

			unsigned char* bytes;
			register char i;
		
			// exploration parameter
			bytes = (unsigned char*)&vi->expParameter;
			for (i=0; i < sizeof(vi->expParameter); i++) {
				bytes[i] = rx_command.data[i];
			}

			// exploration mode			
			bytes = (unsigned char*)&vi->expMode;
			bytes[0] = rx_command.data[sizeof(vi->expParameter)];

			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;

		/***********************************
		* send current position, main-mode and iteration-count to GUI    *
		***********************************/
		} else if (cmd == V24_COMMAND_GET_CURRENT_STATE_MODE_ITERATIONS ) {
	
			unsigned long int _itCount = vi->getIterationCount();
			unsigned char* bytes = (unsigned char*)&_itCount;
			signed char wert;
			
			tx_command.command = V24_COMMAND_SET_CURRENT_STATE_MODE_ITERATIONS;
			tx_command.dataLength = 0x09;
			
			ENV_POS pos;
			vi->env->getCurrentState(&pos);
			tx_command.data[0] = pos.y;
			tx_command.data[1] = pos.x;
			tx_command.data[2] = (unsigned char)this->mainMode;
			tx_command.data[3] = bytes[0];
			tx_command.data[4] = bytes[1];
			tx_command.data[5] = bytes[2];
			tx_command.data[6] = bytes[3];
			wert = 127;
			tx_command.data[7] = *(unsigned char*)&wert;
			wert = -127;
			tx_command.data[8] = *(unsigned char*)&wert;
			
	
			if (!this->transmitCommand (&tx_command)) return; 
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;

		/***********************************
		* send current position, main-mode and iteration-count to GUI    *
		***********************************/
		} else if (cmd == V24_COMMAND_GET_CURRENT_STATE ) {
				
			tx_command.command = V24_COMMAND_SET_CURRENT_STATE;
			tx_command.dataLength = 0x02;
			
			ENV_POS pos;
			vi->env->getCurrentState(&pos);
			tx_command.data[0] = pos.x;
			tx_command.data[1] = pos.y;
			
			if (!this->transmitCommand (&tx_command)) return; 
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;

	
// 		/*********************************************************
// 		* receive states (values+rewards) and gamma from the GUI *
// 		**********************************************************/
// 		} else if (cmd == V24_COMMAND_SET_ALL_STATES_AND_GAMMA ) {
// 		
// 			// set gamma
// 			if (!this->receiveCommand(&tmp_command)) return;
// 			processSetGamma(&tmp_command);
// 			if (!v24.transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;
// 	
// 			// set grid size
// 			if (!this->receiveCommand(&tmp_command)) return;
// 			processSetGridsize(&tmp_command);
// 	
// 			for (_row=0; _row < BoardController::states.scalY; _row++) {
// 				for (_column=0; _column < BoardController::states.scalX; _column++) {
// 	
// 					// receive state
// 					if (!this->receiveCommand (&tmp_command)) return;
// 					processSetState(&tmp_command);
// 				}
// 			}		
// 		
// 		/********************************************************
// 		* send all states (Values+Rewards) and gamma to the GUI *
// 		*********************************************************/
// 		} else if (cmd == V24_COMMAND_GET_ALL_STATES_AND_GAMMA ) {
// 	
// 			// send gamma
// 			if (!this->receiveCommand(&tmp_command)) return;
// 			processGetGamma();
// 			if (!v24.transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;
// 	
// 			// send grid size
// 			if (!this->receiveCommand(&tmp_command)) return;
// 			processGetGridsize();
// 			if (!v24.transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;
// 	
// 			for (_row=0; _row < BoardController::states.scalY; _row++) {
// 				for (_column=0; _column < BoardController::states.scalX; _column++) {
// 	
// 					// receive getState Command
// 					if (!this->receiveCommand (&tmp_command)) return;
// 
// 					// send state
// 					processGetState(&tmp_command);
// 				}
// 			}		

		// perform next action and return reward to the GUI
		} else if (cmd == V24_COMMAND_WALK_AND_RETURN ) {

			ENV_POS tmpPos;

			tx_command.command = V24_COMMAND_WALK_AND_RETURN_RESPONSE;
			tx_command.dataLength = 3;

			unsigned char action = rx_command.data[0];
			ACTION_RETURN aRet;
 			vi->env->doAction(action, 100, &aRet);

			/* char reward = vi->moveAndSaveFeedback(
					action, 0, &tmpPos);
			*/

			tx_command.data[0] = aRet.state.x;
			tx_command.data[1] = aRet.state.y;
			tx_command.data[2] = aRet.reward;
			//tx_command.data[3] = rx_command.command;
			//tx_command.data[4] = rx_command.dataLength;
			//tx_command.data[5] = action;


			if (!this->transmitCommand (&tx_command)) return; 
			//if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return;

			this->setMainMode(MAIN_MODE_STOP);

		/*******************************************************************************
		* return "UNKNOWN COMMAND" in case of an unknown command
		*******************************************************************************/
		} else {
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_ERROR, V24_ERRORCODE_UNKNOWN_COMMAND)) return;
		}
	
	
		// enable serial interrupt
		BoardController::enableV24Interrupt();
	}
}

/**
 * debug: ..
 */
bool V24Control::transmitReceiveCommandProcessed(unsigned char _code, unsigned char _error) {

	if (BoardController::getSwitchState(SWITCH_V24)) {
	
// 		V24COMMAND ok_command;
// 		ok_command.command = _code;
// 	
// 		// Im Fehlerfall, den Fehlercode anhängen
// 		if (_error > 0) {
// 			ok_command.dataLength = 0x01;
// 			ok_command.data[0] = _error;
// 	
// 		// Ansonsten gibt es keinen Datenteil
// 		} else {
// 			ok_command.dataLength = 0x00;
// 		}
// 	
// 		// Bestätigung senden
// 		if (!v24.transmitCommand(&ok_command)) return false;
// 	
	// 	return true;
	
	}
	return true;
}

/**
 * reset the serial buffer 
 */
void V24Control::flushReceiveBuffer() {
	if (BoardController::getSwitchState(SWITCH_V24)) {
		unsigned char dummy;
		while (UCSRA & (1<<RXC)) dummy = UDR;
	}
}


/** 
 * returns exploration parameter
 */
bool V24Control::processGetExplorationConfig() {

	if (BoardController::getSwitchState(SWITCH_V24)) {
	
		V24COMMAND tx_command;
		unsigned char* bytes;
		register char i;
	
		tx_command.command = V24_COMMAND_SET_EXPLORATION_CONFIG;
		tx_command.dataLength = 5;

		// Exploration Parameter
		bytes = (unsigned char*)&(vi->expParameter);
		for (i=0; i < sizeof(vi->expParameter); i++) {
			tx_command.data[i] = bytes[i];
		}

		// Exploration Mode
		tx_command.data[4] = vi->expMode;
		
	
		if (!transmitCommand (&tx_command)) return false;
		return true;
	}
}


/** 
 * send gamma to GUI
 */
bool V24Control::processGetGamma() {

	if (BoardController::getSwitchState(SWITCH_V24)) {
	
		V24COMMAND tx_command;
		unsigned char* bytes;
		register char i;
	
		tx_command.command = V24_COMMAND_SET_GAMMA;
		tx_command.dataLength = 4;
		bytes = (unsigned char*)&(vi->gamma);
	
		for (i=0; i < tx_command.dataLength; i++) {
			tx_command.data[i] = bytes[i];
		}
	
		if (!transmitCommand (&tx_command)) return false;
		return true;
	}
}

/**
 * set gamma in Value-Iteration
 */
void V24Control::processSetGamma(V24COMMAND *cmd) {

	if (BoardController::getSwitchState(SWITCH_V24)) {
	
		unsigned char* bytes;
		register char i;
	
		bytes = (unsigned char*)&vi->gamma;
	
		for (i=0; i < 4; i++) {
			bytes[i] = cmd->data[i];
		}
	}
}

/** 
 * returns user defined debug information
 */
bool V24Control::processGetDebug() {

	if (BoardController::getSwitchState(SWITCH_V24)) {
	
		V24COMMAND tx_command;
	
		tx_command.command = V24_COMMAND_GET_DEBUG;
		tx_command.dataLength = 10;

		register char i;

		unsigned char* bytes = (unsigned char*)&(vi->currentExpProbability);
		for (i=0; i < tx_command.dataLength; i++) {
			tx_command.data[i] = bytes[i];
		}

		for (i=4; i<tx_command.dataLength; i++) {
			tx_command.data[i] = rand()%(NUM_ACTIONS);
			srand (rand()+1);
		}
	
		if (!transmitCommand (&tx_command)) return false;
		return true;
	}
}


/** 
 * send grid-size to GUI
 */
bool V24Control::processGetGridsize() {

	if (BoardController::getSwitchState(SWITCH_V24)) {
	
		V24COMMAND tx_command;
	
		tx_command.command = V24_COMMAND_SET_GRIDSIZE;
		tx_command.dataLength = 2;

		ENV_POS pos;
		vi->env->getCurrentState(&pos);

		tx_command.data[0] = pos.y;
		tx_command.data[1] = pos.x;
	
		if (!transmitCommand (&tx_command)) return false;
		return true;
	}
}

/** 
 * set local grid-size
 */
bool V24Control::processSetGridsize(V24COMMAND *cmd) { 

	if (BoardController::getSwitchState(SWITCH_V24)) {
	
		unsigned char _scalX, _scalY;
	
		_scalY = cmd->data[0];
		_scalX = cmd->data[1];
	
		// check for maximum statespace size
		if ((_scalX*_scalY) > (MAX_STATES)) {
			if (!transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_ERROR, V24_ERRORCODE_GRIDSIZE_TOO_LARGE)) return false;
			return false;

		} else {
			//ENV_POS newDiscretization(_scalX, _scalY);
			//vi->env->setDiscretization(&newDiscretization);

			// create new VI object with statespace (scalx, scaly)
			delete this->vi;
			this->vi = new ValueIteration(new ENV_POS(_scalX, _scalY));

			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return false;
			return true;
		}
	}
}

/**
 * send a specific state to the GUI
 */
bool V24Control::processGetState(V24COMMAND *cmd) {

	if (BoardController::getSwitchState(SWITCH_V24)) {
	
		ENV_POS pos, d;
		vi->env->getDiscretization(&d);

		V24COMMAND tx_command;
		pos.x = cmd->data[1];
		pos.y = cmd->data[0];

		
		if (pos.y >= d.y || pos.x >= d.x) {
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_ERROR, V24_ERRORCODE_GRIDSIZE_TOO_LARGE)) return false;
			return false;
		} else {
	
			// get state
			vi->states->getState(&pos, &tmpState);
			unsigned char* bytes = (unsigned char*)&tmpState;
		
			tx_command.command = V24_COMMAND_SET_STATE;
			tx_command.dataLength = STATESIZE+2;
			tx_command.data[0] = pos.y;
			tx_command.data[1] = pos.x;
		
			for (unsigned char i=0; i < STATESIZE; i++) {
				tx_command.data[2 + i] = bytes[i];
			}
	
			if (!this->transmitCommand (&tx_command)) return false;
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return false;
			return true;
		}
	}
}

/** 
 * set received state (value+rewards) 
 */
bool V24Control::processSetState(V24COMMAND *cmd) {

	if (BoardController::getSwitchState(SWITCH_V24)) {

		ENV_POS pos, d;
		pos.x = cmd->data[1];
		pos.y = cmd->data[0];
		vi->env->getDiscretization(&d);
			
		unsigned char* bytes = (unsigned char*)&tmpState;
	
		// check, if state exists in grid
		if (pos.y>= d.y || pos.x >= d.x) {
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_ERROR, V24_ERRORCODE_GRIDSIZE_TOO_LARGE)) return false;
			return false;
		} else {
		
			for (unsigned char i=0; i<STATESIZE; i++) {
				bytes[i] = cmd->data[i + 2];
			}
	
			// save state to memory
			vi->states->setState (&pos, &tmpState);
			if (!this->transmitReceiveCommandProcessed (V24_COMMAND_COMMAND_OK, 0)) return false;
			return true;
		}
	}
}

/**
 * returns a new main mode (if available) and reset's the internal newMainMode variable. If no newMainMode is set, the oldMainMode is returned
 */
int V24Control::getNewMainMode(int oldMainMode) {
	if (this->newMainMode != MAIN_MODE_UNCHANGED) {
		int retMode = this->newMainMode;
		this->newMainMode = MAIN_MODE_UNCHANGED;
		return retMode;
	} else {
		return oldMainMode;
	}
}

/** 
 * set's the mainMode which can be requested by the PC
 */
void V24Control::setMainMode (int mainMode) {
	this->mainMode = mainMode;
}


/** Interrupt-Service-Routine (ISR) for command reception. */
//SIGNAL (SIG_UART_RECV) {

// ISR (USART_RXC_vect) {
//  	newDataAvailable = true;
// }
//void SIG_UART_RECV (void) __attribute__ ((signal));
//void SIG_UART_RECV (void) {

// 	if (BoardController::getSwitchState(SWITCH_V24)) {
// 	
// 		// disable interrups
// 		cli();
// 		BoardController::disableV24Interrupt();
// 	
// 		// signal that a new command arrived
// 		newDataAvailable = true;
// 		commandParsedByIRQ = true;	
// 	
// 		// switch off LEDs
// 		BoardController::powerOnLED (LED_LEARN, false);
// 		BoardController::powerOnLED (LED_WALK, false);
// 		BoardController::powerOnLED (LED_EXPLORATION, false);
// 	
// 		// receive command
// 		if(!v24.receiveCommand ( &rx_command )) return;
// 	
// 		// send immediate answer for some commands
// 		if (rx_command.command == V24_COMMAND_GET_CURRENTPOSITION_MODE_ITERATIONS ||
// 		rx_command.command == V24_COMMAND_GET_GRIDSIZE ||
// 		rx_command.command == V24_COMMAND_GET_GAMMA ) {
// 	
// 			// signal that command has already been processed
// 			newDataAvailable = false;
// 			
// 			// process command
// 			v24.processCommand();
// 	
// 			// enable interrupts
// 			BoardController::enableV24Interrupt();
// 			sei();
// 		}
// 	}
//}


