#ifndef __V24_H__
#define __V24_H__

#include "v24_commands.h"
#include "ValueIteration.h"

/********************************************************
 * uniform command structure
 *****************************************************/
typedef struct V24Command {
	unsigned char command;
	unsigned char dataLength;
	unsigned char data[18];
} V24COMMAND;

/**
 * Controll Class for Serial Interface
 * @author Michel
 */
class V24Control {

  public:
		V24Control();
		~V24Control();
		void init();
		void processCommand();
		bool newDataReceived();
		int getNewMainMode(int oldMainMode);

		// used by main
		ValueIteration* vi;

		// for testing in main-function
		void transmit(unsigned char data);
		bool receive(unsigned char*);

  private:
		void setMainMode (int mainMode);
		bool receiveCommand (V24COMMAND *c);
		

		int newMainMode;
		int mainMode;
		bool transmitCommand (V24COMMAND *c);

		bool processGetDebug();
		bool processGetGamma();
		bool processGetExplorationConfig();
		void processSetGamma(V24COMMAND *cmd);
		bool processGetGridsize();
		bool processSetGridsize(V24COMMAND *cmd);
		bool processGetState(V24COMMAND *cmd);
		bool processSetState(V24COMMAND *cmd);
		bool processSetMotorPosition(V24Command);
// 		bool processGetIterations();


		void flushReceiveBuffer();
		bool transmitReceiveCommandProcessed(unsigned char _code, unsigned char _error);

		//unsigned char loopCount;
		STATE tmpState;
		bool rxtx;
		bool newDataAvailable;

		V24COMMAND tx_command, rx_command;
};


#endif
