#include "gui.h"
#include "V24_strings.h"

/**
*
*/
void Simulation::v24_start_walking_button_clicked() {

	V24COMMAND c;

	this->lg->v24Data->setText("walking...");

	// Laufkommando schicken
	c.command = V24_COMMAND_START_WALKING;
	c.dataLength = 0;

	if (!v24->transmitCommand(&c)) {
		v24->closeDevice();
		this->lg->v24Data->setText("!!transmit error!!");
		return;
	}

	v24->receiveTransmitOKCommand();
	v24->closeDevice();	

	//}
}

/**
*
*/
void Simulation::v24_start_learning_button_clicked() {

	V24COMMAND c;

	this->lg->v24Data->setText("learning...");

	// Laufkommando schicken
	c.command = V24_COMMAND_START_LEARNING;
	c.dataLength = 0;

	if (!v24->transmitCommand(&c)) {
		v24->closeDevice();
		this->lg->v24Data->setText("!!transmit error!!");
		return;
	}

	v24->receiveTransmitOKCommand();
	v24->closeDevice();	

	//}
}

/**
*
*/
void Simulation::v24_exploration_mode_changed() {

	if (this->lg->v24ExplorationModeEGreedy->isChecked()) {
		this->lg->v24ExplorationLabel->setText("<b>exploration/exploitation parameter (epsilon):</b>");
	}
	else if (this->lg->v24ExplorationModeValueBased->isChecked()) {
		this->lg->v24ExplorationLabel->setText("<b>exploration/exploitation parameter (sigma):</b>");
	}
}

/**
*
*/
void Simulation::v24_debug_button_clicked() {

	V24COMMAND c, d;
	QString output;
	unsigned char randomNumbers[10];
	//unsigned char* bytes;

	this->lg->v24Data->setText("performing debug function...");

	c.command = V24_COMMAND_GET_DEBUG;
	c.dataLength = 0x00;

	if (!v24->transmitCommand(&c)) {
		v24->closeDevice();
		this->lg->v24Data->setText("!!transmit error!!");
		return;
	}

	if (!v24->receiveCommand(&d)) {
		DBG();
		v24->closeDevice();
		this->lg->v24Data->setText("!!receive error!!");
		return;
	}

	for (int i=4; i<d.dataLength; i++) {
		randomNumbers[i] = d.data[i];
	}

	float _expProbability;
	v24->parseFloatData(&d.data[0], (float*)&_expProbability);

	this->lg->v24Data->setText(	"eta=" + QString::number (_expProbability) + ", Random actions: " +
					QString::number (randomNumbers[4]) + ", " + 
					QString::number (randomNumbers[5]) + ", " + 
					QString::number (randomNumbers[6]) + ", " + 
					QString::number (randomNumbers[7]) + ", " + 
					QString::number (randomNumbers[8]) + ", " + 
					QString::number (randomNumbers[9]));

}

/**
*
*/
void Simulation::v24_get_exploration_clicked() {

	V24COMMAND c, d;
	QString output;
	float _expParameter;
	unsigned char _expMode;
	//unsigned char* bytes;

	this->lg->v24Data->setText("reading exploration config...");

	c.command = V24_COMMAND_GET_EXPLORATION_CONFIG;
	c.dataLength = 0x00;

	if (!v24->transmitCommand(&c)) {
		v24->closeDevice();
		this->lg->v24Data->setText("!!transmit error!!");
		return;
	}

	if (!v24->receiveCommand(&d)) {
		DBG();
		v24->closeDevice();
		this->lg->v24Data->setText("!!receive error!!");
		return;
	}
	
	// deserialize data
	v24->parseFloatData(&d.data[0], (float*)&_expParameter);
	_expMode = d.data[4];

	// set proper variables
	this->expMode = _expMode;
	if (this->expMode == 0) {
		this->lg->v24ExplorationModeEGreedy->setChecked(true);
		this->lg->v24ExplorationModeValueBased->setChecked(false);
	}
	if (this->expMode == 1) {
		this->lg->v24ExplorationModeEGreedy->setChecked(false);
		this->lg->v24ExplorationModeValueBased->setChecked(true);
	}
	this->v24_exploration_mode_changed();

	this->expParameter = _expParameter;
	this->lg->v24ExplorationParameterInput->setText(QString::number(this->expParameter));


	// debug output
	output =  QString("mode: " + QString::number(_expMode) + ", parameter: " + QString::number(_expParameter));
	this->lg->v24Data->setText(output);

	// send ok
	v24->receiveTransmitOKCommand();
	v24->closeDevice();
}


/**
*
*/
void Simulation::v24_exploration_accept_clicked() {

	V24COMMAND c;
	unsigned char* _bytes;

	this->lg->v24Data->setText("new expParameter set to " + this->lg->v24ExplorationParameterInput->text());

	// String nach Float konvertieren
	this->expParameter = this->lg->v24ExplorationParameterInput->text().toDouble();
	if (this->lg->v24ExplorationModeEGreedy->isOn()) {
		this->expMode = V24_EXP_MODE_EGREEDY;
	} else if (this->lg->v24ExplorationModeValueBased->isOn()) {
		this->expMode = V24_EXP_MODE_VALUE_BASED;
	}

	printf ("Exploration Mode %d, Parameter: %f\n", this->expMode, this->expParameter);
	

	/****************************
	 * expBeta schicken         *
	 ****************************/
	c.command = V24_COMMAND_SET_EXPLORATION_CONFIG;
	c.dataLength = sizeof(this->expParameter)+1;
	_bytes = (unsigned char*) &expParameter;
	for (int i=0; i < c.dataLength-1; i++) {
		c.data[i] = _bytes[i];
	}
	c.data[c.dataLength-1] = expMode;

	
	// Befehl schicken um expBeta zu setzen
	if (!v24->transmitCommand(&c)) {
		v24->closeDevice();
		this->lg->v24Data->setText("!!transmit error!!");
		return;
	}

	v24->receiveTransmitOKCommand();
	v24->closeDevice();
}

/**
*
*/
void Simulation::v24_accept_button_clicked() {
	
	if (v24->setInterface(this->lg->v24Interface->text())) {
		this->lg->v24StartWalkingButton->setEnabled(true);
		this->lg->v24StartLearningButton->setEnabled(true);
		this->lg->v24StopWalkingButton->setEnabled(true);
		this->lg->v24CopyStatesAndGammaToRobotButton->setEnabled(true);
		this->lg->v24GetStatesAndGammaFromRobotButton->setEnabled(true);
		this->lg->v24GetPositionAndModeButton->setEnabled(true);
		this->lg->v24RamtestButton->setEnabled(true);
		this->lg->v24ExploreFeedbackButton->setEnabled(true);
		this->lg->v24GetDistanceButton->setEnabled(true);
		this->lg->v24ExplorationAcceptButton->setEnabled(true);
		this->lg->v24GetExplorationButton->setEnabled(true);
		this->lg->v24ExplorationParameterInput->setEnabled(true);
		this->lg->v24ResetDistanceButton->setEnabled(true);
		this->lg->v24DebugButton->setEnabled(true);
	} else {
		this->lg->v24StartWalkingButton->setEnabled(false);
		this->lg->v24StartLearningButton->setEnabled(false);
		this->lg->v24StopWalkingButton->setEnabled(false);
		this->lg->v24CopyStatesAndGammaToRobotButton->setEnabled(false);
		this->lg->v24GetStatesAndGammaFromRobotButton->setEnabled(false);
		this->lg->v24GetPositionAndModeButton->setEnabled(false);
		this->lg->v24RamtestButton->setEnabled(false);
		this->lg->v24ExploreFeedbackButton->setEnabled(false);	
		this->lg->v24GetDistanceButton->setEnabled(false);
		this->lg->v24ExplorationAcceptButton->setEnabled(false);
		this->lg->v24GetExplorationButton->setEnabled(false);
		this->lg->v24ExplorationParameterInput->setEnabled(false);
		this->lg->v24ResetDistanceButton->setEnabled(false);
		this->lg->v24DebugButton->setEnabled(false);
	} 
}

/**
*
*/
void Simulation::v24_distance_button_clicked() {

	V24COMMAND c, d;
	QString output;
	signed int _distance=5;
	//unsigned char* bytes;

	this->lg->v24Data->setText("reading distance traveled...");

	c.command = V24_COMMAND_GET_DISTANCE;
	c.dataLength = 0x00;

	if (!v24->transmitCommand(&c)) {
		v24->closeDevice();
		this->lg->v24Data->setText("!!transmit error!!");
		return;
	}


	if (!v24->receiveCommand(&d)) {
		DBG();
		v24->closeDevice();
		this->lg->v24Data->setText("!!receive error!!");
		return;
	}
	
	// IterationCount auslesen
	v24->parseIntegerData(&d.data[0], (unsigned int*)&_distance);
	
	output =  QString("distance covered: " + QString::number(_distance));
	
	this->lg->v24Data->setText(output);

	// Okey abwarten, bis nächstes Kommando gesendet werden kann
	v24->receiveTransmitOKCommand();
	v24->closeDevice();
}

/**
*
*/
void Simulation::v24_reset_distance_button_clicked() {

	V24COMMAND c;

	this->lg->v24Data->setText("distance cleared...");

	// Laufkommando schicken
	c.command = V24_COMMAND_RESET_DISTANCE_COUNTER;
	c.dataLength = 0;

	if (!v24->transmitCommand(&c)) {
		v24->closeDevice();
		this->lg->v24Data->setText("!!transmit error!!");
		return;
	}

	v24->receiveTransmitOKCommand();
	v24->closeDevice();	

}

/**
*
*/
void Simulation::v24_stop_walking_button_clicked() {

	//for (int i=0; i<100; i++) {
	this->lg->v24Data->setText("stopped.");
	
	V24COMMAND c;

	// LED ausschalten
	//c.command = V24_COMMAND_LED_OFF;
	//c.dataLength = 0;

	// LED Kommando schicken
	//v24->transmitCommand(&c);
	//v24->receiveTransmitOKCommand();

	// Laufkommando schicken
	c.command = V24_COMMAND_STOP_WALKING;
	c.dataLength = 0;

	if (!v24->transmitCommand(&c)) {
		v24->closeDevice();
		this->lg->v24Data->setText("!!transmit error!!");
		return;
	}

	v24->receiveTransmitOKCommand();

	// Schnittstelle wieder schließen
	v24->closeDevice();
	//}
}

/**
*
*/
void Simulation::v24_copy_states_and_gamma_to_robot_button_clicked() {

	V24COMMAND c;
	unsigned char* _bytes;
	State _state;

	this->stopLearningThread();

	// Kommando für Bulk-Anfrage schicken
	c.command = V24_COMMAND_SET_ALL_STATES_AND_GAMMA;
	c.dataLength = 0;

	// Kommando über V24 senden
	if (!v24->transmitCommand(&c)) {
		v24->closeDevice();
		this->lg->v24Data->setText("!!transmit error!!");
		return;
	}

	/****************************
	 * Gamma schicken           *
	 ****************************/
	c.command = V24_COMMAND_SET_GAMMA;
	c.dataLength = sizeof(this->gamma);
	_bytes = (unsigned char*) &gamma;
	for (int i=0; i < c.dataLength; i++) {
		c.data[i] = _bytes[i];
	}
	
	// Befehl schicken um Gamma zu setzen
	if (!v24->transmitCommand(&c)) {
		v24->closeDevice();
		this->lg->v24Data->setText("!!transmit error!!");
		return;
	}
	if (!v24->receiveTransmitOKCommand()) {
		v24->closeDevice();
		this->lg->v24Data->setText("!!transmit error!!");
		return;
	}
	
	/*****************************
	 * Grid-Größe schicken       *
	 *****************************/
	c.command = V24_COMMAND_SET_GRIDSIZE;
	c.dataLength = 2;
	c.data[0] = (unsigned char)scalY; // anzahl der spalten
	c.data[1] = (unsigned char)scalX; // anzahl der zeilen

	// Befehl schicken um Grid-Größe zu setzen
	if (!v24->transmitCommand(&c)) {
		v24->closeDevice();
		this->lg->v24Data->setText("!!transmit error!!");
		return;
	}
	if (!v24->receiveTransmitOKCommand()) {
		v24->closeDevice();
		this->lg->v24Data->setText("!!transmit error!!");
		return;
	}
	

	/***************************************
	 * Values+Feedbacks-schicken schicken  *
	 ***************************************/
	c.command = V24_COMMAND_SET_STATE;
	c.dataLength = STATESIZE+2;

	//int _row=0, _column=0;
	for (int _row=0; _row < this->scalY; _row++) {
		for (int _column=0; _column < this->scalX; _column++) {

			// Position in Puffer schreiben
			c.data[0] = (unsigned char)_row;
			c.data[1] = (unsigned char)_column;		

			// Zustand mit Daten befüllen
			_state.value = (float)fValueMemory[_row][_column];
			//_state.best_action = (unsigned char)aktion[_row][_column];
			_state.feedback_up = (char)getReward(_row, _column, UP);
			_state.feedback_down = (char)getReward(_row, _column, DOWN);
			_state.feedback_left = (char)getReward(_row, _column, LEFT);
			_state.feedback_right = (char)getReward(_row, _column, RIGHT);

			// Zustand dem Puffer hinzufügen
			_bytes = (unsigned char*)&_state;
			for (int i=0; i < STATESIZE; i++) {
				c.data[2 + i] = _bytes[i];
			}

			// Zustand �ber V24 senden
			if (!v24->transmitCommand(&c)) {
				v24->closeDevice();
				this->lg->v24Data->setText("!!receive error!!");
				return;
			}

			// Okey abwarten, bis n�chstes Kommando gesendet werden kann
			if (!v24->receiveTransmitOKCommand()) {
				v24->closeDevice();
				this->lg->v24Data->setText("!!receive error!!");
				return;
			}
		}
	}

	// Okey abwarten, bis nächstes Kommando gesendet werden kann
	/*if (!v24->receiveTransmitOKCommand()) {
		v24->closeDevice();
		return;
	}*/
	
	v24->closeDevice();

	this->lg->v24Data->setText("Configuration successfully applied...");
}

/**
*
*/
void Simulation::v24_getPositionAndMode_button_clicked() {

	V24COMMAND c, d;
	QString output;
	unsigned int _itCount=5;
	//unsigned char* bytes;

	this->lg->v24Data->setText("reading position...");

	v24->openDevice();

	// Kommando schicken
	c.command = V24_COMMAND_GET_CURRENTPOSITION_AND_MODE;
	c.dataLength = 0x00;
	
	if (!v24->transmitCommand(&c)) {
		DBG();
		v24->closeDevice();
		this->lg->v24Data->setText("!!receive error!!");
		return;
	}

	if (!v24->receiveCommand(&d)) {
		DBG();
		v24->closeDevice();
		this->lg->v24Data->setText("!!receive error!!");
		return;
	}
	
	// IterationCount auslesen
	v24->parseIntegerData(&d.data[3], &_itCount);
	
	output =  QString("Y=" + QString::number (d.data[0]+1) + 
			 ", X=" + QString::number (d.data[1]+1) +
			 ", Mode: " + v24Modes[d.data[2]] + 
			 ", Iteration#: " + QString::number(_itCount));
	
	this->lg->v24Data->setText(output);

	// Okey abwarten, bis nächstes Kommando gesendet werden kann
	v24->receiveTransmitOKCommand();

	// Schnittstelle schließen
	v24->closeDevice();
}

/**
*
*/
void Simulation::v24_explore_feedback_button_clicked() {

	V24COMMAND c;

	v24->openDevice();

	this->lg->v24Data->setText("exploring environment...");

	// Kommando schicken
	c.command = V24_COMMAND_EXPLORE_FEEDBACK;
	c.dataLength = 0x00;
	if (!v24->transmitCommand(&c)) {
		v24->closeDevice();
		this->lg->v24Data->setText("!!transmit error!!");
		return;
	}

	// Okey abwarten, bis nächstes Kommando gesendet werden kann
	v24->receiveTransmitOKCommand();

	// Schnittstelle schließen
	v24->closeDevice();
}

/**
*
*/
void Simulation::v24_ramtest_button_clicked() {

	// RAM Test
	#define RAMSIZE ((9*200)/16)*16
	//#define RAMSIZE 16
	unsigned char ramArrayLocal[RAMSIZE], ramArrayRemote[RAMSIZE];

	// Zufallszahlengenerator initialisieren
	srand( (unsigned int) time(NULL));

	V24COMMAND c, d;

	this->lg->v24Data->setText("initializing...");
	// Arrays Initialisieren
	for (int i=0; i<RAMSIZE; i++) {
		ramArrayLocal[i] = (rand() %255)+1;
		ramArrayRemote[i] = 0x00;
	}
	
	// Array mit Zufallszahlen befüllen
	//V24_COMMAND_WRITE_RAM_BYTE
	//V24_COMMAND_READ_RAM_BYTE


	this->lg->v24Data->setText("writing random data to RAM...");
	c.command = V24_COMMAND_WRITE_RAM_BYTES;
	c.dataLength = 0x12;
	// ein paar Bytes schreiben ...
	for (int i=0; i<RAMSIZE; i+=16) {

		// Zelle beschreiben
		// => Adresse
		c.data[0] = i/256; // Highbyte
		c.data[1] = i%256; // Lowbyte

		for (int j=0; j<16; j++) {
			c.data[2+j] = ramArrayLocal[i+j];
		}

		// Kommando senden
		if (!v24->transmitCommand(&c)) {
			this->lg->v24Data->setText("!!transmit error!!");
			v24->closeDevice();
			return;
		}

		// OK empfangen
		if (!v24->receiveTransmitOKCommand()) {
			this->lg->v24Data->setText("!!receive error!!");
			v24->closeDevice();
			return;
		}

		printf ("written data=%d to addr=%d\n", ramArrayLocal[i], i);
	}
	
	// und überprüfen, ob alles richtig im RAM liegt
	this->lg->v24Data->setText("reading data from RAM...");
	c.command = V24_COMMAND_READ_RAM_BYTES;
	c.dataLength = 0x03;
	for (int i=0; i<RAMSIZE; i=i+16) {

		// => Adresse
		c.data[0] = i/256; // Highbyte
		c.data[1] = i%256; // Lowbyte
		c.data[2] = 16;	   // Anzahl der Datenbytes

		// Kommando senden
		if (!v24->transmitCommand(&c)) {
			this->lg->v24Data->setText("!!transmit error!!");
			v24->closeDevice();
			return;
		}

		// Antwort empfangen
		DBG();
		v24->receiveCommand(&d);
		DBG();
		for (int j=0; j<16; j++) {
			ramArrayRemote[i+j] = d.data[2+j];
		}

		// OK empfangen
		if (!v24->receiveTransmitOKCommand()) {
			this->lg->v24Data->setText("!!receive error!!");
			v24->closeDevice();
			return;
		}

		// Überprüfen, ob beide Werte übereinstimmen
		for (int j=0; j<16; j++) {
			if (ramArrayLocal[i+j] != ramArrayRemote[i+j]) {
				printf ("ERROR: ramArrayLocal[%d]=%d  !=  ramArrayRemote[%d]=%d\n", 
					i+j, ramArrayLocal[i+j], i+j, ramArrayRemote[i+j]);
				this->lg->v24Data->setText("ERROR: RAM test failed...");
				return;
			} else {
				printf ("OK: data=%d from addr=%d, matches local data\n", ramArrayLocal[i+j], i+j);
			}
		}
	}


	// Schnittstelle wieder schließen
	v24->closeDevice();
	this->lg->v24Data->setText("RAM test OK...");
}

/**
* Diese Funktion, liefert den gesamten Zustandsraum 	*
* und das Gamma vom Laufroboter.
*/
void Simulation::v24_get_states_and_gamma_from_robot_button_clicked() {

	V24COMMAND c, d;
	STATE _state;
	float _gamma;
	unsigned char* _bytes = (unsigned char*) &_state;
	int _scalX, _scalY;

	// neuen KI Thread erzeugen, falls noch keiner existiert
	if (this->pkt != NULL) {
		this->apply_button_clicked();		
	}

	this->stopLearningThread();

	/*******************************************************
	 * Kommando schicken, dass nun eine Bulk-Anfrage kommt *
	 *******************************************************/
	c.command = V24_COMMAND_GET_ALL_STATES_AND_GAMMA;
	c.dataLength = 0x00;

	// Befehl schicken
	if (!v24->transmitCommand(&c)) {
		this->lg->v24Data->setText("!!transmit error!!");
		DBG();
		v24->closeDevice();
		return;
	}

	/**************************************************
	 * Befehl schicken um Gamma auszulesen     *
	 **************************************************/
	c.command = V24_COMMAND_GET_GAMMA;
	c.dataLength = 0x00;

	// senden
	if (!v24->transmitCommand(&c)) {
		this->lg->v24Data->setText("!!transmit error!!");
		DBG();
		v24->closeDevice();
		return;
	}

	// Gamma empfangen
	if (!v24->receiveCommand(&d)) {
		this->lg->v24Data->setText("!!receive error!!");
		DBG();
		v24->closeDevice();
		return;
	}
	v24->parseFloatData(&d.data[0], &_gamma);
	
	// Gamma im Textfeld ausgeben
	this->lg->viGammaInput->setText(QString::number(_gamma));

	// noch kurz warten bis das OK kommt, dass der Befehl abgearbeitet wurde
	if (!v24->receiveTransmitOKCommand()) {
		this->lg->v24Data->setText("!!receive error!!");
		DBG();
		v24->closeDevice();
		return;
	}


	/**************************************************
	 * Aktuelle Grid Größe auslesen			  *
	 **************************************************/
	c.command = V24_COMMAND_GET_GRIDSIZE;
	c.dataLength = 0x00;

	// Befehl schicken auf Leitung
	if (!v24->transmitCommand(&c)) {
		this->lg->v24Data->setText("!!transmit error!!");
		DBG();
		v24->closeDevice();
		return;
	}

	// Grid-Größe empfangen
	if (!v24->receiveCommand(&d)) {
		this->lg->v24Data->setText("!!receive error!!");
		DBG();
		v24->closeDevice();
		return;
	}

	// OK-Kommando empfangen
	if (!v24->receiveTransmitOKCommand()) {
		this->lg->v24Data->setText("!!receive error!!");
		DBG();
		v24->closeDevice();
		return;
	}

	_scalY = scalY = d.data[0];
	_scalX = scalX = d.data[1];

	this->lg->numCols->setValue(_scalX);
	this->scalX = _scalX;
	this->lg->numRows->setValue(_scalY);
	this->scalY = _scalY;


	/*******************************************************
	 * GUI neu konfigurieren, mit den aktuellen Parametern *
	 *******************************************************/
	this->guiReady = false;
	this->apply_button_clicked();

	// KI-Thread anhalten
	this->pkt->setModusFromExtern('q');
	while (this->pkt->running()) {
		this->pkt->unlockMutex();
	}


	/**************************************************
	 * Zustände auslesen			          *
	 **************************************************/
	for (int _row=0; _row<scalY; _row++) {
		for (int _column=0; _column<scalX; _column++) {
		
			// Anforderung schicken
			c.command = V24_COMMAND_GET_STATE;
			c.dataLength = 0x02;
			c.data[0] = (unsigned char)_row;
			c.data[1] = (unsigned char)_column;
		
			// Befehl schicken
			if (!v24->transmitCommand(&c)) {
				this->lg->v24Data->setText("!!transmit error!!");
				DBG();
				v24->closeDevice();
				return;
			}
		
			// Zustand empfangen
			if (!v24->receiveCommand(&d)) {
				this->lg->v24Data->setText("!!receive error!!");
				DBG();
				v24->closeDevice();
				return;
			}
		
			// noch kurz warten bis das OK kommt, dass der Befehl abgearbeitet wurde
			if (!v24->receiveTransmitOKCommand()) {
				this->lg->v24Data->setText("!!receive error!!");
				v24->closeDevice();
				return;
			}

			for (uint i=0; i<STATESIZE; i++) {
				// Kopiere die empfangenen Daten nach (STATE)_state
				_bytes[i] = d.data[2+i]; 
			}
			
			// Value in GUI und VI-Thread speichern
			//while (this->pkt->running());
			this->updatePolicyTable ( _row, _column, (double)_state.value, NOTHING);
			this->pkt->setViValue( _row, _column, _state.value );

			// Feedback in GUI und VI-Thread speichern
			this->updateFeedbackTable(_row, _column, _state.feedback_up, 
							  	 _state.feedback_down, 
								 _state.feedback_left, 
								 _state.feedback_right );
			this->pkt->setFeedback(	_row, _column,	_state.feedback_up, 
								_state.feedback_down, 
								_state.feedback_left, 
								_state.feedback_right );

			// Value- und Policy-Table Zelle updaten
			lg->viValueTable->updateCell(_row, _column);
			lg->strategyTable->updateCell(_row, _column);
		}
	}

	// noch kurz warten bis das OK kommt, dass der Befehl abgearbeitet wurde
	/*if (!v24->receiveTransmitOKCommand()) {
		v24->closeDevice();
		return;
	}*/

	// Value- und Policy Tabelle resizen
	this->dad->lock();
	for (int i=0; i < this->scalX; i++) {
		lg->viValueTable->adjustColumn (i);
		lg->strategyTable->adjustColumn (i);
	}
	this->dad->unlock();

	// Text-Label aktualisieren
	this->lg->v24Data->setText("grid size: " + QString::number(scalY) + "x" + QString::number(scalX) + 
				   ", Gamma: " + QString::number(_gamma));

	// Schnittstelle öffnen
	v24->closeDevice();
}	
