#include "test.h"
#include "V24Control.h"

// the global object
V24Control v24;

/** the main function */
int main( void )
{	
	v24.init();

	// initialize auto start counter 
	unsigned char autoStartCounter = 0;

	v24.vi.ctr.newMainMode = MAIN_MODE_WALK_POLICY;

	while ( 1 ) {	

		// set new mode in case of interrupt
		v24.vi.ctr.mainMode = v24.vi.ctr.newMainMode;

		// only walk when PAUSE 
		if (v24.vi.ctr.getSwitchState(SWITCH_WALK)) {

			// Check for new serial data
			if (v24.newDataReceived()) {

				// stop robot
				v24.vi.ctr.disableV24Interrupt();
				cli();
				v24.vi.ctr.newMainMode = MAIN_MODE_STOP; 
	
				v24.processCommand();
				v24.vi.ctr.mainMode = v24.vi.ctr.newMainMode;
				v24.vi.ctr.autoStart = 0;
				
		
			// Autostart-Routine
			} else if (v24.vi.ctr.autoStart) {

				// delay for at least 5 seconds
				if (autoStartCounter < 5) {

					autoStartCounter++;
	
					v24.vi.ctr.blinkLED (
					  (1<<LED_WALK)|(1<<LED_LEARN)|(1<<LED_EXPLORATION), 1, 500);
			
				// start after 5 seconds
				} else {

					// disable auto-start
					v24.vi.ctr.autoStart = false;
					
					// explore environment if SWITCH_FULL_EXP_AT_START is enabled
					if (v24.vi.ctr.getSwitchState(SWITCH_FULL_EXP_AT_START)) {
						v24.vi.exploreAllFeedbacks(false);
					}
					v24.vi.ctr.enableV24Interrupt();
							
					v24.vi.ctr.newMainMode = MAIN_MODE_WALK_POLICY;
				}

			// do next action
			} else if (v24.vi.ctr.mainMode == MAIN_MODE_WALK_POLICY) {

				// execute next action - with respect on SWTICH_ENABLE_EXPLORATION switch
				v24.vi.moveAndSaveFeedback( 
					// choose best action
					v24.vi.getGreedyAction(
						v24.vi.ctr.currentY, v24.vi.ctr.currentX
					),  
					// pure greedy or exploitation/exploration
					v24.vi.ctr.getSwitchState(SWITCH_ENABLE_EXPLORATION)
				);

				// save old value(currentState) (for VDBE)
				v24.vi.oldValue = v24.vi.ctr.states.getValue(
					v24.vi.ctr.currentY, v24.vi.ctr.currentX
				); 
				
				// value iterate all states
				v24.vi.valueIterateAllStates();
			
				// save new value(currentState) (for VDBE)
				v24.vi.newValue = v24.vi.ctr.states.getValue(
					v24.vi.ctr.currentY, v24.vi.ctr.currentX
				);

				v24.vi.ctr.newMainMode = MAIN_MODE_WALK_POLICY;

			// explore environment
			} else if (v24.vi.ctr.mainMode == MAIN_MODE_EXPLORE_FEEDBACK) {
	
				v24.vi.exploreAllFeedbacks(true);
	
				// set learning mode
				v24.vi.ctr.newMainMode = MAIN_MODE_LEARN;
	
			// learning-only mode
			} else if (v24.vi.ctr.mainMode == MAIN_MODE_LEARN) {

				// value iterate all states
				v24.vi.valueIterateAllStates();	
				
			// pause
			} else if (v24.vi.ctr.mainMode == MAIN_MODE_STOP) {

				//vi.valueIterateAllStates();	
			}

		// if robot is in PAUSE-mode, only check for new serial data
		} else {

			// check for new serial data
			if (v24.newDataReceived()) {

				// stop robot
				v24.vi.ctr.newMainMode = MAIN_MODE_STOP; 
	
				// process serial command
				v24.processCommand();
	
				// update main mode
				v24.vi.ctr.mainMode = v24.vi.ctr.newMainMode;

			// otherwise perform value-iteration
			} else if (v24.vi.ctr.mainMode == MAIN_MODE_LEARN) {
				v24.vi.valueIterateAllStates();	
			}
		}
	}
}


